@@ -5,7 +5,7 @@
  *    Michael Meeks (mmeeks@gnu.org)
  *    Jon K Hellan  (hellan@acm.org)
  *
- * (C) 1998, 1999 Michael Meeks, Jon K Hellan
+ * (C) 1998-2000 Michael Meeks, Jon K Hellan
  **/
 
 /**
@@ -28,8 +28,6 @@
 
 #include "gnumeric.h"
 #include "gnumeric-util.h"
-#include "gnome-xml/tree.h"
-#include "gnome-xml/parser.h"
 #include "gnumeric-sheet.h"
 #include "format.h"
 #include "color.h"
@@ -1387,10 +1385,10 @@ put_mstyle (ExcelWorkbook *wb, MStyle *st)
 /**
  * Get ExcelCell record for cell position.
  **/
-static ExcelCell *
+inline static ExcelCell *
 excel_cell_get (ExcelSheet *sheet, int col, int row)
 {
-	return sheet->cells + row * sheet->maxx + col;
+	return *(sheet->cells + row) + col;
 }
 
 /**
@@ -2703,7 +2701,7 @@ write_colinfos (BiffPut *bp, ExcelSheet *sheet)
 			col.first = i;
 		}
 	}
-	col.last = sheet->maxx;
+	col.last = sheet->maxx - 1;
 	write_colinfo (bp, &col);
 }
 
@@ -2843,7 +2841,7 @@ write_sheet_bools (BiffPut *bp, ExcelSheet *sheet)
 	} else {
 		data = ms_biff_put_len_next (bp, (0x200 | BIFF_DIMENSIONS), 10);
 		MS_OLE_SET_GUINT16 (data +  0, 0);
-		MS_OLE_SET_GUINT16 (data +  2, sheet->maxy-1);
+		MS_OLE_SET_GUINT16 (data +  2, sheet->maxy);
 		MS_OLE_SET_GUINT16 (data +  4, 0);
 		MS_OLE_SET_GUINT16 (data +  6, sheet->maxx);
 		MS_OLE_SET_GUINT16 (data +  8, 0x0000);
@@ -3177,8 +3175,9 @@ write_sheet (BiffPut *bp, ExcelSheet *sheet)
 static void
 new_sheet (ExcelWorkbook *wb, Sheet *value)
 {
-	ExcelSheet     *sheet = g_new (ExcelSheet, 1);
+	ExcelSheet      *sheet = g_new (ExcelSheet, 1);
 	Range           extent;
+	ExcelCell       **p, **pmax;
 
 	g_return_if_fail (value);
 	g_return_if_fail (wb);
@@ -3196,14 +3195,21 @@ new_sheet (ExcelWorkbook *wb, Sheet *value)
 
 	ms_formula_cache_init (sheet);
 	sheet->cell_used_map = cell_used_map_new (sheet);
-	sheet->cells = g_new0 (ExcelCell, sheet->maxy * sheet->maxx);
+	
+	sheet->cells = g_new (ExcelCell *, sheet->maxy);
+	for (p = sheet->cells, pmax = p + sheet->maxy; p < pmax; p++)
+		*p = g_new0 (ExcelCell, sheet->maxx);
 }
 
 static void
 free_sheet (ExcelSheet *sheet)
 {
+	ExcelCell     **p, **pmax;
+
 	if (sheet) {
 		g_free (sheet->cell_used_map);
+		for (p = sheet->cells, pmax = p + sheet->maxy; p < pmax; p++)
+			g_free (*p);
 		g_free (sheet->cells);
 		g_array_free (sheet->dbcells, TRUE);
 		ms_formula_cache_shutdown (sheet);
