@@ -20,12 +20,6 @@
  */
 static EvalPosition *eval_pos = NULL;
 
-static int
-scm_num2int (SCM num)
-{
-	return (int)scm_num2long(num, (char*)SCM_ARG1, "scm_num2int"); /* may I use scm_num2long? */
-}
-
 static SCM
 scm_symbolfrom0str (char *name)
 {
@@ -61,8 +55,8 @@ scm_to_cell_ref (SCM scm)
 	    && SCM_NIMP(SCM_CDR(scm)) && SCM_CONSP(SCM_CDR(scm))
 	    && SCM_NFALSEP(scm_number_p(SCM_CADR(scm))) && SCM_NFALSEP(scm_number_p(SCM_CDDR(scm))))
 	{
-		cell.col = (int)scm_num2int(SCM_CADR(scm));
-		cell.row = (int)scm_num2int(SCM_CDDR(scm));
+		cell.col = gh_scm2int(SCM_CADR(scm));
+		cell.row = gh_scm2int(SCM_CDDR(scm));
 	}
 	else
 		;		/* FIXME: should report error */
@@ -81,11 +75,10 @@ value_to_scm (Value *val, CellRef cell_ref)
 		case VALUE_EMPTY :
 			/* FIXME ?? what belongs here */
 			return scm_long2num(0);
-
+ 
 		case VALUE_BOOLEAN :
-			/* FIXME ?? what belongs here */
-			return scm_long2num(val->v.v_bool);
-
+			return gh_bool2scm(val->v.v_bool);	
+			
 		case VALUE_ERROR :
 			/* FIXME ?? what belongs here */
 			return scm_makfrom0str(val->v.error.mesg->str);
@@ -142,8 +135,6 @@ scm_to_value (SCM scm)
                  *  size of scm is bigger than the size of int
 		 */
 
-		//return value_new_int((int)scm_num2int(scm));
-
 		return value_new_float ((float_t)scm_num2dbl(scm, 0));
 	}
 	else if (SCM_NIMP(scm) && SCM_CONSP(scm))
@@ -160,6 +151,10 @@ scm_to_value (SCM scm)
 		}
 	}
 
+	else if (gh_boolean_p (scm))
+		      
+		return value_new_bool ((gboolean) gh_scm2bool (scm));		       
+
 	return NULL;		/* maybe we should return something more meaningful!? */
 }
 
@@ -208,7 +203,11 @@ expr_to_scm (ExprTree *expr, CellRef cell_ref)
 					 expr_to_scm(expr->u.binary.value_a, cell_ref),
 					 expr_to_scm(expr->u.binary.value_b, cell_ref));
 
-		case OPER_NEG :
+		case OPER_UNARY_PLUS :
+			return SCM_LIST2(scm_symbolfrom0str("+"),
+					 expr_to_scm(expr->u.value, cell_ref));
+
+		case OPER_UNARY_NEG :
 			return SCM_LIST2(scm_symbolfrom0str("neg"),
 					 expr_to_scm(expr->u.value, cell_ref));
 
@@ -249,6 +248,11 @@ expr_to_scm (ExprTree *expr, CellRef cell_ref)
 			return scm_cons(scm_symbolfrom0str("var"),
 					cell_ref_to_scm(expr->u.ref, cell_ref));
 
+	        case OPER_NAME :
+
+	        case OPER_ARRAY :
+		
+
 		/* FIXME : default : */
 	}
 
@@ -266,19 +270,19 @@ func_scm_apply (FunctionEvalInfo *ei, GList *expr_node_list)
 		result;
 
 	if (g_list_length(expr_node_list) < 1)
-		return value_new_error (&ei->pos, _("Invalid number of arguments"));
+		return value_new_error (ei->pos, _("Invalid number of arguments"));
 
-	value = eval_expr(ei, (ExprTree*)expr_node_list->data);
+	value = eval_expr(ei->pos, (ExprTree*)expr_node_list->data);
 	if (value == NULL)
-		return value_new_error (&ei->pos, _("First argument to SCM must be a Guile expression"));
+		return value_new_error (ei->pos, _("First argument to SCM must be a Guile expression"));
 
 	symbol = value_get_as_string (value);
 	if (symbol == NULL)
-		return value_new_error (&ei->pos, _("First argument to SCM must be a Guile expression"));
+		return value_new_error (ei->pos, _("First argument to SCM must be a Guile expression"));
 
 	function = scm_eval_0str(symbol);
 	if (SCM_UNBNDP(function))
-		return value_new_error (&ei->pos, _("Undefined scheme function"));
+		return value_new_error (ei->pos, _("Undefined scheme function"));
 
 	value_release(value);
 
@@ -286,15 +290,15 @@ func_scm_apply (FunctionEvalInfo *ei, GList *expr_node_list)
 	{
 		CellRef eval_cell;
 
-		eval_cell.col = ei->pos.eval.col;
-		eval_cell.row = ei->pos.eval.row;
+		eval_cell.col = ei->pos->eval.col;
+		eval_cell.row = ei->pos->eval.row;
 		eval_cell.col_relative = 0;
 		eval_cell.row_relative = 0;
 		eval_cell.sheet = NULL;
 
 		value = eval_expr(ei, (ExprTree*)g_list_nth(expr_node_list, i)->data);
 		if (value == NULL)
-			return value_new_error (&ei->pos, _("Could not evaluate argument"));
+			return value_new_error (ei->pos, _("Could not evaluate argument"));
 
 		args = scm_cons(value_to_scm(value, eval_cell), args);
 		value_release(value);
@@ -401,7 +405,7 @@ static Value*
 func_marshal_func (FunctionEvalInfo *ei, Value *argv[])
 {
 	GList *l;
-	FunctionDefinition *fndef = ei->func_def;
+	FunctionDefinition const *fndef = ei->func_def;
 	SCM args = SCM_EOL, result, function;
 	CellRef dummy = { 0, 0, 0, 0 };
 	EvalPosition *old_eval_pos;
@@ -411,7 +415,7 @@ func_marshal_func (FunctionEvalInfo *ei, Value *argv[])
 
 	l = g_list_find_custom(funclist, fndef, (GCompareFunc)fndef_compare);
 	if (l == NULL)
-		return value_new_error (&ei->pos, _("Unable to lookup Guile function."));
+		return value_new_error (ei->pos, _("Unable to lookup Guile function."));
 
 	function = ((FuncData*)l->data)->function;
 
@@ -419,7 +423,7 @@ func_marshal_func (FunctionEvalInfo *ei, Value *argv[])
 		args = scm_cons(value_to_scm(argv[i], dummy), args);
 
 	old_eval_pos = eval_pos;
-	eval_pos = &ei->pos;
+	eval_pos = ei->pos;
 	result = scm_apply(function, args, SCM_EOL);
 	eval_pos = old_eval_pos;
 
