@@ -20,6 +20,7 @@
  */
 
 #include <gnome.h>
+#include <errno.h>
 #include "config.h"
 #include "html.h"
 #include "font.h"
@@ -76,6 +77,14 @@ html_write_cell_str (FILE *fp, Cell *cell, MStyle *mstyle)
 		fprintf (fp, "</TT>");
 }
 
+static void
+html_get_color (MStyle *mstyle, MStyleElementType t, int *r, int *g, int *b)
+{
+	*r = mstyle_get_color (mstyle, t)->color.red >> 8;
+	*g = mstyle_get_color (mstyle, t)->color.green >> 8;
+	*b = mstyle_get_color (mstyle, t)->color.blue >> 8;
+}
+
 /*
  * write a TD
  */
@@ -83,20 +92,22 @@ static void
 html_write_cell32 (FILE *fp, Cell *cell)
 {
 	MStyle *mstyle;
-	unsigned char r, g, b;
+	unsigned int r, g, b;
 
 	if (!cell) {	/* empty cell */
 		fprintf (fp, "\t<TD>");
 	} else {
 		mstyle = cell_get_mstyle (cell);
 		g_return_if_fail (mstyle != NULL);
-		
+
+		fprintf (fp, "\t<TD");
+
 		switch (cell_get_horizontal_align (cell,
 						   mstyle_get_align_h (mstyle))) {
 		case HALIGN_RIGHT :
 			fprintf (fp, " align=right");
 			break;
-			
+
 		case HALIGN_CENTER :
 			fprintf (fp, " align=center");
 			break;
@@ -105,15 +116,11 @@ html_write_cell32 (FILE *fp, Cell *cell)
 		}
 		if (mstyle_get_align_v (mstyle) & VALIGN_TOP)
 			fprintf (fp, " valign=top");
-		r = mstyle_get_color (mstyle, MSTYLE_COLOR_BACK)->color.red >> 8;
-		g = mstyle_get_color (mstyle, MSTYLE_COLOR_BACK)->color.green >> 8;
-		b = mstyle_get_color (mstyle, MSTYLE_COLOR_BACK)->color.blue >> 8;
-		if (r != 255 || g != 255 || b != 255)
+		html_get_color (mstyle, MSTYLE_COLOR_BACK, &r, &g, &b);
+		if (r < 255 || g < 255 || b < 255)
 			fprintf (fp, " bgcolor=\"#%02X%02X%02X\"", r, g, b);
 		fprintf (fp, ">");
-		r = mstyle_get_color (mstyle, MSTYLE_COLOR_FORE)->color.red >> 8;
-		g = mstyle_get_color (mstyle, MSTYLE_COLOR_FORE)->color.green >> 8;
-		b = mstyle_get_color (mstyle, MSTYLE_COLOR_FORE)->color.blue >> 8;
+		html_get_color (mstyle, MSTYLE_COLOR_FORE, &r, &g, &b);
 		if (r != 0 || g != 0 || b != 0)
 			fprintf (fp, "<FONT color=\"#%02X%02X%02X\">",
 				 r, g, b);
@@ -133,7 +140,7 @@ static void
 html_write_cell40 (FILE *fp, Cell *cell)
 {
 	MStyle *mstyle;
-	unsigned char r, g, b;
+	unsigned int r, g, b;
 
 	if (!cell) {	/* empty cell */
 		fprintf (fp, "\t<TD>");
@@ -141,12 +148,14 @@ html_write_cell40 (FILE *fp, Cell *cell)
 		mstyle = cell_get_mstyle (cell);
 		g_return_if_fail (mstyle != NULL);
 
+		fprintf (fp, "\t<TD");
+
 		switch (cell_get_horizontal_align (cell,
 						   mstyle_get_align_h (mstyle))) {
 		case HALIGN_RIGHT :
 			fprintf (fp, " halign=right");
 			break;
-			
+
 		case HALIGN_CENTER :
 			fprintf (fp, " halign=center");
 			break;
@@ -155,15 +164,11 @@ html_write_cell40 (FILE *fp, Cell *cell)
 		}
 		if (mstyle_get_align_v (mstyle) & VALIGN_TOP)
 			fprintf (fp, " valign=top");
-		r = mstyle_get_color (mstyle, MSTYLE_COLOR_BACK)->color.red >> 8;
-		g = mstyle_get_color (mstyle, MSTYLE_COLOR_BACK)->color.green >> 8;
-		b = mstyle_get_color (mstyle, MSTYLE_COLOR_BACK)->color.blue >> 8;
-		if (r != 255 || g != 255 || b != 255)
+		html_get_color (mstyle, MSTYLE_COLOR_BACK, &r, &g, &b);
+		if (r < 255 || g < 255 || b < 255)
 			fprintf (fp, " bgcolor=\"#%02X%02X%02X\"", r, g, b);
 		fprintf (fp, ">");
-		r = mstyle_get_color (mstyle, MSTYLE_COLOR_FORE)->color.red >> 8;
-		g = mstyle_get_color (mstyle, MSTYLE_COLOR_FORE)->color.green >> 8;
-		b = mstyle_get_color (mstyle, MSTYLE_COLOR_FORE)->color.blue >> 8;
+		html_get_color (mstyle, MSTYLE_COLOR_FORE, &r, &g, &b);
 		if (r != 0 || g != 0 || b != 0)
 			fprintf (fp, "<FONT color=\"#%02X%02X%02X\">",
 				 r, g, b);
@@ -347,7 +352,7 @@ html_get_string (char *s, int *flags)
 /*
  * try at least to read back what we have written before..
  */
-gboolean
+char *
 html_read (Workbook *wb, const char *filename)
 {
 	FILE *fp;
@@ -358,14 +363,13 @@ html_read (Workbook *wb, const char *filename)
 	char name[64];
 	char buf[LINESIZE];
 
-	if (!filename)
-		return FALSE;
+	g_return_val_if_fail (filename != NULL, "");
 
 	workbook_set_filename (wb, filename);
 
 	fp = fopen (filename, "r");
-	if (!fp) 
-		return FALSE;
+	if (!fp)
+		return g_strdup (g_strerror(errno));
 
 	sheet = NULL;
 	col = 0;
@@ -428,13 +432,13 @@ html_read (Workbook *wb, const char *filename)
 							 */
 							if (flags & HTML_BOLD)
 								mstyle_set_font_bold (mstyle, TRUE);
-							
+
 							if (flags & HTML_ITALIC)
 								mstyle_set_font_italic (mstyle, TRUE);
-							
+
 							if (flags & HTML_RIGHT)
 								mstyle_set_align_h (mstyle, HALIGN_CENTER);
-							
+
 							sheet_style_attach_single (cell->sheet,
 										   cell->col->pos,
 										   cell->row->pos, mstyle);
@@ -448,6 +452,6 @@ html_read (Workbook *wb, const char *filename)
 		}
 	}
 	fclose (fp);
-	return TRUE;
+	return NULL;
 }
 
