@@ -51,15 +51,15 @@ oleo_probe (const char *filename)
 }
 
 
-static gboolean
+static char *
 oleo_load (Workbook *wb, const char *filename)
 {
 	char *name, *p;
-	gboolean ret;
-	
+	char * ret;
+
 	ret = oleo_read (wb, filename);
 
-	if (ret) {
+	if (ret == NULL) {
 		if ((p = filename_ext (filename)) != NULL)
 			*p = '\0'; /* remove "oleo" */
 		name = g_strconcat (p, "gnumeric", NULL);
@@ -83,16 +83,19 @@ oleo_cleanup_plugin (PluginData *pd)
 	g_free (pd->title);
 }
 
-int
-init_plugin (PluginData *pd)
+PluginInitResult
+init_plugin (CommandContext *context, PluginData *pd)
 {
 	char *descr  = _("GNU Oleo (*.oleo) file format");
 
+	if (plugin_version_mismatch  (context, pd, GNUMERIC_VERSION))
+		return PLUGIN_QUIET_ERROR;
+
 	file_format_register_open (100, descr, oleo_probe, oleo_load);
 
 	pd->can_unload     = oleo_can_unload;
 	pd->cleanup_plugin = oleo_cleanup_plugin;
 	pd->title = g_strdup (_("GNU Oleo import plugin"));
 
-	return 0;
+	return PLUGIN_OK;
 }
