@@ -73,16 +73,20 @@ cell_set_formula (Cell *cell, const char *text)
 {
 	ExprTree *new_expr;
 	char *error_msg = _("ERROR");
-	const char *desired_format = NULL;
+	char *desired_format = NULL;
 	ParsePosition pp;
+	gboolean may_set_format;
 
 	g_return_if_fail (cell != NULL);
 	g_return_if_fail (text != NULL);
+	g_return_if_fail (gnumeric_char_start_expr_p (*text));
+
+	may_set_format = !cell_has_assigned_format (cell);
 
 	cell_modified (cell);
-	new_expr = expr_parse_string (&text [1],
+	new_expr = expr_parse_string (text+1, /* Ignore leading char (=,@,+) */
 				      parse_pos_cell (&pp, cell),
-				      &desired_format,
+				      may_set_format ? &desired_format : NULL,
 				      &error_msg);
 	cell_cleanout (cell);
 
@@ -94,21 +98,21 @@ cell_set_formula (Cell *cell, const char *text)
 		return;
 	}
 
-	if (desired_format)
+	/* Until the value is recomputed, we put in this value.  */
+	cell->value = value_new_error (NULL, _("Pending recomputation"));
+
+	if (desired_format) {
 		cell_set_format (cell, desired_format);
+		g_free (desired_format);
+	}
 
 	if (new_expr->oper == OPER_ARRAY) {
 		/* The corner sets up the entire array block */
 		if (new_expr->u.array.x != 0 || new_expr->u.array.y != 0) {
+			/* Throw away the expression, including the inner
+			   one.  */
+			expr_tree_unref (new_expr->u.array.corner.func.expr);
 			expr_tree_unref (new_expr);
-			/*
-			 *   Reading an xml stream the terminal node may be
-			 * very late in the stream & this value needed for
-			 * the premature recomputation that occurs, hence
-			 * zero it.
-			 */
-			if (!cell->value)
-				cell->value = value_new_int (0);
 			return;
 		}
 
@@ -127,8 +131,6 @@ cell_set_formula (Cell *cell, const char *text)
 					new_expr->u.array.corner.func.expr);
 	} else {
 		cell->parsed_node = new_expr;
-		/* Until the value is recomputed, we put in this value.  */
-		cell->value = value_new_error (NULL, _("Pending recomputation"));
 		cell_formula_changed (cell);
 	}
 }
@@ -378,7 +380,9 @@ cell_get_formatted_val (Cell *cell, StyleColor **col)
  * cell_render_value
  * @cell: The cell whose value needs to be rendered
  *
- * The value of the cell is formated according to the format style
+ * The value of the cell is formated according to the format style, but if
+ * formulas are being displayed then use the text of the formula instead of 
+ * its value.
  */
 void
 cell_render_value (Cell *cell)
@@ -397,8 +401,16 @@ cell_render_value (Cell *cell)
 		cell->render_color = NULL;
 	}
 
-	str = cell_get_formatted_val (cell, &color);
-	cell->render_color = color;
+	if (cell->sheet->display_formulas && cell->parsed_node){
+		ParsePosition pp;
+		char *tmpstr = expr_decode_tree (cell->parsed_node,
+						 parse_pos_cell (&pp, cell));
+		str = g_strconcat ("=", tmpstr, NULL);
+		g_free (tmpstr);
+	} else {
+		str = cell_get_formatted_val (cell, &color);		
+		cell->render_color = color;
+	}
 
 	cell_set_rendered_text (cell, str);
 	g_free (str);
@@ -471,9 +483,9 @@ cell_set_text_simple (Cell *cell, const char *text)
 	cell_modified (cell);
 	cell_cleanout (cell);
 
- 	if (text [0] == '=' && text [1] != 0){
+	if (gnumeric_char_start_expr_p (*text) && text[1] != '\0')
 		cell_set_formula (cell, text);
-	} else {
+	else {
 		char *end;
 		long l;
 		int  set=0;
@@ -572,9 +584,6 @@ cell_set_text (Cell *cell, const char *text)
  *
  *   - It does not queue redraws (so you have to queue the redraw yourself
  *     or queue a full redraw).
- *
- *   - It does not queue any recomputations.  You have to queue the
- *     recompute yourself.
  */
 void
 cell_set_formula_tree_simple (Cell *cell, ExprTree *formula)
@@ -882,7 +891,7 @@ cell_relocate (Cell *cell, gboolean const check_bounds)
 	cell_modified (cell);
 
 	/* 2. If the cell contains a formula, relocate the formula */
-	if (cell->parsed_node){
+	if (cell->parsed_node) {
 		sheet_cell_formula_unlink (cell);
 
 		/*
@@ -909,8 +918,8 @@ cell_relocate (Cell *cell, gboolean const check_bounds)
 		 * location.  All the move is doing is a bounds check.
 		 */
 		if (check_bounds) {
-			struct expr_relocate_info	rinfo;
-			EvalPosition     		pos;
+			ExprRelocateInfo	rinfo;
+			EvalPosition 		pos;
 			ExprTree    	*expr = cell->parsed_node;
 
 			rinfo.origin.start.col =
@@ -935,6 +944,9 @@ cell_relocate (Cell *cell, gboolean const check_bounds)
 	/* 3. Move any auxiliary canvas items */
 	if (cell->comment)
 		cell_comment_reposition (cell);
+
+	/* 4. Tag the contents as having changed */
+	cell_content_changed (cell);
 }
 
 /*
@@ -969,11 +981,30 @@ cell_get_horizontal_align (const Cell *cell, int align)
 	return align;
 }
 
-int inline
+gboolean inline
 cell_is_number (const Cell *cell)
 {
 	return cell->value && VALUE_IS_NUMBER (cell->value);
 }
+gboolean
+cell_is_zero (const Cell *cell)
+{
+	Value const * const v = cell->value;
+	if (v == NULL)
+		return FALSE;
+	switch (v->type) {
+	case VALUE_BOOLEAN : return !v->v.v_bool;
+	case VALUE_INTEGER : return v->v.v_int == 0;
+	case VALUE_FLOAT :
+	{
+		double const res = v->v.v_float;
+		return (-1e-10 < res && res < 1e-10);
+	}
+
+	default :
+		return FALSE;
+	}
+}
 
 static inline int
 cell_contents_fit_inside_column (const Cell *cell)
@@ -1471,3 +1502,50 @@ cell_style_changed (Cell *cell)
 					  cell->col->pos + width,
 					  cell->row->pos);
 }
+
+
+gboolean
+cell_has_assigned_format (const Cell *cell)
+{
+	MStyle *mstyle;
+	gboolean result;
+
+	mstyle = sheet_style_compute (cell->sheet, cell->col->pos,
+				      cell->row->pos);
+	if (mstyle_is_element_set (mstyle, MSTYLE_FORMAT)) {
+		const char *format;
+		format = mstyle_get_format (mstyle)->format;
+		/* FIXME: we really should distinguish between "not assigned"
+		   and "assigned General".  */
+		result = (format && strcmp (format, "General") != 0);
+	} else
+		result = FALSE;
+
+	mstyle_unref (mstyle);
+	return result;
+}
+
+
+char *
+cell_get_format (const Cell *cell)
+{
+	MStyle *mstyle;
+	char *result;
+
+	mstyle = sheet_style_compute (cell->sheet, cell->col->pos,
+				      cell->row->pos);
+	if (mstyle_is_element_set (mstyle, MSTYLE_FORMAT)) {
+		const char *format;
+		format = mstyle_get_format (mstyle)->format;
+		/* FIXME: we really should distinguish between "not assigned"
+		   and "assigned General".  */
+		if (format && strcmp (format, "General") != 0)
+			result = g_strdup (format);
+		else
+			result = NULL;
+	} else
+		result = NULL;
+
+	mstyle_unref (mstyle);
+	return result;
+}
