@@ -138,8 +138,6 @@ void        cell_comment_destroy         (Cell *cell);
 void        cell_comment_reposition      (Cell *cell);
 char       *cell_get_comment             (Cell *cell);
 
-void	    cell_cleanout		 (Cell *cell);
-
 void        cell_set_rendered_text       (Cell *cell, const char *rendered_text);
 char *      cell_get_formatted_val       (Cell *cell, StyleColor **col);
 MStyle     *cell_get_mstyle              (const Cell *cell);
@@ -158,7 +156,8 @@ Cell       *cell_copy                    (const Cell *cell);
 void        cell_destroy                 (Cell *cell);
 void        cell_queue_redraw            (Cell *cell);
 int         cell_get_horizontal_align    (const Cell *cell, int align);
-int	    cell_is_number  		 (const Cell *cell);
+gboolean    cell_is_number  		 (const Cell *cell);
+gboolean    cell_is_zero		 (const Cell *cell);
 
 int         cell_draw                    (Cell *cell, SheetView *sheet_view,
 					  GdkGC *gc, GdkDrawable *drawable,
@@ -184,7 +183,7 @@ void        cell_deep_freeze_redraws          (void);
  *
  * These are defined in cellspan.c
  */
-Cell *      row_cell_get_displayed_at    (ColRowInfo *ri, int col);
+Cell *      row_cell_get_displayed_at    (ColRowInfo const * const ri, int const col);
 void        cell_register_span           (Cell *cell, int left, int right);
 void        cell_unregister_span         (Cell *cell);
 
@@ -197,4 +196,7 @@ gboolean    cell_is_blank		 (Cell *cell);
 /* If a cell has an error value return it */
 Value *     cell_is_error                 (Cell const *cell);
 
+char *      cell_get_format               (const Cell *cell);
+gboolean    cell_has_assigned_format      (const Cell *cell);
+
 #endif /* GNUMERIC_CELL_H */
