@@ -8,6 +8,7 @@
 #include <gnome.h>
 #include <locale.h>
 #include <string.h>
+#include <ctype.h>
 #include "gnumeric.h"
 #include "gnumeric-util.h"
 #include "clipboard.h"
@@ -39,29 +40,28 @@ typedef struct {
  *
  * Pastes a cell in the spreadsheet
  */
-static int
+static void
 paste_cell (Sheet *dest_sheet, Cell *new_cell,
 	    int target_col, int target_row,
 	    int paste_flags)
 {
-	g_return_val_if_fail (target_col < SHEET_MAX_COLS, 0);
-	g_return_val_if_fail (target_row < SHEET_MAX_ROWS, 0);
+	g_return_if_fail (target_col < SHEET_MAX_COLS);
+	g_return_if_fail (target_row < SHEET_MAX_ROWS);
 
 	sheet_cell_add (dest_sheet, new_cell, target_col, target_row);
 
-	if (!(paste_flags & PASTE_FORMULAS)){
-		if (new_cell->parsed_node){
+	if (!(paste_flags & PASTE_FORMULAS)) {
+		if (new_cell->parsed_node) {
 			expr_tree_unref (new_cell->parsed_node);
 			new_cell->parsed_node = NULL;
 		}
 	}
 
-	if (new_cell->parsed_node){
-		if (paste_flags & PASTE_FORMULAS){
+	if (new_cell->parsed_node) {
+		if (paste_flags & PASTE_FORMULAS) {
 			cell_relocate (new_cell, TRUE);
 			cell_content_changed (new_cell);
-		}
-		else
+		} else
 			cell_make_value (new_cell);
 	}
 
@@ -71,11 +71,9 @@ paste_cell (Sheet *dest_sheet, Cell *new_cell,
 	sheet_redraw_cell_region (dest_sheet,
 				  target_col, target_row,
 				  target_col, target_row);
-
-	return new_cell->parsed_node != 0;
 }
 
-static int
+static void
 paste_cell_flags (Sheet *dest_sheet, int target_col, int target_row,
 		  CellCopy *c_copy, int paste_flags)
 {
@@ -108,17 +106,14 @@ paste_cell_flags (Sheet *dest_sheet, int target_col, int target_row,
 						    c_copy->u.cell.mstyle);
 			}
 
-			return paste_cell (
-				dest_sheet, new_cell,
-				target_col, target_row, paste_flags);
+			paste_cell (dest_sheet, new_cell,
+				    target_col, target_row, paste_flags);
 		} else {
 			new_cell = sheet_cell_new (dest_sheet,
 						   target_col, target_row);
 			cell_set_text (new_cell, c_copy->u.text);
 		}
 	}
-
-	return 0;
 }
 
 /**
@@ -139,8 +134,7 @@ do_clipboard_paste_cell_region (CommandContext *context,
 				int paste_flags)
 {
 	CellCopyList *l;
-	GList *deps;
-	int formulas = 0;
+	GList        *deps;
 	int col, row, col_inc, row_inc;
 
 	/* clear the region where we will paste */
@@ -167,13 +161,13 @@ do_clipboard_paste_cell_region (CommandContext *context,
 		row_inc = region->rows;
 	}
 
-	for (col = 0; col < paste_width; col += col_inc){
-		for (row = 0; row < paste_height; row += row_inc){
-			for (l = region->list; l; l = l->next){
+	for (col = 0; col < paste_width; col += col_inc) {
+		for (row = 0; row < paste_height; row += row_inc) {
+			for (l = region->list; l; l = l->next) {
 				CellCopy *c_copy = l->data;
 				int target_col, target_row;
 
-				if (paste_flags & PASTE_TRANSPOSE){
+				if (paste_flags & PASTE_TRANSPOSE) {
 					target_col = col + dest_col + c_copy->row_offset;
 					target_row = row + dest_row + c_copy->col_offset;
 				} else {
@@ -187,48 +181,35 @@ do_clipboard_paste_cell_region (CommandContext *context,
 				if (target_row > dest_row + paste_height - 1)
 					continue;
 
-				formulas |= paste_cell_flags (
-					dest_sheet, target_col, target_row,
-					c_copy, paste_flags);
+				paste_cell_flags (dest_sheet, target_col, target_row,
+						  c_copy, paste_flags);
 			}
 		}
 	}
 
-	deps = region_get_dependencies (
-		dest_sheet,
-		dest_col, dest_row,
-		dest_col + paste_width - 1,
-		dest_row + paste_height -1);
+	deps = sheet_region_get_deps (dest_sheet,
+				      dest_col, dest_row,
+				      dest_col + paste_width - 1,
+				      dest_row + paste_height - 1);
 
-	if (deps) {
+	if (deps)
 		cell_queue_recalc_list (deps, TRUE);
-		formulas = 1;
-	}
-
-	/* Trigger a recompute if required */
-	if (formulas)
-		workbook_recalc (dest_sheet->workbook);
 }
 
 static GList *
-new_node (GList *list, char *data, char *p, int col, int row)
+new_node (GList *list, char const *data, char const *p, int col, int row)
 {
 	CellCopy *c_copy;
-	char *text;
 
 	/* Eliminate spaces */
 	while (*data == ' ' && *data)
 		data++;
 
-	text = g_malloc (p-data+1);
-	text = strncpy (text, data, p-data);
-	text [p-data] = 0;
-
 	c_copy = g_new (CellCopy, 1);
 	c_copy->type = CELL_COPY_TYPE_TEXT;
 	c_copy->col_offset = col;
 	c_copy->row_offset = row;
-	c_copy->u.text = text;
+	c_copy->u.text = g_strndup (data, p-data);
 
 	return g_list_prepend (list, c_copy);
 }
@@ -244,16 +225,32 @@ new_node (GList *list, char *data, char *p, int col, int row)
  * \n is a line separator
  */
 static CellRegion *
-x_selection_to_cell_region (char *data, int len)
+x_selection_to_cell_region (char const * data, int len)
 {
 	CellRegion *cr;
 	int cols = 1, cur_col = 0;
 	int rows = 0;
 	GList *list = NULL;
-	char *p = data;
+	char const *p = data;
+	gboolean not_comma_decimal, not_semicolon_decimal;
 
-	for (;len >= 0; len--, p++){
-		if (*p == '\t' || *p == ';' || *p == ',' || *p == '\n'){
+	/* Points to the locale information for number display */
+	static struct lconv *lc = NULL;
+
+	if (!lc)
+		lc = localeconv ();
+	g_return_val_if_fail (lc != NULL, NULL);
+
+	/* Do not use something as a seperator if it is a decimal point.
+	 * This is not perfect.  There is no way to handle thousands seperators
+	 */
+	not_comma_decimal     = NULL == strchr (lc->decimal_point, ',');
+	not_semicolon_decimal = NULL == strchr (lc->decimal_point, ';');
+
+	for (;--len >= 0; p++){
+		if (*p == '\t' || *p == '\n' ||
+		    (*p == ',' && not_comma_decimal) ||
+		    (*p == ';' && not_semicolon_decimal)) {
 			if (p != data)
 				list = new_node (list, data, p, cur_col, rows);
 
@@ -264,7 +261,7 @@ x_selection_to_cell_region (char *data, int len)
 				if (p [1])
 					rows++;
 				cur_col = 0;
-			} 
+			}
 
 			data = p+1;
 		}
@@ -293,7 +290,8 @@ x_selection_to_cell_region (char *data, int len)
  */
 static void
 sheet_paste_selection (CommandContext *context, Sheet *sheet,
-		       CellRegion *content, SheetSelection *ss, clipboard_paste_closure_t *pc)
+		       CellRegion *content, SheetSelection *ss,
+		       clipboard_paste_closure_t *pc)
 {
 	int        paste_height, paste_width;
 	int        end_col, end_row;
@@ -353,7 +351,6 @@ sheet_paste_selection (CommandContext *context, Sheet *sheet,
 	sheet_selection_reset_only (pc->dest_sheet);
 	sheet_selection_append (pc->dest_sheet, pc->dest_col, pc->dest_row);
 	sheet_selection_extend_to (pc->dest_sheet, end_col, end_row);
-
 }
 
 /**
@@ -392,6 +389,8 @@ x_selection_received (GtkWidget *widget, GtkSelectionData *sel, guint time, gpoi
 		g_free (wb->clipboard_paste_callback_data);
 		wb->clipboard_paste_callback_data = NULL;
 	}
+
+	workbook_recalc (wb);
 }
 
 /**
@@ -406,14 +405,27 @@ x_selection_handler (GtkWidget *widget, GtkSelectionData *selection_data, guint
 	CellRegion *clipboard = application_clipboard_contents_get ();
 	char *rendered_selection;
 
+	/* If the region was marked for a cut we need to copy it for pasting
+	 * then clear it
+	 */
 	if (clipboard == NULL) {
 		Sheet *sheet = application_clipboard_sheet_get ();
 		Range const *a = application_clipboard_area_get ();
+
+		g_return_if_fail (sheet != NULL);
+		g_return_if_fail (a != NULL);
+
 		content_needs_free = TRUE;
 		clipboard =
 		    clipboard_copy_cell_range (sheet,
 					       a->start.col, a->start.row,
 					       a->end.col,   a->end.row);
+
+		/* Clear the region that was pasted into another application */
+		sheet_clear_region (workbook_command_context_gui (sheet->workbook),
+				    sheet,
+				    a->start.col, a->start.row,
+				    a->end.col,   a->end.row, FALSE);
 	}
 
 	g_return_if_fail (clipboard != NULL);
@@ -425,8 +437,10 @@ x_selection_handler (GtkWidget *widget, GtkSelectionData *selection_data, guint
 		rendered_selection, strlen (rendered_selection));
 	g_free (rendered_selection);
 
-	if (content_needs_free)
+	if (content_needs_free) {
 		clipboard_release (clipboard);
+		application_clipboard_clear ();
+	}
 }
 
 /**
@@ -437,7 +451,7 @@ x_selection_handler (GtkWidget *widget, GtkSelectionData *selection_data, guint
 static gint
 x_selection_clear (GtkWidget *widget, GdkEventSelection *event, Workbook *wb)
 {
-	wb->have_x_selection = FALSE;
+	application_clipboard_clear ();
 
 	return TRUE;
 }
@@ -451,7 +465,6 @@ x_selection_clear (GtkWidget *widget, GdkEventSelection *event, Workbook *wb)
 void
 x_clipboard_bind_workbook (Workbook *wb)
 {
-	wb->have_x_selection = FALSE;
 	wb->clipboard_paste_callback_data = NULL;
 
 	gtk_signal_connect (
@@ -471,21 +484,6 @@ x_clipboard_bind_workbook (Workbook *wb)
 		GDK_SELECTION_PRIMARY, GDK_SELECTION_TYPE_STRING, 0);
 }
 
-/**
- * clipboard_export_cell_region:
- *
- * This routine exports a CellRegion to the X selection
- */
-static void
-clipboard_export_cell_region (Workbook *wb)
-{
-	wb->have_x_selection = gtk_selection_owner_set (
-		wb->toplevel,
-		GDK_SELECTION_PRIMARY,
-		GDK_CURRENT_TIME);
-
-}
-
 typedef struct {
 	int        base_col, base_row;
 	CellRegion *r;
@@ -542,32 +540,9 @@ clipboard_copy_cell_range (Sheet *sheet,
 	/* reverse the list so that upper left corner is first */
 	c.r->list = g_list_reverse (c.r->list);
 
-	clipboard_export_cell_region (sheet->workbook);
-
 	return c.r;
 }
 
-static gboolean
-workbook_selection_locator (Workbook *wb, gpointer data)
-{
-	Workbook **target = data;
-
-	if (wb->have_x_selection)
-		*target = wb;
-
-	return TRUE;
-}
-
-static Workbook *
-find_local_workbook_with_selection (void)
-{
-	Workbook *result = NULL;
-
-	workbook_foreach (workbook_selection_locator, &result);
-
-	return result;
-}
-
 /**
  * clipboard_paste_region:
  * @region:      A cell region
@@ -586,25 +561,11 @@ clipboard_paste_region (CommandContext *context,
 			int paste_flags,    guint32 time)
 {
 	clipboard_paste_closure_t *data;
-	Workbook *workbook_holding_selection;
 
 	g_return_if_fail (dest_sheet != NULL);
 	g_return_if_fail (IS_SHEET (dest_sheet));
 
-	if (dest_sheet->workbook->clipboard_paste_callback_data != NULL) {
-		g_free (dest_sheet->workbook->clipboard_paste_callback_data);
-		dest_sheet->workbook->clipboard_paste_callback_data = NULL;
-	}
-
-	/*
-	 * OK, we dont have the X selection, so we try to get it:
-	 * we setup all of the parameters for the callback in
-	 * case the X selection fails and we have to fallback to
-	 * using our internal selection
-	 */
 	data = g_new (clipboard_paste_closure_t, 1);
-	dest_sheet->workbook->clipboard_paste_callback_data = data;
-
 	data->region       = region;
 	data->dest_sheet   = dest_sheet;
 	data->dest_col     = dest_col;
@@ -619,29 +580,23 @@ clipboard_paste_region (CommandContext *context,
 	 * to paste from in this case (instead of the simplistic text
 	 * we get from X
 	 */
-
-	workbook_holding_selection = find_local_workbook_with_selection ();
-	if (workbook_holding_selection && region){
-		CellRegion *content;
-
-		content = region;
-
+	if (region) {
 		sheet_paste_selection (context, dest_sheet,
-				       content, dest_sheet->selections->data, data);
-
-		/* Check that this has not already been freed */
-		if (workbook_holding_selection->clipboard_paste_callback_data != NULL) {
-			workbook_holding_selection->clipboard_paste_callback_data = NULL;
-			g_free (data);
-		}
+				       region, dest_sheet->selections->data, data);
+		g_free (data);
 		return;
 	}
 
 	/*
+	 * OK, we dont have the X selection, so we try to get it:
+	 *
 	 * Now, trigger a grab of the X selection.
 	 *
 	 * This will callback x_selection_received
 	 */
+	if (dest_sheet->workbook->clipboard_paste_callback_data != NULL)
+		g_free (dest_sheet->workbook->clipboard_paste_callback_data);
+	dest_sheet->workbook->clipboard_paste_callback_data = data;
 	gtk_selection_convert (
 		dest_sheet->workbook->toplevel, GDK_SELECTION_PRIMARY,
 		GDK_TARGET_STRING, time);
