@@ -16,6 +16,7 @@
 #include "utils.h"
 #include "ranges.h"
 #include "selection.h"
+#include "commands.h"
 #include "command-context-corba.h"
 
 #define verify(cond)          if (!(cond)){ out_of_range (ev); return; }
@@ -217,7 +218,7 @@ Sheet_clear_region (PortableServer_Servant servant,
 	sheet_clear_region (
 		command_context_corba (sheet->workbook),
 		sheet, start_col, start_row,
-		end_col, end_row, TRUE);
+		end_col, end_row, CLEAR_VALUES|CLEAR_FORMATS|CLEAR_COMMENTS);
 }
 
 static void
@@ -229,9 +230,11 @@ Sheet_clear_region_content (PortableServer_Servant servant,
 	Sheet *sheet = sheet_from_servant (servant);
 
 	verify_region (start_col, start_row, end_col, end_row);
-	sheet_clear_region_content (
-		command_context_corba (sheet->workbook), sheet,
-		start_col, start_row, end_col, end_row);
+
+	sheet_clear_region (
+		command_context_corba (sheet->workbook),
+		sheet, start_col, start_row,
+		end_col, end_row, CLEAR_VALUES|CLEAR_COMMENTS);
 }
 
 static void
@@ -243,7 +246,11 @@ Sheet_clear_region_comments (PortableServer_Servant servant,
 	Sheet *sheet = sheet_from_servant (servant);
 
 	verify_region (start_col, start_row, end_col, end_row);
-	sheet_clear_region_comments (sheet, start_col, start_row, end_col, end_row, NULL);
+
+	sheet_clear_region (
+		command_context_corba (sheet->workbook),
+		sheet, start_col, start_row,
+		end_col, end_row, CLEAR_COMMENTS);
 }
 
 static void
@@ -255,7 +262,11 @@ Sheet_clear_region_formats (PortableServer_Servant servant,
 	Sheet *sheet = sheet_from_servant (servant);
 
 	verify_region (start_col, start_row, end_col, end_row);
-	sheet_clear_region_formats (sheet, start_col, start_row, end_col, end_row, NULL);
+
+	sheet_clear_region (
+		command_context_corba (sheet->workbook),
+		sheet, start_col, start_row,
+		end_col, end_row, CLEAR_FORMATS);
 }
 
 static void
@@ -803,7 +814,7 @@ Sheet_insert_col (PortableServer_Servant servant,
 	Sheet *sheet = sheet_from_servant (servant);
 
 	verify_col (col);
-	sheet_insert_cols (
+	cmd_insert_cols (
 		command_context_corba (sheet->workbook), sheet,
 		col, count);
 }
@@ -816,7 +827,7 @@ Sheet_delete_col (PortableServer_Servant servant,
 	Sheet *sheet = sheet_from_servant (servant);
 
 	verify_col (col);
-	sheet_delete_cols (
+	cmd_delete_cols (
 		command_context_corba (sheet->workbook), sheet,
 		col, count);
 }
@@ -829,7 +840,7 @@ Sheet_insert_row (PortableServer_Servant servant,
 	Sheet *sheet = sheet_from_servant (servant);
 
 	verify_row (row);
-	sheet_insert_rows (
+	cmd_insert_rows (
 		command_context_corba (sheet->workbook), sheet,
 		row, count);
 }
@@ -842,7 +853,7 @@ Sheet_delete_row (PortableServer_Servant servant,
 	Sheet *sheet = sheet_from_servant (servant);
 
 	verify_row (row);
-	sheet_delete_rows (
+	cmd_delete_rows (
 		command_context_corba (sheet->workbook), sheet,
 		row, count);
 }
