@@ -69,6 +69,8 @@ gnumeric_goal_seek (Workbook *wb, Sheet *sheet,
 	GoalSeekData seekdata;
 	GoalEvalData evaldata;
 	GoalSeekStatus status;
+	gboolean hadold;
+	float_t oldx;
 
 	goal_seek_initialise (&seekdata);
 	seekdata.xmin = xmin;
@@ -78,12 +80,15 @@ gnumeric_goal_seek (Workbook *wb, Sheet *sheet,
 	evaldata.ycell = set_cell;
 	evaldata.ytarget = target_value;
 
+	hadold = (change_cell->value != NULL);
+	oldx = hadold ? value_get_as_float (change_cell->value) : 0;
+
 	/* PLAN A: Newton's iterative method.  */
 	{
 		float_t x0;
 
-		if (change_cell->value)
-			x0 = value_get_as_float (change_cell->value);
+		if (hadold)
+			x0 = oldx;
 		else
 			x0 = (seekdata.xmin + seekdata.xmax) / 2;
 
@@ -170,6 +175,9 @@ gnumeric_goal_seek (Workbook *wb, Sheet *sheet,
 	if (status == GOAL_SEEK_OK) {
 		float_t yroot;
 		(void) goal_seek_eval (seekdata.root, &yroot, &evaldata);
+	} else if (hadold) {
+		float_t ydummy;
+		(void) goal_seek_eval (oldx, &ydummy, &evaldata);
 	}
 
 	cell_queue_redraw (change_cell);
@@ -230,8 +238,10 @@ dialog_found_solution (Cell *set_cell, Cell *change_cell, float_t target_value)
 					      (dialog)->vbox), label_box);
 
 	gtk_widget_show_all (dialog);
-        selection = gnome_dialog_run (GNOME_DIALOG (dialog));
-	gnome_dialog_close (GNOME_DIALOG (dialog));
+        selection = gnumeric_dialog_run (set_cell->sheet->workbook, GNOME_DIALOG (dialog));
+
+	if (selection >= 0)
+		gnome_dialog_close (GNOME_DIALOG (dialog));
 
 	g_free (status_str);
 	g_free (target_str);
@@ -329,7 +339,7 @@ dialog_goal_seek (Workbook *wb, Sheet *sheet)
 	gtk_widget_grab_focus (set_entry);
 
 dialog_loop:
-        selection = gnome_dialog_run (GNOME_DIALOG (dialog));
+        selection = gnumeric_dialog_run (wb, GNOME_DIALOG (dialog));
 	if (selection == 0) {
 		Cell    *set_cell;
  	        int     set_cell_col, set_cell_row;
@@ -415,8 +425,10 @@ dialog_loop:
 		        gnome_dialog_close (GNOME_DIALOG (dialog));
 			if (dialog_found_solution (set_cell, change_cell, target_value)) {
 			        /* Goal seek cancelled */
-				cell_set_value (change_cell, old_value);
-				cell_eval (set_cell);
+				if (old_value) {
+					cell_set_value (change_cell, old_value);
+					workbook_recalc (set_cell->sheet->workbook);
+				}
 				return;
 			}
 			if (old_value)
@@ -430,5 +442,6 @@ dialog_loop:
 			value_release (old_value);
 	}
 
-	gnome_dialog_close (GNOME_DIALOG (dialog));
+	if (selection >= 0)
+		gnome_dialog_close (GNOME_DIALOG (dialog));
 }
