@@ -46,7 +46,7 @@ dialog_get_number (Workbook *wb, const char *glade_file, double *init_and_return
 	gnome_dialog_editable_enters (dialog, GTK_EDITABLE (entry));
 
 	gnome_dialog_set_parent (dialog, GTK_WINDOW (wb->toplevel));
-	switch (gnome_dialog_run (dialog)){
+	switch (gnumeric_dialog_run (wb, dialog)){
 	case 1:			/* cancel */
 		res = FALSE;
 		break;
@@ -84,10 +84,12 @@ dialog_get_sheet_name (Workbook *wb, const char *current)
 
 	entry = glade_xml_get_widget (gui, "entry");
 	gtk_entry_set_text (GTK_ENTRY (entry), current);
+	gtk_editable_select_region(GTK_EDITABLE (entry), 0, -1);
+
 	gnome_dialog_editable_enters (dialog, GTK_EDITABLE (entry));	
 
 	gnome_dialog_set_parent (dialog, GTK_WINDOW (wb->toplevel));
-	switch (gnome_dialog_run (dialog)){
+	switch (gnumeric_dialog_run (wb, dialog)){
 	case 1:			/* cancel */
 		break;
 	case -1:		/* window manager close */
