@@ -20,8 +20,8 @@
 /* Different constraint types */
 static const char *constraint_strs[] = {
         N_("<="),
-/*	N_("="),
-	N_(">="),
+/*	N_(">="),
+	N_("="),
 	N_("int"),
 	N_("bool"), */
 	NULL
@@ -114,10 +114,11 @@ add_check_buttons (GtkWidget *box, const char *ops[])
 }
 
 typedef struct {
-        GSList   *constraints;
-        GtkCList *clist;
-        Sheet    *sheet;
-        Workbook *wb;
+        GtkWidget *dialog;
+        GSList    *constraints;
+        GtkCList  *clist;
+        Sheet     *sheet;
+        Workbook  *wb;
 } constraint_dialog_t;
 
 
@@ -167,7 +168,7 @@ dialog_solver_options (Workbook *wb, Sheet *sheet)
 
 	/* Run the dialog */
 	gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
-	gnome_dialog_run (GNOME_DIALOG (dialog));
+	gnumeric_dialog_run (wb, GNOME_DIALOG (dialog));
 
 	sheet->solver_parameters.options.assume_linear_model = 1;
 	sheet->solver_parameters.options.assume_non_negative = 1;
@@ -178,7 +179,7 @@ dialog_solver_options (Workbook *wb, Sheet *sheet)
 }
 
 
-static void
+static int
 add_constraint(constraint_dialog_t *constraint_dialog,
 	       int lhs_col, int lhs_row, int rhs_col, int rhs_row,
 	       char *type_str)
@@ -186,19 +187,30 @@ add_constraint(constraint_dialog_t *constraint_dialog,
 	SolverConstraint *constraint;
 	char             constraint_buf[512];
 	char             *constraint_str[2] = { constraint_buf, NULL };
+	gint             row;
 
 	sprintf(constraint_buf, "%s %s ", 
 		cell_name (lhs_col, lhs_row),
 		type_str);
 	strcat(constraint_buf, cell_name(rhs_col, rhs_row));
 
-	gtk_clist_append (constraint_dialog->clist, constraint_str);
 	constraint = g_new (SolverConstraint, 1);
-	constraint->lhs = sheet_cell_fetch (constraint_dialog->sheet,
-					    lhs_col, lhs_row);
-	constraint->rhs = sheet_cell_fetch (constraint_dialog->sheet,
-					    rhs_col, rhs_row);
+	constraint->lhs = sheet_cell_get (constraint_dialog->sheet,
+					  lhs_col, lhs_row);
+	if (constraint->lhs == NULL) {
+	cell_error:
+ 	        gnumeric_notice (constraint_dialog->wb,
+				 GNOME_MESSAGE_BOX_ERROR,
+				 _("You gave a cell reference that contain "
+				   "no data."));
+	        return 1;
+	}
+	constraint->rhs = sheet_cell_get (constraint_dialog->sheet,
+					  rhs_col, rhs_row);
+	if (constraint->rhs == NULL)
+	        goto cell_error;
 
+	row = gtk_clist_append (constraint_dialog->clist, constraint_str);
 	constraint->type = g_malloc (strlen (type_str) + 1);
 	strcpy (constraint->type, type_str);
 	constraint->str = g_malloc (strlen (constraint_buf)+1);
@@ -206,6 +218,10 @@ add_constraint(constraint_dialog_t *constraint_dialog,
 	constraint_dialog->constraints = 
 	        g_slist_append(constraint_dialog->constraints,
 			       (gpointer) constraint);
+	gtk_clist_set_row_data (constraint_dialog->clist, row, 
+				(gpointer) constraint);
+
+	return 0;
 }
 
 /* 'Constraint Add' button clicked */
@@ -225,6 +241,7 @@ constr_add_click (gpointer data)
 	int                 lhs_col, lhs_row;
 	char                *type_str;
 
+	gtk_widget_hide (constraint_dialog->dialog);
 	if (!dialog) {
 	        GtkWidget *box;
 
@@ -265,10 +282,12 @@ add_dialog:
 	gtk_widget_grab_focus (lhs_entry);
 
 	/* Run the dialog */
-	selection = gnome_dialog_run (GNOME_DIALOG (dialog));
+	selection = gnumeric_dialog_run (constraint_dialog->wb,
+					 GNOME_DIALOG (dialog));
 	
 	if (selection == 1) {
 	        gnome_dialog_close (GNOME_DIALOG (dialog));
+		gtk_widget_show (constraint_dialog->dialog);
 		return;
 	}
 	
@@ -291,8 +310,10 @@ add_dialog:
 
 	type_str = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(type_entry)->entry));
 
-	add_constraint(constraint_dialog, lhs_col, lhs_row, rhs_col, rhs_row,
-		       type_str);
+	if (add_constraint(constraint_dialog, lhs_col,
+			   lhs_row, rhs_col, rhs_row,
+			   type_str))
+	        goto add_dialog;
 
 	gtk_entry_set_text (GTK_ENTRY (lhs_entry), "");
 	gtk_entry_set_text (GTK_ENTRY (rhs_entry), "");
@@ -301,8 +322,10 @@ add_dialog:
 	        goto add_dialog;
 
 	gnome_dialog_close (GNOME_DIALOG (dialog));
+	gtk_widget_show (constraint_dialog->dialog);
 }
 
+static gint selected_row = -1;
 
 /* 'Constraint Change' button clicked */
 static void
@@ -316,10 +339,30 @@ constr_change_click (gpointer data)
 static void
 constr_delete_click (gpointer data)
 {
-        printf("Delete: Not implemented yet.\n");
+        constraint_dialog_t *constraint_dialog = (constraint_dialog_t *) data;
+	gpointer            p;
+
+	if (selected_row >= 0) {
+	        p = gtk_clist_get_row_data (constraint_dialog->clist,
+					    selected_row);
+		constraint_dialog->constraints = 
+		        g_slist_remove (constraint_dialog->constraints, p);
+
+	        gtk_clist_remove (constraint_dialog->clist, selected_row);
+	}
 }
 
 
+static void
+constraint_select_click (GtkWidget      *clist,
+			 gint           row,
+			 gint           column,
+			 GdkEventButton *event,
+			 gpointer       data)
+{
+        selected_row = row;
+}
+
 void
 dialog_solver (Workbook *wb, Sheet *sheet)
 
@@ -404,12 +447,17 @@ dialog_solver (Workbook *wb, Sheet *sheet)
 		gtk_clist_column_titles_passive (GTK_CLIST (constraint_list));
 		gtk_clist_column_titles_show (GTK_CLIST (constraint_list));
 		gtk_clist_clear (GTK_CLIST (constraint_list));
+		gtk_signal_connect(GTK_OBJECT(constraint_list),
+				   "select_row",
+				   GTK_SIGNAL_FUNC(constraint_select_click),
+				   GTK_OBJECT (&constraint_dialog));
 
 		/* Constraint buttons */
 		constr_add_button = gtk_button_new_with_label (_("Add"));
 		constr_change_button = gtk_button_new_with_label (_("Change"));
 		constr_delete_button = gtk_button_new_with_label (_("Delete"));
 		constr_button_box = gtk_vbox_new (FALSE, 0);
+		constraint_dialog.dialog = dialog;
 		constraint_dialog.constraints = NULL;
 		constraint_dialog.clist = GTK_CLIST (constraint_list);
 		constraint_dialog.sheet = sheet;
@@ -456,7 +504,7 @@ dialog_solver (Workbook *wb, Sheet *sheet)
 main_dialog:
 	
 	/* Run the dialog */
-	selection = gnome_dialog_run (GNOME_DIALOG (dialog));
+	selection = gnumeric_dialog_run (wb, GNOME_DIALOG (dialog));
 
 	switch (selection) {
 	case 1:
