@@ -35,7 +35,7 @@ void      expr_name_clean_sheet     (Sheet    *sheet);
 GList    *expr_name_list       (Workbook *wb, Sheet *sheet, gboolean builtins_too);
 
 /* Evaluate the name's expression */
-Value    *eval_expr_name       (FunctionEvalInfo *ei, const ExprName *exprn);
+Value    *eval_expr_name       (EvalPosition const * const pos, const ExprName *exprn);
 
 char     *expr_name_value      (const ExprName *expr_name);
 
