@@ -75,41 +75,6 @@ parse_pos_cell (ParsePosition *pp, Cell *cell)
 		cell->row->pos);
 }
 
-FunctionEvalInfo *
-func_eval_info_init (FunctionEvalInfo *eval_info, Sheet *sheet, int col, int row)
-{
-	g_return_val_if_fail (eval_info != NULL, NULL);
-       
-	eval_pos_init (&eval_info->pos, sheet, col, row);
-	eval_info->func_def = 0;
-
-	return eval_info;
-}
-
-FunctionEvalInfo *
-func_eval_info_cell (FunctionEvalInfo *eval_info, Cell *cell)
-{
-	g_return_val_if_fail (eval_info != NULL, NULL);
-	g_return_val_if_fail (cell != NULL, NULL);
-
-	return func_eval_info_init (
-		eval_info, cell->sheet,
-		cell->col->pos, cell->row->pos);
-}
-
-FunctionEvalInfo *
-func_eval_info_pos (FunctionEvalInfo *eval_info, const EvalPosition *eval_pos)
-{
-	g_return_val_if_fail (eval_info != NULL, NULL);
-	g_return_val_if_fail (eval_pos != NULL, NULL);
-
-	return func_eval_info_init (
-		eval_info,
-		eval_pos->sheet,
-		eval_pos->eval.col,
-		eval_pos->eval.row);
-}
-
 ExprTree *
 expr_tree_new_constant (Value *v)
 {
@@ -202,7 +167,7 @@ expr_tree_get_const_str (ExprTree const *const expr)
 
 ExprTree *
 expr_parse_string (const char *expr, const ParsePosition *pp,
-		   const char **desired_format, char **error_msg)
+		   char **desired_format, char **error_msg)
 {
 	ExprTree *tree;
 	g_return_val_if_fail (expr != NULL, NULL);
@@ -210,7 +175,6 @@ expr_parse_string (const char *expr, const ParsePosition *pp,
 	switch (gnumeric_expr_parser (expr, pp, desired_format, &tree)) {
 	case PARSE_OK:
 		*error_msg = NULL;
-		tree->ref_count = 1;
 		return tree;
 
 	case PARSE_ERR_SYNTAX:
@@ -422,25 +386,27 @@ expr_tree_unref (ExprTree *tree)
 }
 
 static Value *
-eval_funcall (FunctionEvalInfo *ei, ExprTree const *tree)
+eval_funcall (EvalPosition const * const pos, ExprTree const *tree)
 {
+	FunctionEvalInfo ei;
 	const Symbol *sym;
 	FunctionDefinition *fd;
 	GList *args;
 
-	g_return_val_if_fail (ei != NULL, NULL);
+	g_return_val_if_fail (pos != NULL, NULL);
 	g_return_val_if_fail (tree != NULL, NULL);
 
 	sym = tree->u.function.symbol;
 
 	if (sym->type != SYMBOL_FUNCTION)
-		return value_new_error (&ei->pos, _("Internal error"));
+		return value_new_error (pos, _("Internal error"));
 
 	fd = (FunctionDefinition *)sym->data;
-	ei->func_def = fd;
+	ei.func_def = fd;
+	ei.pos = pos;
 	args = tree->u.function.arg_list;
 
-	return function_call_with_list (ei, args);
+	return function_call_with_list (&ei, args);
 }
 
 /**
@@ -622,20 +588,20 @@ compare (Value const * const a, Value const * const b)
  * necessary.
  */
 static void
-eval_range (FunctionEvalInfo *s, Value *v)
+eval_range (EvalPosition const * const pos, Value *v)
 {
 	int start_col, start_row, end_col, end_row;
 	CellRef * a = &v->v.cell_range.cell_a;
 	CellRef * b = &v->v.cell_range.cell_b;
-	Sheet * sheet = a->sheet ? a->sheet : s->pos.sheet;
+	Sheet * sheet = a->sheet ? a->sheet : pos->sheet;
 	Cell * cell;
 	int r, c;
-	int const gen = s->pos.sheet->workbook->generation;
+	int const gen = pos->sheet->workbook->generation;
 
-	cell_get_abs_col_row (a, &s->pos.eval, &start_col, &start_row);
-	cell_get_abs_col_row (b, &s->pos.eval, &end_col, &end_row);
+	cell_get_abs_col_row (a, &pos->eval, &start_col, &start_row);
+	cell_get_abs_col_row (b, &pos->eval, &end_col, &end_row);
 
-	if (a->sheet != b->sheet) {
+	if (b->sheet && a->sheet != b->sheet) {
 		g_warning ("3D references not-fully supported.\n"
 			   "Recalc may be incorrect");
 		return;
@@ -651,12 +617,12 @@ eval_range (FunctionEvalInfo *s, Value *v)
 }
 
 static Value *
-eval_expr_real (FunctionEvalInfo * const s, ExprTree const * const tree)
+eval_expr_real (EvalPosition const * const pos, ExprTree const * const tree)
 {
 	Value *res = NULL, *a = NULL, *b = NULL;
 	
 	g_return_val_if_fail (tree != NULL, NULL);
-	g_return_val_if_fail (s != NULL, NULL);
+	g_return_val_if_fail (pos != NULL, NULL);
 
 	switch (tree->oper){
 	case OPER_EQUAL:
@@ -667,23 +633,23 @@ eval_expr_real (FunctionEvalInfo * const s, ExprTree const * const tree)
 	case OPER_LTE: {
 		int comp;
 
-		a = eval_expr_real (s, tree->u.binary.value_a);
+		a = eval_expr_real (pos, tree->u.binary.value_a);
 		if (a != NULL) {
 			if (a->type == VALUE_CELLRANGE) {
-				a = expr_implicit_intersection (&s->pos, a);
+				a = expr_implicit_intersection (pos, a);
 				if (a == NULL)
-					return value_new_error (&s->pos, gnumeric_err_VALUE);
+					return value_new_error (pos, gnumeric_err_VALUE);
 			} else if (a->type == VALUE_ERROR)
 				return a;
 		}
 
-		b = eval_expr_real (s, tree->u.binary.value_b);
+		b = eval_expr_real (pos, tree->u.binary.value_b);
 		if (b != NULL) {
 			Value *res = NULL;
 			if (b->type == VALUE_CELLRANGE) {
-				b = expr_implicit_intersection (&s->pos, b);
+				b = expr_implicit_intersection (pos, b);
 				if (b == NULL)
-					res = value_new_error (&s->pos, gnumeric_err_VALUE);
+					res = value_new_error (pos, gnumeric_err_VALUE);
 			} else if (b->type == VALUE_ERROR)
 				res = b;
 
@@ -711,7 +677,7 @@ eval_expr_real (FunctionEvalInfo * const s, ExprTree const * const tree)
 			if (tree->oper == OPER_NOT_EQUAL)
 				return value_new_bool (TRUE);
 
-			return value_new_error (&s->pos, gnumeric_err_VALUE);
+			return value_new_error (pos, gnumeric_err_VALUE);
 		}
 
 		switch (tree->oper) {
@@ -741,7 +707,7 @@ eval_expr_real (FunctionEvalInfo * const s, ExprTree const * const tree)
 
 		default:
 			g_assert_not_reached ();
-			res = value_new_error (&s->pos,
+			res = value_new_error (pos,
 						_("Internal type error"));
 		}
 		return res;
@@ -762,13 +728,13 @@ eval_expr_real (FunctionEvalInfo * const s, ExprTree const * const tree)
 		 */
 
 	        /* Garantees that a != NULL */
-		a = eval_expr (s, tree->u.binary.value_a);
+		a = eval_expr (pos, tree->u.binary.value_a);
 
 		/* Handle implicit intersection */
 		if (a->type == VALUE_CELLRANGE) {
-			a = expr_implicit_intersection (&s->pos, a);
+			a = expr_implicit_intersection (pos, a);
 			if (a == NULL)
-				return value_new_error (&s->pos, gnumeric_err_VALUE);
+				return value_new_error (pos, gnumeric_err_VALUE);
 		}
 
 		/* 1) Error from A */
@@ -778,17 +744,17 @@ eval_expr_real (FunctionEvalInfo * const s, ExprTree const * const tree)
 		/* 2) #!VALUE error if A is not a number */
 		if (!VALUE_IS_NUMBER (a)) {
 			value_release (a);
-			return value_new_error (&s->pos, gnumeric_err_VALUE);
+			return value_new_error (pos, gnumeric_err_VALUE);
 		}
 
 	        /* Garantees that b != NULL */
-		b = eval_expr (s, tree->u.binary.value_b);
+		b = eval_expr (pos, tree->u.binary.value_b);
 
 		/* Handle implicit intersection */
 		if (b->type == VALUE_CELLRANGE) {
-			b = expr_implicit_intersection (&s->pos, a);
+			b = expr_implicit_intersection (pos, a);
 			if (b == NULL)
-				return value_new_error (&s->pos, gnumeric_err_VALUE);
+				return value_new_error (pos, gnumeric_err_VALUE);
 		}
 
 		/* 3) Error from B */
@@ -801,7 +767,7 @@ eval_expr_real (FunctionEvalInfo * const s, ExprTree const * const tree)
 		if (!VALUE_IS_NUMBER (b)) {
 			value_release (a);
 			value_release (b);
-			return value_new_error (&s->pos, gnumeric_err_VALUE);
+			return value_new_error (pos, gnumeric_err_VALUE);
 		}
 
 		if (a->type != VALUE_FLOAT && b->type != VALUE_FLOAT){
@@ -842,7 +808,7 @@ eval_expr_real (FunctionEvalInfo * const s, ExprTree const * const tree)
 
 			case OPER_DIV:
 				if (ib == 0)
-					return value_new_error (&s->pos, gnumeric_err_DIV0);
+					return value_new_error (pos, gnumeric_err_DIV0);
 				dres = (double)ia / (double)ib;
 				ires = (int)dres;
 				if (dres == ires)
@@ -852,7 +818,7 @@ eval_expr_real (FunctionEvalInfo * const s, ExprTree const * const tree)
 
 			case OPER_EXP:
 				if (ia == 0 && ib <= 0)
-					return value_new_error (&s->pos, gnumeric_err_NUM);
+					return value_new_error (pos, gnumeric_err_NUM);
 				dres = pow ((double)ia, (double)ib);
 				ires = (int)dres;
 				if (dres == ires)
@@ -881,41 +847,46 @@ eval_expr_real (FunctionEvalInfo * const s, ExprTree const * const tree)
 
 			case OPER_DIV:
 				return (vb == 0.0)
-				    ? value_new_error (&s->pos,
+				    ? value_new_error (pos,
 						       gnumeric_err_DIV0)
 				    : value_new_float (va / vb);
 
 			case OPER_EXP:
-				if (va == 0 && vb <= 0)
-					return value_new_error (&s->pos, gnumeric_err_NUM);
+				if ((va == 0 && vb <= 0) ||
+				    (va < 0 && vb != (int)vb))
+					return value_new_error (pos, gnumeric_err_NUM);
 				return value_new_float (pow (va, vb));
 
 			default:
 				break;
 			}
 		}
-		return value_new_error (&s->pos, _("Unknown operator"));
+		return value_new_error (pos, _("Unknown operator"));
 
 	case OPER_PERCENT:
-	case OPER_NEG:
+	case OPER_UNARY_NEG:
+	case OPER_UNARY_PLUS:
 	        /* Garantees that a != NULL */
-		a = eval_expr (s, tree->u.value);
+		a = eval_expr (pos, tree->u.value);
 
 		/* Handle implicit intersection */
 		if (a->type == VALUE_CELLRANGE) {
-			a = expr_implicit_intersection (&s->pos, a);
+			a = expr_implicit_intersection (pos, a);
 			if (a == NULL)
-				return value_new_error (&s->pos, gnumeric_err_VALUE);
+				return value_new_error (pos, gnumeric_err_VALUE);
 		}
 
 		if (a->type == VALUE_ERROR)
 			return a;
 
+		if (tree->oper == OPER_UNARY_PLUS)
+			return a;
+
 		if (!VALUE_IS_NUMBER (a)){
 			value_release (a);
-			return value_new_error (&s->pos, gnumeric_err_VALUE);
+			return value_new_error (pos, gnumeric_err_VALUE);
 		}
-		if (tree->oper == OPER_NEG) {
+		if (tree->oper == OPER_UNARY_NEG) {
 			if (a->type == VALUE_INTEGER)
 				res = value_new_int (-a->v.v_int);
 			else if (a->type == VALUE_FLOAT)
@@ -930,10 +901,10 @@ eval_expr_real (FunctionEvalInfo * const s, ExprTree const * const tree)
 	case OPER_CONCAT: {
 		char *sa, *sb, *tmp;
 
-		a = eval_expr_real (s, tree->u.binary.value_a);
+		a = eval_expr_real (pos, tree->u.binary.value_a);
 		if (a != NULL && a->type == VALUE_ERROR)
 			return a;
-		b = eval_expr_real (s, tree->u.binary.value_b);
+		b = eval_expr_real (pos, tree->u.binary.value_b);
 		if (b != NULL && b->type == VALUE_ERROR) {
 			if (a != NULL)
 				value_release (a);
@@ -957,10 +928,10 @@ eval_expr_real (FunctionEvalInfo * const s, ExprTree const * const tree)
 	}
 
 	case OPER_FUNCALL:
-		return eval_funcall (s, tree);
+		return eval_funcall (pos, tree);
 
 	case OPER_NAME:
-		return eval_expr_name (s, tree->u.name);
+		return eval_expr_name (pos, tree->u.name);
 
 	case OPER_VAR: {
 		Sheet *cell_sheet;
@@ -968,20 +939,20 @@ eval_expr_real (FunctionEvalInfo * const s, ExprTree const * const tree)
 		Cell *cell;
 		int col, row;
 
-		if (s->pos.sheet == NULL) {
+		if (pos->sheet == NULL) {
 			/* Only the test program requests this */
 			return value_new_float (3.14);
 		}
 
 		ref = &tree->u.ref;
-		cell_get_abs_col_row (ref, &s->pos.eval, &col, &row);
+		cell_get_abs_col_row (ref, &pos->eval, &col, &row);
 
-		cell_sheet = eval_sheet (ref->sheet, s->pos.sheet);
+		cell_sheet = eval_sheet (ref->sheet, pos->sheet);
 		cell = sheet_cell_get (cell_sheet, col, row);
 		if (cell == NULL)
 			return NULL;
 
-		if (cell->generation != s->pos.sheet->workbook->generation)
+		if (cell->generation != pos->sheet->workbook->generation)
 			cell_eval (cell);
 
 		return value_duplicate (cell->value);
@@ -990,7 +961,7 @@ eval_expr_real (FunctionEvalInfo * const s, ExprTree const * const tree)
 	case OPER_CONSTANT:
 		res = tree->u.constant;
 		if (res->type == VALUE_CELLRANGE)
-			eval_range (s, res);
+			eval_range (pos, res);
 		return value_duplicate (res);
 
 	case OPER_ARRAY:
@@ -1020,11 +991,11 @@ eval_expr_real (FunctionEvalInfo * const s, ExprTree const * const tree)
 			 * is an array return the result, else build an array and
 			 * iterate over the elements, but that theory needs validation.
 			 */
-			a = eval_expr_real (s, tree->u.array.corner.func.expr);
+			a = eval_expr_real (pos, tree->u.array.corner.func.expr);
 			*((Value **)&(tree->u.array.corner.func.value)) = a;
 		} else {
 			ExprTree const * const array =
-			    expr_tree_array_formula_corner (tree, &s->pos);
+			    expr_tree_array_formula_corner (tree, pos);
 			if (array)
 				a = array->u.array.corner.func.value;
 			else
@@ -1033,11 +1004,11 @@ eval_expr_real (FunctionEvalInfo * const s, ExprTree const * const tree)
 
 		if (a != NULL &&
 		    (a->type == VALUE_CELLRANGE || a->type == VALUE_ARRAY)) {
-			int const num_x = value_area_get_width (&s->pos, a);
-			int const num_y = value_area_get_height (&s->pos, a);
+			int const num_x = value_area_get_width (pos, a);
+			int const num_y = value_area_get_height (pos, a);
 
 			/* Evaluate relative to the upper left corner */
-			EvalPosition tmp_ep = s->pos;
+			EvalPosition tmp_ep = *pos;
 			tmp_ep.eval.col -= x;
 			tmp_ep.eval.row -= y;
 
@@ -1047,7 +1018,7 @@ eval_expr_real (FunctionEvalInfo * const s, ExprTree const * const tree)
 			if (y >= 1 && num_y == 1)
 				y = 0;
 			if (x >= num_x || y >= num_y)
-				return value_new_error (&s->pos, gnumeric_err_NA);
+				return value_new_error (pos, gnumeric_err_NA);
 
 			a = (Value *)value_area_get_x_y (&tmp_ep, a, x, y);
 		}
@@ -1058,13 +1029,13 @@ eval_expr_real (FunctionEvalInfo * const s, ExprTree const * const tree)
 	}
 	}
 
-	return value_new_error (&s->pos, _("Unknown evaluation error"));
+	return value_new_error (pos, _("Unknown evaluation error"));
 }
 
 Value *
-eval_expr (FunctionEvalInfo *s, ExprTree const *tree)
+eval_expr (EvalPosition const * const pos, ExprTree const *tree)
 {
-	Value * res = eval_expr_real (s, tree);
+	Value * res = eval_expr_real (pos, tree);
 	if (res == NULL)
 		return value_new_int (0);
 
@@ -1075,6 +1046,20 @@ eval_expr (FunctionEvalInfo *s, ExprTree const *tree)
 	return res;
 }
 
+Value *
+eval_expr_empty (EvalPosition const * const pos, ExprTree const * const tree)
+{
+	Value * res = eval_expr_real (pos, tree);
+
+	if (res && res->type == VALUE_EMPTY) {
+		value_release (res);
+		res = NULL;
+	}
+
+	return res;
+}
+
+
 int
 cell_ref_get_abs_col (CellRef const * const ref, EvalPosition const * const pos)
 {
@@ -1117,14 +1102,50 @@ cell_get_abs_col_row (CellRef const * const cell_ref,
 		*row = cell_ref->row;
 }
 
+/*
+ * Escapes all backslashes and quotes in a string. It is based on glib's
+ * g_strescape.
+ */
+static char * 
+strescape (char *string)
+{
+	char *q;
+	char *escaped;
+	int escapechars = 0;
+	char *p = string;
+
+	g_return_val_if_fail (string != NULL, NULL);
+
+	while (*p != '\000') {
+		if (*p == '\\' || *p == '\"')
+			escapechars++;
+		p++;
+	}
+
+	if (!escapechars)
+		return g_strdup (string);
+
+	escaped = g_new (char, strlen (string) + escapechars + 1);
+
+	p = string;
+	q = escaped;
+
+	while (*p != '\000'){
+		if (*p == '\\' || *p == '\"')
+			*q++ = '\\';
+		*q++ = *p++;
+	}
+	*q = '\000';
+
+	return escaped;
+}
+ 
 /*
  * Converts a parsed tree into its string representation
  * assuming that we are evaluating at col, row
  *
  * This routine is pretty simple: it walks the ExprTree and
  * creates a string representation.
- *
- * FIXME: strings containing quotes will come out wrong.
  */
 static char *
 do_expr_decode_tree (ExprTree *tree, ParsePosition const *pp,
@@ -1151,7 +1172,8 @@ do_expr_decode_tree (ExprTree *tree, ParsePosition const *pp,
 		{ NULL, 0, 0, 0 }, /* Name     */
 		{ NULL, 0, 0, 0 }, /* Constant */
 		{ NULL, 0, 0, 0 }, /* Var      */
-		{ "-",  5, 0, 0 },
+		{ "-",  5, 0, 0 }, /* Unary - */
+		{ "+",  5, 0, 0 }, /* Unary + */
 		{ "%",  5, 0, 0 },
 		{ NULL, 0, 0, 0 }  /* Array    */
 	};
@@ -1190,7 +1212,7 @@ do_expr_decode_tree (ExprTree *tree, ParsePosition const *pp,
 		a = do_expr_decode_tree (tree->u.value, pp, operations[op].prec);
 		opname = operations[op].name;
 
-		if (tree->oper == OPER_NEG) {
+		if (tree->oper != OPER_PERCENT) {
 			if (prec <= paren_level)
 				res = g_strconcat ("(", opname, a, ")", NULL);
 			else
@@ -1278,9 +1300,13 @@ do_expr_decode_tree (ExprTree *tree, ParsePosition const *pp,
 			return res;
 		}
 
-		case VALUE_STRING:
-			/* FIXME: handle quotes in string.  */
-			return g_strconcat ("\"", v->v.str->str, "\"", NULL);
+		case VALUE_STRING: {
+			char *str1, *str2;
+			str1 = strescape(v->v.str->str);
+			str2 = g_strconcat ("\"", str1, "\"", NULL);
+			g_free(str1);
+			return str2;
+		}
 
 		case VALUE_EMPTY:
 			return g_strdup ("");
@@ -1295,11 +1321,11 @@ do_expr_decode_tree (ExprTree *tree, ParsePosition const *pp,
 			vstr = value_get_as_string (v);
 
 			/* If the number has a sign, pretend that it is the
-			   result of OPER_NEG.  It is not clear how we would
+			   result of OPER_UNARY_{NEG,PLUS}.  It is not clear how we would
 			   currently get negative numbers here, but some
 			   loader might do it.  */
 			if ((vstr[0] == '-' || vstr[0] == '+') &&
-			    operations[OPER_NEG].prec <= paren_level) {
+			    operations[OPER_UNARY_NEG].prec <= paren_level) {
 				res = g_strconcat ("(", vstr, ")", NULL);
 				g_free (vstr);
 			} else
@@ -1375,7 +1401,7 @@ enum CellRefRelocate {
 static enum CellRefRelocate
 cellref_relocate (CellRef * const ref,
 		  const EvalPosition * const pos,
-		  const struct expr_relocate_info * const rinfo)
+		  const ExprRelocateInfo * const rinfo)
 {
 	/* For row or column refs
 	 * Ref	From	To
@@ -1408,6 +1434,11 @@ cellref_relocate (CellRef * const ref,
 		(rinfo->origin_sheet == pos->sheet) &&
 		range_contains (&rinfo->origin, pos->eval.col, pos->eval.row);
 
+	/* All references should be valid initially.  We assume that later. */
+	if (col < 0 || col >= SHEET_MAX_COLS ||
+	    row < 0 || row >= SHEET_MAX_ROWS)
+		return CELLREF_RELOCATE_ERR;
+
 	/* Case (a) */
 	if (!from_inside && !to_inside)
 		return CELLREF_NO_RELOCATE;
@@ -1434,18 +1465,18 @@ cellref_relocate (CellRef * const ref,
 			row += rinfo->row_offset;
 		else if (!ref->row_relative && !from_inside)
 			row -= rinfo->row_offset;
-	} else {
+
+		if (col < 0 || col >= SHEET_MAX_COLS ||
+		    row < 0 || row >= SHEET_MAX_ROWS)
+		    return CELLREF_RELOCATE_ERR;
+	} else if (from_inside) {
 		/* Case (c) */
-		if (ref->col_relative && from_inside)
+		if (ref->col_relative)
 			col -= rinfo->col_offset;
-		if (ref->row_relative && from_inside)
+		if (ref->row_relative)
 			row -= rinfo->row_offset;
 	}
 
-	if (col < 0 || col >= SHEET_MAX_COLS ||
-	    row < 0 || row >= SHEET_MAX_ROWS)
-	    return CELLREF_RELOCATE_ERR;
-
 	if (ref->col_relative)
 		col -= pos->eval.col;
 	if (ref->row_relative)
@@ -1477,7 +1508,7 @@ cellref_relocate (CellRef * const ref,
 ExprTree *
 expr_relocate (const ExprTree *expr,
 	       const EvalPosition *pos,
-	       const struct expr_relocate_info *rinfo)
+	       const ExprRelocateInfo *rinfo)
 {
 	g_return_val_if_fail (expr != NULL, NULL);
 
@@ -1667,12 +1698,15 @@ expr_dump_tree (const ExprTree *tree)
 		return;
 		
 	case OPER_PERCENT:
-	case OPER_NEG:
+	case OPER_UNARY_NEG:
+	case OPER_UNARY_PLUS:
 		expr_dump_tree (tree->u.value);
 		if (tree->oper == OPER_PERCENT)
 			printf ("PERCENT\n");
-		else
+		else if (tree->oper == OPER_UNARY_NEG)
 			printf ("NEGATIVE\n");
+		else
+			printf ("POSITIVE\n");
 		return;
 
 	case OPER_ARRAY:
