@@ -31,7 +31,8 @@ typedef enum {
 
         OPER_CONSTANT,		/* Constant value */
 	OPER_VAR,		/* Cell content lookup (variable) */
-	OPER_NEG,		/* Sign inversion */
+	OPER_UNARY_NEG,		/* Sign inversion */
+	OPER_UNARY_PLUS,	/* Mark as positive */
 	OPER_PERCENT,		/* Percentage (value/100) */
 	OPER_ARRAY		/* Array access */
 } Operation;
@@ -41,7 +42,7 @@ typedef enum {
 	case OPER_LTE: case OPER_NOT_EQUAL: \
 	case OPER_ADD: case OPER_SUB: case OPER_MULT: case OPER_DIV: \
 	case OPER_EXP: case OPER_CONCAT
-#define OPER_ANY_UNARY OPER_NEG: case OPER_PERCENT
+#define OPER_ANY_UNARY OPER_UNARY_NEG: case OPER_UNARY_PLUS : case OPER_PERCENT
 
 struct _ArrayRef {
 	int   x, y;
@@ -107,8 +108,8 @@ typedef Value *(FunctionArgs)  (FunctionEvalInfo *ei, Value **args);
 typedef Value *(FunctionNodes) (FunctionEvalInfo *ei, GList *nodes);
 
 struct _FunctionEvalInfo {
-	EvalPosition        pos;
-	FunctionDefinition *func_def;
+	EvalPosition const * pos;
+	FunctionDefinition const *func_def;
 };
 
 /*
@@ -122,9 +123,6 @@ EvalPosition     *eval_pos_init       (EvalPosition *, Sheet *s, int col, int ro
 EvalPosition     *eval_pos_cell       (EvalPosition *, Cell *);
 ParsePosition    *parse_pos_init      (ParsePosition *, Workbook *wb, int col, int row);
 ParsePosition    *parse_pos_cell      (ParsePosition *, Cell *);
-FunctionEvalInfo *func_eval_info_init (FunctionEvalInfo *s, Sheet *sheet, int col, int row);
-FunctionEvalInfo *func_eval_info_cell (FunctionEvalInfo *s, Cell *cell);
-FunctionEvalInfo *func_eval_info_pos  (FunctionEvalInfo *s, const EvalPosition *fp);
 
 /*
  * Built in / definable sheet names.
@@ -149,11 +147,11 @@ void        cell_get_abs_col_row   (CellRef const * const cell_ref,
 				    int * const col, int * const row);
 
 ExprTree   *expr_parse_string      (const char *expr, const ParsePosition *pp,
-				    const char **desired_format, char **error_msg);
+				    char **desired_format, char **error_msg);
 /* In parser.y  */
 ParseErr    gnumeric_expr_parser   (const char *expr,
 				    const ParsePosition *pp,
-				    const char **desired_format,
+				    char **desired_format,
 				    ExprTree **result);
 
 ExprTree   *expr_tree_duplicate    (ExprTree *expr);
@@ -172,7 +170,7 @@ ExprTree   *expr_tree_array_formula (int const x, int const y, int const rows,
 void        expr_tree_ref          (ExprTree *tree);
 void        expr_tree_unref        (ExprTree *tree);
 
-struct expr_relocate_info {
+struct _ExprRelocateInfo {
 	Range origin;		/* References to cells in origin_sheet!range */
 	Sheet *origin_sheet;	/* should to adjusted */
 
@@ -182,7 +180,7 @@ struct expr_relocate_info {
 
 ExprTree       *expr_relocate (const ExprTree *expr,
 			       const EvalPosition *pos,
-			       const struct expr_relocate_info *info);
+			       const ExprRelocateInfo *info);
 
 int             expr_tree_get_const_int (ExprTree const *const expr);
 char const *	expr_tree_get_const_str (ExprTree const *const expr);
@@ -194,9 +192,13 @@ void expr_dump_tree (const ExprTree *tree);
  * Returns int(0) if the expression uses a non-existant cell for anything
  * other than an equality test.
  */
-Value       *eval_expr (FunctionEvalInfo * const s,
+Value       *eval_expr (EvalPosition const * const pos,
 			ExprTree const * const tree);
 
+/* Same as eval_expr, except that this return NULL for empty values.  */
+Value       *eval_expr_empty (EvalPosition const * const pos,
+			      ExprTree const * const tree);
+
 Value       *expr_implicit_intersection (EvalPosition const * const pos,
 					 Value * const v);
 
