@@ -37,6 +37,7 @@
 #include "dates.h"
 #include "utils.h"
 #include "portability.h"
+#include "datetime.h"
 
 /* Points to the locale information for number display */
 static struct lconv *lc = NULL;
@@ -688,30 +689,19 @@ split_time (gdouble number)
 {
 	static struct tm tm;
 	guint secs;
+	GDate* date;
 
-	GDate* date = g_date_new_serial (number);
-
+	date = datetime_serial_to_g (datetime_serial_raw_to_serial (number));
 	g_date_to_struct_tm (date, &tm);
+	g_date_free (date);
 
-	/*
-	 * WARNING : Be very careful about rounding here
-	 *   86400 is not a great number to multiply by and one can easily
-	 *   loose a second, and hence an entire minute.
-	 *   We first round the seconds, then do integer calculations
-	 *   for the rest.
-	 */
-	secs = ((number * 86400. + .5)) - (((int)number) * 86400);
-
+	secs = datetime_serial_raw_to_seconds (number);
 	tm.tm_hour = secs / 3600;
 	secs -= tm.tm_hour * 3600;
-
 	tm.tm_min  = secs / 60;
 	secs -= tm.tm_min * 60;
-
 	tm.tm_sec  = secs;
 
-	g_date_free (date);
-
 	return &tm;
 }
 
