@@ -121,16 +121,20 @@ FunctionDefinition *function_add_nodes (FunctionCategory *parent,
 					char **help,
 					FunctionNodes *fn);
 
+gboolean            function_is_unused (FunctionDefinition *fndef);
+void                function_remove    (FunctionCategory   *parent,
+					FunctionDefinition *fndef);
+
 gpointer function_def_get_fn           (FunctionDefinition *fndef);
 void     function_def_set_user_data    (FunctionDefinition *fndef,
 					gpointer user_data);
 
 gpointer function_def_get_user_data    (FunctionDefinition *fndef);
 
-void     function_def_count_args       (FunctionDefinition *fndef,
+void     function_def_count_args       (FunctionDefinition const *fndef,
 					int *min, int *max);
 /* 0 based arg_index */
-char     function_def_get_arg_type     (FunctionDefinition *fndef,
+char     function_def_get_arg_type     (FunctionDefinition const *fndef,
 					int arg_idx);
 const char *function_def_get_name      (FunctionDefinition *fndef);
 
@@ -197,9 +201,6 @@ void parse_criteria                 (const char *criteria,
 				     criteria_test_fun_t *fun,
 				     Value **test_value);
 
-float_t get_serial_date (const Value *v);
-GDate *get_date (const Value *v);
-
 int  solver_simplex                 (Workbook *wb, Sheet *sheet);
 
 #endif /* GNUMERIC_FUNC_H */
