@@ -54,7 +54,7 @@ val_to_base (FunctionEvalInfo *ei, Value **argv, int num_argv,
 	int digit;
 
 	if (src_base <= 1 || dest_base <= 1 || dest_base > 36)
-		return value_new_error (&ei->pos, _("Base error"));
+		return value_new_error (ei->pos, _("Base error"));
 
 	value = argv[0];
 	if (num_argv > 1)
@@ -64,7 +64,7 @@ val_to_base (FunctionEvalInfo *ei, Value **argv, int num_argv,
 
 	if (val_places) {
 		if (!VALUE_IS_NUMBER (val_places))
-			return value_new_error (&ei->pos, gnumeric_err_VALUE);
+			return value_new_error (ei->pos, gnumeric_err_VALUE);
 		places = value_get_as_int (val_places);
 	} else
 		places = 0;
@@ -90,12 +90,12 @@ val_to_base (FunctionEvalInfo *ei, Value **argv, int num_argv,
 		break;
 	case VALUE_EMPTY:
 	default:
-		return value_new_error (&ei->pos, gnumeric_err_NUM);
+		return value_new_error (ei->pos, gnumeric_err_NUM);
 	}
 
 	v = strtol (str, &err, src_base);
 	if (*err)
-		return value_new_error (&ei->pos, gnumeric_err_NUM);
+		return value_new_error (ei->pos, gnumeric_err_NUM);
 
 	b10 = pow (src_base, 10);
 	if (v >= b10 / 2) /* N's complement */
@@ -117,7 +117,7 @@ val_to_base (FunctionEvalInfo *ei, Value **argv, int num_argv,
 	if (places > max)
 		max = places;
 	if (max >= sizeof (buffer))
-		return value_new_error (&ei->pos, _("Unimplemented"));
+		return value_new_error (ei->pos, _("Unimplemented"));
 
 	for (digit = max - 1; digit >= 0; digit--) {
 		int thisdigit;
@@ -137,7 +137,7 @@ static char *help_bin2dec = {
 
 	   "@DESCRIPTION="
 	   "BIN2DEC function converts a binary number "
-	   "in string or number to its decimal equivalent."
+	   "in string or number to its decimal equivalent. "
 	   "This function is Excel compatible. "
 	   "\n"
 	   "@EXAMPLES=\n"
@@ -163,7 +163,7 @@ static char *help_bin2oct = {
 	   "@places is an optional field, specifying to zero pad to that "
 	   "number of spaces."
 	   "\n"
-	   "If @places is too small or negative #NUM! error is returned."
+	   "If @places is too small or negative #NUM! error is returned. "
 	   "This function is Excel compatible. "
 	   "\n"
 	   "@EXAMPLES=\n"
@@ -189,7 +189,7 @@ static char *help_bin2hex = {
 	   "hexadecimal number.  @places is an optional field, specifying "
 	   "to zero pad to that number of spaces."
 	   "\n"
-	   "If @places is too small or negative #NUM! error is returned."
+	   "If @places is too small or negative #NUM! error is returned. "
 	   "This function is Excel compatible. "
 	   "\n"
 	   "@EXAMPLES=\n"
@@ -215,7 +215,7 @@ static char *help_dec2bin = {
 	   "@places is an optional field, specifying to zero pad to that "
 	   "number of spaces."
 	   "\n"
-	   "If @places is too small or negative #NUM! error is returned."
+	   "If @places is too small or negative #NUM! error is returned. "
 	   "This function is Excel compatible. "
 	   "\n"
 	   "@EXAMPLES=\n"
@@ -241,7 +241,7 @@ static char *help_dec2oct = {
 	   "@places is an optional field, specifying to zero pad to that "
 	   "number of spaces."
 	   "\n"
-	   "If @places is too small or negative #NUM! error is returned."
+	   "If @places is too small or negative #NUM! error is returned. "
 	   "This function is Excel compatible. "
 	   "\n"
 	   "@EXAMPLES=\n"
@@ -267,7 +267,7 @@ static char *help_dec2hex = {
 	   "number. @places is an optional field, specifying to zero pad "
 	   "to that number of spaces."
 	   "\n"
-	   "If @places is too small or negative #NUM! error is returned."
+	   "If @places is too small or negative #NUM! error is returned. "
 	   "This function is Excel compatible. "
 	   "\n"
 	   "@EXAMPLES=\n"
@@ -290,7 +290,7 @@ static char *help_oct2dec = {
 
 	   "@DESCRIPTION="
 	   "OCT2DEC function converts an octal number "
-	   "in a string or number to its decimal equivalent."
+	   "in a string or number to its decimal equivalent. "
 	   "This function is Excel compatible. "
 	   "\n"
 	   "@EXAMPLES=\n"
@@ -314,7 +314,7 @@ static char *help_oct2bin = {
 	   "@DESCRIPTION="
 	   "The OCT2BIN function converts an octal number to a binary "
 	   "number.  @places is an optional field, specifying to zero pad "
-	   "to that number of spaces."
+	   "to that number of spaces. "
 	   "This function is Excel compatible. "
 	   "\n"
 	   "If @places is too small or negative #NUM! error is returned."
@@ -340,7 +340,7 @@ static char *help_oct2hex = {
 	   "@DESCRIPTION="
 	   "The OCT2HEX function converts an octal number to a hexadecimal "
 	   "number.  @places is an optional field, specifying to zero pad "
-	   "to that number of spaces."
+	   "to that number of spaces. "
 	   "This function is Excel compatible. "
 	   "\n"
 	   "If @places is too small or negative #NUM! error is returned."
@@ -368,7 +368,7 @@ static char *help_hex2bin = {
 	   "number.  @places is an optional field, specifying to zero pad "
 	   "to that number of spaces."
 	   "\n"
-	   "If @places is too small or negative #NUM! error is returned."
+	   "If @places is too small or negative #NUM! error is returned. "
 	   "This function is Excel compatible. "
 	   "\n"
 	   "@EXAMPLES=\n"
@@ -394,7 +394,7 @@ static char *help_hex2oct = {
 	   "number.  @places is an optional field, specifying to zero pad "
 	   "to that number of spaces."
 	   "\n"
-	   "If @places is too small or negative #NUM! error is returned."
+	   "If @places is too small or negative #NUM! error is returned. "
 	   "This function is Excel compatible. "
 	   "\n"
 	   "@EXAMPLES=\n"
@@ -417,7 +417,7 @@ static char *help_hex2dec = {
 
 	   "@DESCRIPTION="
 	   "The HEX2DEC function converts a hexadecimal number "
-	   "to its decimal equivalent."
+	   "to its decimal equivalent. "
 	   "This function is Excel compatible. "
 	   "\n"
 	   "@EXAMPLES=\n"
@@ -446,7 +446,7 @@ static char *help_besseli = {
 	   "truncated."
 	   "\n"
 	   "If @x or @y are not numeric a #VALUE! error is returned. "
-	   "If @y < 0 a #NUM! error is returned." 
+	   "If @y < 0 a #NUM! error is returned. " 
 	   "This function is Excel compatible. "
 	   "\n"
 	   "@EXAMPLES=\n"
@@ -467,7 +467,7 @@ gnumeric_besseli (FunctionEvalInfo *ei, Value **argv)
 	order = value_get_as_float (argv[1]);	/* the order */
 
 	if (order < 0)
-		return value_new_error (&ei->pos, gnumeric_err_NUM);
+		return value_new_error (ei->pos, gnumeric_err_NUM);
 
 	result = bessel_i (x, order, 1.0);
 	return value_new_float (result);
@@ -486,8 +486,8 @@ static char *help_besselk = {
 	   "@y is the order of the bessel function, if non-integer it is "
 	   "truncated."
 	   "\n"
-	   "If x or n are not numeric a #VALUE! error is returned."
-	   "If y < 0 a #NUM! error is returned." 
+	   "If x or n are not numeric a #VALUE! error is returned. "
+	   "If y < 0 a #NUM! error is returned. " 
 	   "This function is Excel compatible. "
 	   "\n"
 	   "@EXAMPLES=\n"
@@ -507,7 +507,7 @@ gnumeric_besselk (FunctionEvalInfo *ei, Value **argv)
 	order = value_get_as_float (argv[1]);	/* the order */
 
 	if (order < 0)
-		return value_new_error (&ei->pos, gnumeric_err_NUM);
+		return value_new_error (ei->pos, gnumeric_err_NUM);
 
 	result = bessel_k (x, order, 1.0);
 	return value_new_float (result);
@@ -526,7 +526,7 @@ static char *help_besselj = {
 	   "truncated."
 	   "\n"
 	   "If @x or @y are not numeric a #VALUE! error is returned.  "
-	   "If @y < 0 a #NUM! error is returned." 
+	   "If @y < 0 a #NUM! error is returned. " 
 	   "This function is Excel compatible. "
 	   "\n"
 	   "@EXAMPLES=\n"
@@ -544,7 +544,7 @@ gnumeric_besselj (FunctionEvalInfo *ei, Value **argv)
 	y = value_get_as_int (argv [1]);
 
 	if (y < 0)
-		return value_new_error (&ei->pos, gnumeric_err_NUM);
+		return value_new_error (ei->pos, gnumeric_err_NUM);
 
 	return value_new_float (jn (y, value_get_as_float (argv [0])));
 }
@@ -562,8 +562,8 @@ static char *help_bessely = {
 	   "@y is the order of the bessel function, if non-integer it is "
 	   "truncated."
 	   "\n"
-	   "If x or n are not numeric a #VALUE! error is returned."
-	   "If n < 0 a #NUM! error is returned." 
+	   "If x or n are not numeric a #VALUE! error is returned. "
+	   "If n < 0 a #NUM! error is returned. " 
 	   "This function is Excel compatible. "
 	   "\n"
 	   "@EXAMPLES=\n"
@@ -580,10 +580,10 @@ gnumeric_bessely (FunctionEvalInfo *ei, Value **argv)
 	    argv[1]->type != VALUE_INTEGER &&
 	    argv[0]->type != VALUE_FLOAT &&
 	    argv[1]->type != VALUE_FLOAT)
-		return value_new_error (&ei->pos, gnumeric_err_VALUE);
+		return value_new_error (ei->pos, gnumeric_err_VALUE);
 
 	if ((y = value_get_as_int (argv [1])) < 0)
-		return value_new_error (&ei->pos, gnumeric_err_NUM);
+		return value_new_error (ei->pos, gnumeric_err_NUM);
 
 	return value_new_float (yn (y, value_get_as_float (argv [0])));
 }
@@ -664,7 +664,7 @@ gnumeric_complex (FunctionEvalInfo *ei, Value **argv)
 
 	if (strcmp(suffix, "i") != 0 &&
 	    strcmp(suffix, "j") != 0)
-		return value_new_error (&ei->pos, gnumeric_err_VALUE);
+		return value_new_error (ei->pos, gnumeric_err_VALUE);
 
 	return value_new_complex (&c, *suffix);
 }
@@ -676,7 +676,7 @@ static char *help_imaginary = {
 	   "@SYNTAX=IMAGINARY(inumber)\n"
 	   "@DESCRIPTION="
 	   "IMAGINARY returns the imaginary coefficient of a complex "
-	   "number."
+	   "number. "
 	   "This function is Excel compatible. "
 	   "\n"
 	   "@EXAMPLES=\n"
@@ -695,7 +695,7 @@ gnumeric_imaginary (FunctionEvalInfo *ei, Value **argv)
 	        return value_new_float (0.0);
 
 	if (value_get_as_complex (argv [0], &c, &imunit))
-		return value_new_error (&ei->pos, gnumeric_err_VALUE);
+		return value_new_error (ei->pos, gnumeric_err_VALUE);
 
 	return value_new_float (c.im);
 }
@@ -706,7 +706,7 @@ static char *help_imreal = {
 	N_("@FUNCTION=IMREAL\n"
 	   "@SYNTAX=IMREAL(inumber)\n"
 	   "@DESCRIPTION="
-	   "IMREAL returns the real coefficient of a complex number."
+	   "IMREAL returns the real coefficient of a complex number. "
 	   "This function is Excel compatible. "
 	   "\n"
 	   "@EXAMPLES=\n"
@@ -726,7 +726,7 @@ gnumeric_imreal (FunctionEvalInfo *ei, Value **argv)
 		return value_duplicate (argv [0]);
 
 	if (value_get_as_complex (argv [0], &c, &imunit))
-		return value_new_error (&ei->pos, gnumeric_err_VALUE);
+		return value_new_error (ei->pos, gnumeric_err_VALUE);
 
 	return value_new_float (c.re);
 }
@@ -737,7 +737,7 @@ static char *help_imabs = {
 	N_("@FUNCTION=IMABS\n"
 	   "@SYNTAX=IMABS(inumber)\n"
 	   "@DESCRIPTION="
-	   "IMABS returns the absolute value of a complex number."
+	   "IMABS returns the absolute value of a complex number. "
 	   "This function is Excel compatible. "
 	   "\n"
 	   "@EXAMPLES=\n"
@@ -754,10 +754,10 @@ gnumeric_imabs (FunctionEvalInfo *ei, Value **argv)
 	char imunit;
 
 	if (value_get_as_complex (argv [0], &c, &imunit))
-	  return value_new_error (&ei->pos, gnumeric_err_VALUE);
+	  return value_new_error (ei->pos, gnumeric_err_VALUE);
 
 	if (argv [0]->type != VALUE_STRING)
-		return value_new_error (&ei->pos, gnumeric_err_VALUE);
+		return value_new_error (ei->pos, gnumeric_err_VALUE);
 
 	return value_new_float (complex_mod (&c));
 }
@@ -768,7 +768,7 @@ static char *help_imconjugate = {
 	N_("@FUNCTION=IMCONJUGATE\n"
 	   "@SYNTAX=IMCONJUGATE(inumber)\n"
 	   "@DESCRIPTION="
-	   "IMCONJUGATE returns the complex conjugate of a complex number."
+	   "IMCONJUGATE returns the complex conjugate of a complex number. "
 	   "This function is Excel compatible. "
 	   "\n"
 	   "@EXAMPLES=\n"
@@ -784,10 +784,10 @@ gnumeric_imconjugate (FunctionEvalInfo *ei, Value **argv)
 	char imunit;
 
 	if (value_get_as_complex (argv[0], &c, &imunit))
-		return value_new_error (&ei->pos, gnumeric_err_VALUE);
+		return value_new_error (ei->pos, gnumeric_err_VALUE);
 
 	if (argv[0]->type != VALUE_STRING)
-		return value_new_error (&ei->pos, gnumeric_err_VALUE);
+		return value_new_error (ei->pos, gnumeric_err_VALUE);
 
 	complex_conj (&res, &c);
 	return value_new_complex (&res, imunit);
@@ -799,7 +799,7 @@ static char *help_imcos = {
 	N_("@FUNCTION=IMCOS\n"
 	   "@SYNTAX=IMCOS(inumber)\n"
 	   "@DESCRIPTION="
-	   "IMCOS returns the cosine of a complex number."
+	   "IMCOS returns the cosine of a complex number. "
 	   "This function is Excel compatible. "
 	   "\n"
 	   "@EXAMPLES=\n"
@@ -815,7 +815,7 @@ gnumeric_imcos (FunctionEvalInfo *ei, Value **argv)
 	char imunit;
 
 	if (value_get_as_complex (argv [0], &c, &imunit))
-		return value_new_error (&ei->pos, gnumeric_err_VALUE);
+		return value_new_error (ei->pos, gnumeric_err_VALUE);
 
 	complex_cos (&res, &c);
 	return value_new_complex (&res, imunit);
@@ -827,7 +827,7 @@ static char *help_imtan = {
 	N_("@FUNCTION=IMTAN\n"
 	   "@SYNTAX=IMTAN(inumber)\n"
 	   "@DESCRIPTION="
-	   "IMTAN returns the tangent of a complex number."
+	   "IMTAN returns the tangent of a complex number. "
 	   "This function is Excel compatible. "
 	   "\n"
 	   "@EXAMPLES=\n"
@@ -842,7 +842,7 @@ gnumeric_imtan (FunctionEvalInfo *ei, Value **argv)
 	char imunit;
 
 	if (value_get_as_complex (argv [0], &c, &imunit))
-		return value_new_error (&ei->pos, gnumeric_err_VALUE);
+		return value_new_error (ei->pos, gnumeric_err_VALUE);
 
 	complex_tan (&res, &c);
 	return value_new_complex (&res, imunit);
@@ -854,7 +854,7 @@ static char *help_imexp = {
 	N_("@FUNCTION=IMEXP\n"
 	   "@SYNTAX=IMEXP(inumber)\n"
 	   "@DESCRIPTION="
-	   "IMEXP returns the exponential of a complex number."
+	   "IMEXP returns the exponential of a complex number. "
 	   "This function is Excel compatible. "
 	   "\n"
 	   "@EXAMPLES=\n"
@@ -870,7 +870,7 @@ gnumeric_imexp (FunctionEvalInfo *ei, Value **argv)
 	char imunit;
 
 	if (value_get_as_complex (argv [0], &c, &imunit))
-		return value_new_error (&ei->pos, gnumeric_err_VALUE);
+		return value_new_error (ei->pos, gnumeric_err_VALUE);
 
 	complex_exp (&res, &c);
 	return value_new_complex (&res, imunit);
@@ -882,7 +882,7 @@ static char *help_imargument = {
 	N_("@FUNCTION=IMARGUMENT\n"
 	   "@SYNTAX=IMARGUMENT(inumber)\n"
 	   "@DESCRIPTION="
-	   "IMARGUMENT returns the argument theta of a complex number."
+	   "IMARGUMENT returns the argument theta of a complex number. "
 	   "This function is Excel compatible. "
 	   "\n"
 	   "@EXAMPLES=\n"
@@ -898,7 +898,7 @@ gnumeric_imargument (FunctionEvalInfo *ei, Value **argv)
 	char imunit;
 
 	if (value_get_as_complex (argv [0], &c, &imunit))
-		return value_new_error (&ei->pos, gnumeric_err_VALUE);
+		return value_new_error (ei->pos, gnumeric_err_VALUE);
 
 	return value_new_float (complex_angle (&c));
 }
@@ -929,7 +929,7 @@ gnumeric_imln (FunctionEvalInfo *ei, Value **argv)
 	char imunit;
 
 	if (value_get_as_complex (argv [0], &c, &imunit))
-		return value_new_error (&ei->pos, gnumeric_err_VALUE);
+		return value_new_error (ei->pos, gnumeric_err_VALUE);
 
 	complex_ln (&res, &c);
 	return value_new_complex (&res, imunit);
@@ -941,7 +941,7 @@ static char *help_imlog2 = {
 	N_("@FUNCTION=IMLOG2\n"
 	   "@SYNTAX=IMLOG2(inumber)\n"
 	   "@DESCRIPTION="
-	   "IMLOG2 returns the logarithm of a complex number in base 2."
+	   "IMLOG2 returns the logarithm of a complex number in base 2. "
 	   "This function is Excel compatible. "
 	   "\n"
 	   "@EXAMPLES=\n"
@@ -957,7 +957,7 @@ gnumeric_imlog2 (FunctionEvalInfo *ei, Value **argv)
 	char imunit;
 
 	if (value_get_as_complex (argv [0], &c, &imunit))
-		return value_new_error (&ei->pos, gnumeric_err_VALUE);
+		return value_new_error (ei->pos, gnumeric_err_VALUE);
 
 	complex_ln (&res, &c);
 	res.re /= M_LN2;
@@ -971,7 +971,7 @@ static char *help_imlog10 = {
 	N_("@FUNCTION=IMLOG10\n"
 	   "@SYNTAX=IMLOG10(inumber)\n"
 	   "@DESCRIPTION="
-	   "IMLOG10 returns the logarithm of a complex number in base 10."
+	   "IMLOG10 returns the logarithm of a complex number in base 10. "
 	   "This function is Excel compatible. "
 	   "\n"
 	   "@EXAMPLES=\n"
@@ -987,7 +987,7 @@ gnumeric_imlog10 (FunctionEvalInfo *ei, Value **argv)
 	char imunit;
 
 	if (value_get_as_complex (argv [0], &c, &imunit))
-		return value_new_error (&ei->pos, gnumeric_err_VALUE);
+		return value_new_error (ei->pos, gnumeric_err_VALUE);
 
 	complex_ln (&res, &c);
 	res.re /= M_LN10;
@@ -1003,7 +1003,7 @@ static char *help_impower = {
 	   "@DESCRIPTION="
 	   "IMPOWER returns a complex number raised to a power.  @inumber is "
 	   "the complex number to be raised to a power and @number is the "
-	   "power to which you want to raise the complex number."
+	   "power to which you want to raise the complex number. "
 	   "This function is Excel compatible. "
 	   "\n"
 	   "@EXAMPLES=\n"
@@ -1019,13 +1019,13 @@ gnumeric_impower (FunctionEvalInfo *ei, Value **argv)
 	char imunit;
 
 	if (value_get_as_complex (argv [0], &a, &imunit))
-		return value_new_error (&ei->pos, gnumeric_err_VALUE);
+		return value_new_error (ei->pos, gnumeric_err_VALUE);
 
 	if (value_get_as_complex (argv [1], &b, &imunit))
-		return value_new_error (&ei->pos, gnumeric_err_VALUE);
+		return value_new_error (ei->pos, gnumeric_err_VALUE);
 
 	if (complex_real_p (&a) && a.re <= 0  && !complex_real_p (&b))
-		return value_new_error (&ei->pos, gnumeric_err_DIV0);
+		return value_new_error (ei->pos, gnumeric_err_DIV0);
 
 	complex_pow (&res, &a, &b);
 	return value_new_complex (&res, imunit);
@@ -1037,7 +1037,7 @@ static char *help_imdiv = {
 	N_("@FUNCTION=IMDIV\n"
 	   "@SYNTAX=IMDIV(inumber,inumber)\n"
 	   "@DESCRIPTION="
-	   "IMDIV returns the quotient of two complex numbers."
+	   "IMDIV returns the quotient of two complex numbers. "
 	   "This function is Excel compatible. "
 	   "\n"
 	   "@EXAMPLES=\n"
@@ -1053,13 +1053,13 @@ gnumeric_imdiv (FunctionEvalInfo *ei, Value **argv)
 	char imunit;
 
 	if (value_get_as_complex (argv [0], &a, &imunit))
-		return value_new_error (&ei->pos, gnumeric_err_VALUE);
+		return value_new_error (ei->pos, gnumeric_err_VALUE);
 
 	if (value_get_as_complex (argv [1], &b, &imunit))
-		return value_new_error (&ei->pos, gnumeric_err_VALUE);
+		return value_new_error (ei->pos, gnumeric_err_VALUE);
 
 	if (complex_zero_p (&b))
-		return value_new_error (&ei->pos, gnumeric_err_DIV0);
+		return value_new_error (ei->pos, gnumeric_err_DIV0);
 
 	complex_div (&res, &a, &b);
 	return value_new_complex (&res, imunit);
@@ -1071,7 +1071,7 @@ static char *help_imsin = {
 	N_("@FUNCTION=IMSIN\n"
 	   "@SYNTAX=IMSIN(inumber)\n"
 	   "@DESCRIPTION="
-	   "IMSIN returns the sine of a complex number."
+	   "IMSIN returns the sine of a complex number. "
 	   "This function is Excel compatible. "
 	   "\n"
 	   "@EXAMPLES=\n"
@@ -1087,7 +1087,7 @@ gnumeric_imsin (FunctionEvalInfo *ei, Value **argv)
 	char imunit;
 
 	if (value_get_as_complex (argv [0], &c, &imunit))
-		return value_new_error (&ei->pos, gnumeric_err_VALUE);
+		return value_new_error (ei->pos, gnumeric_err_VALUE);
 
 	complex_sin (&res, &c);
 	return value_new_complex (&res, imunit);
@@ -1099,7 +1099,7 @@ static char *help_imsqrt = {
 	N_("@FUNCTION=IMSQRT\n"
 	   "@SYNTAX=IMSQRT(inumber)\n"
 	   "@DESCRIPTION="
-	   "IMSQRT returns the square root of a complex number."
+	   "IMSQRT returns the square root of a complex number. "
 	   "This function is Excel compatible. "
 	   "\n"
 	   "@EXAMPLES=\n"
@@ -1115,7 +1115,7 @@ gnumeric_imsqrt (FunctionEvalInfo *ei, Value **argv)
 	char imunit;
 
 	if (value_get_as_complex (argv [0], &c, &imunit))
-		return value_new_error (&ei->pos, gnumeric_err_VALUE);
+		return value_new_error (ei->pos, gnumeric_err_VALUE);
 
 	complex_sqrt (&res, &c);
 	return value_new_complex (&res, imunit);
@@ -1127,7 +1127,7 @@ static char *help_imsub = {
 	N_("@FUNCTION=IMSUB\n"
 	   "@SYNTAX=IMSUB(inumber,inumber)\n"
 	   "@DESCRIPTION="
-	   "IMSUB returns the difference of two complex numbers."
+	   "IMSUB returns the difference of two complex numbers. "
 	   "This function is Excel compatible. "
 	   "\n"
 	   "@EXAMPLES=\n"
@@ -1143,10 +1143,10 @@ gnumeric_imsub (FunctionEvalInfo *ei, Value **argv)
 	char imunit;
 
 	if (value_get_as_complex (argv [0], &a, &imunit))
-		return value_new_error (&ei->pos, gnumeric_err_VALUE);
+		return value_new_error (ei->pos, gnumeric_err_VALUE);
 
 	if (value_get_as_complex (argv [1], &b, &imunit))
-		return value_new_error (&ei->pos, gnumeric_err_VALUE);
+		return value_new_error (ei->pos, gnumeric_err_VALUE);
 
 	complex_sub (&res, &a, &b);
 	return value_new_complex (&res, imunit);
@@ -1158,7 +1158,7 @@ static char *help_improduct = {
 	N_("@FUNCTION=IMPRODUCT\n"
 	   "@SYNTAX=IMPRODUCT(inumber1[,inumber2,...])\n"
 	   "@DESCRIPTION="
-	   "IMPRODUCT returns the product of given complex numbers."
+	   "IMPRODUCT returns the product of given complex numbers. "
 	   "This function is Excel compatible. "
 	   "\n"
 	   "@EXAMPLES=\n"
@@ -1213,7 +1213,7 @@ gnumeric_improduct (FunctionEvalInfo *ei, GList *expr_node_list)
 	p.imunit = 'j';
 	complex_real (&p.res, 1);
 
-        if ((v = function_iterate_argument_values (&ei->pos,
+        if ((v = function_iterate_argument_values (ei->pos,
 						   &callback_function_imoper,
 						   &p, expr_node_list,
 						   TRUE)) != NULL)
@@ -1228,7 +1228,7 @@ static char *help_imsum = {
 	N_("@FUNCTION=IMSUM\n"
 	   "@SYNTAX=IMSUM(inumber,inumber)\n"
 	   "@DESCRIPTION="
-	   "IMSUM returns the sum of two complex numbers."
+	   "IMSUM returns the sum of two complex numbers. "
 	   "This function is Excel compatible. "
 	   "\n"
 	   "@EXAMPLES=\n"
@@ -1247,7 +1247,7 @@ gnumeric_imsum (FunctionEvalInfo *ei, GList *expr_node_list)
 	p.imunit = 'j';
 	complex_real (&p.res, 0);
 
-        if ((v = function_iterate_argument_values (&ei->pos,
+        if ((v = function_iterate_argument_values (ei->pos,
 						   callback_function_imoper,
 						   &p, expr_node_list,
 						   TRUE)) != NULL)
@@ -1618,37 +1618,37 @@ gnumeric_convert (FunctionEvalInfo *ei, Value **argv)
 	        return value_new_float (n - one_C_to_K+1);
 
 	if (convert(weight_units, prefixes, from_unit, to_unit, n, &v,
-		    &ei->pos))
+		    ei->pos))
 	        return v;
 	if (convert(distance_units, prefixes, from_unit, to_unit, n, &v,
-		    &ei->pos))
+		    ei->pos))
 	        return v;
 	if (convert(time_units, NULL, from_unit, to_unit, n, &v,
-		    &ei->pos))
+		    ei->pos))
 	        return v;
 	if (convert(pressure_units, prefixes, from_unit, to_unit, n, &v,
-		    &ei->pos))
+		    ei->pos))
 	        return v;
 	if (convert(force_units, prefixes, from_unit, to_unit, n, &v,
-		    &ei->pos))
+		    ei->pos))
 	        return v;
 	if (convert(energy_units, prefixes, from_unit, to_unit, n, &v,
-		    &ei->pos))
+		    ei->pos))
 	        return v;
 	if (convert(power_units, prefixes, from_unit, to_unit, n, &v,
-		    &ei->pos))
+		    ei->pos))
 	        return v;
 	if (convert(magnetism_units, prefixes, from_unit, to_unit, n, &v,
-		    &ei->pos))
+		    ei->pos))
 	        return v;
 	if (convert(liquid_units, prefixes, from_unit, to_unit, n, &v,
-		    &ei->pos))
+		    ei->pos))
 	        return v;
 	if (convert(magnetism_units, prefixes, from_unit, to_unit, n, &v,
-		    &ei->pos))
+		    ei->pos))
 	        return v;
 
-	return value_new_error (&ei->pos, gnumeric_err_NUM);
+	return value_new_error (ei->pos, gnumeric_err_NUM);
 }
 
 /***************************************************************************/
@@ -1665,7 +1665,7 @@ static char *help_erf = {
 	   "If either @lower_limit or @upper_limit are not numeric a "
 	   "#VALUE! error is returned.  "
 	   "If either @lower_limit or @upper_limit are < 0 a #NUM! error "
-	   "is returned."
+	   "is returned. "
 	   "This function is Excel compatible. "
 	   "\n"
 	   "@EXAMPLES=\n"
@@ -1685,7 +1685,7 @@ gnumeric_erf (FunctionEvalInfo *ei, Value **argv)
 		upper = value_get_as_float (argv [1]);
 
 	if (lower < 0.0 || upper < 0.0)
-		return value_new_error (&ei->pos, gnumeric_err_NUM);
+		return value_new_error (ei->pos, gnumeric_err_NUM);
 
 	ans = erf(lower);
 
@@ -1719,7 +1719,7 @@ gnumeric_erfc (FunctionEvalInfo *ei, Value **argv)
 	float_t x;
 
 	if ((x=value_get_as_float (argv [0]))<0)
-		return value_new_error (&ei->pos, gnumeric_err_NUM);
+		return value_new_error (ei->pos, gnumeric_err_NUM);
 
 	return value_new_float (erfc (x));
 }
@@ -1735,7 +1735,7 @@ static char *help_delta = {
 	   "arguments returning 1 in equality "
 	   "@y is optional, and defaults to 0."
 	   "\n"
-	   "If either argument is non-numeric returns a #VALUE! error."
+	   "If either argument is non-numeric returns a #VALUE! error. "
 	   "This function is Excel compatible. "
 	   "\n"
 	   "@EXAMPLES=\n"
@@ -1773,7 +1773,7 @@ gnumeric_delta (FunctionEvalInfo *ei, Value **argv)
 		ans = value_get_as_float(vx) == value_get_as_float(vy);
 		break;
 	default:
-		err = value_new_error (&ei->pos, _("Impossible"));
+		err = value_new_error (ei->pos, _("Impossible"));
 	}
 	if (!argv [1])
 		value_release (vy);
@@ -1790,7 +1790,7 @@ static char *help_gestep = {
 	   "GESTEP function test for if @x is >= @y, returning 1 if it "
 	   "is so, and 0 otherwise @y is optional, and defaults to 0."
 	   "\n"
-	   "If either argument is non-numeric returns a #VALUE! error."
+	   "If either argument is non-numeric returns a #VALUE! error. "
 	   "This function is Excel compatible. "
 	   "\n"
 	   "@EXAMPLES=\n"
@@ -1828,7 +1828,7 @@ gnumeric_gestep (FunctionEvalInfo *ei, Value **argv)
 		ans = value_get_as_float(vx) >= value_get_as_float(vy);
 		break;
 	default:
-		err = value_new_error (&ei->pos, _("Impossible"));
+		err = value_new_error (ei->pos, _("Impossible"));
 	}
 	       
 	if (!argv [1])
