@@ -12,7 +12,7 @@
 #include "func.h"
 #include "mathfunc.h"
 #include "collect.h"
-
+#include "auto-format.h"
 
 static int
 gcd (int a, int b)
@@ -232,9 +232,9 @@ gnumeric_lcm (FunctionEvalInfo *ei, GList *nodes)
 	result->type = VALUE_INTEGER;
 	result->v.v_int = 1;
 
-	if (function_iterate_argument_values (&ei->pos, callback_function_lcm,
+	if (function_iterate_argument_values (ei->pos, callback_function_lcm,
 					      result, nodes, TRUE) != NULL)
-	    return value_new_error (&ei->pos, gnumeric_err_NUM);
+	    return value_new_error (ei->pos, gnumeric_err_NUM);
 
 	return result;
 }
@@ -248,7 +248,7 @@ static char *help_abs = {
 	   "@DESCRIPTION="
 	   "ABS implements the Absolute Value function:  the result is "
 	   "to drop the negative sign (if present).  This can be done for "
-	   "integers and floating point numbers."
+	   "integers and floating point numbers. "
 	   "This function is Excel compatible. "
 	   "\n"
 	   "@EXAMPLES=\n"
@@ -291,7 +291,7 @@ gnumeric_acos (FunctionEvalInfo *ei, Value **args)
 
 	t = value_get_as_float (args [0]);
 	if ((t < -1.0) || (t > 1.0))
-		return value_new_error (&ei->pos, gnumeric_err_NUM);
+		return value_new_error (ei->pos, gnumeric_err_NUM);
 
 	return value_new_float (acos (t));
 }
@@ -322,7 +322,7 @@ gnumeric_acosh (FunctionEvalInfo *ei, Value **args)
 
 	t = value_get_as_float (args [0]);
 	if (t < 1.0)
-		return value_new_error (&ei->pos, gnumeric_err_NUM);
+		return value_new_error (ei->pos, gnumeric_err_NUM);
 
 	return value_new_float (acosh (t));
 }
@@ -353,7 +353,7 @@ gnumeric_asin (FunctionEvalInfo *ei, Value **args)
 
 	t = value_get_as_float (args [0]);
 	if ((t < -1.0) || (t > 1.0))
-		return value_new_error (&ei->pos, gnumeric_err_NUM);
+		return value_new_error (ei->pos, gnumeric_err_NUM);
 
 	return value_new_float (asin (t));
 }
@@ -370,7 +370,7 @@ static char *help_asinh = {
 	   "This function is Excel compatible. "
 	   "\n"
 	   "@EXAMPLES=\n"
-	   "ASINH(0.5) equals 0.481212."
+	   "ASINH(0.5) equals 0.481212. "
 	   "ASINH(1.0) equals 0.881374."
 	   "\n"
 	   "@SEEALSO=ASIN, ACOSH, SIN, COS, DEGREES, RADIANS")
@@ -390,8 +390,8 @@ static char *help_atan = {
 
 	   "@DESCRIPTION="
 	   "ATAN function calculates the arc tangent of @x; that "
-	   "is the value whose tangent is @x."
-	   "Return value is in radians."
+	   "is the value whose tangent is @x. "
+	   "Return value is in radians. "
 	   "This function is Excel compatible. "
 	   "\n"
 	   "@EXAMPLES=\n"
@@ -419,7 +419,7 @@ static char *help_atanh = {
 	   "This function is Excel compatible. "
 	   "\n"
 	   "@EXAMPLES=\n"
-	   "ATANH(0.5) equals 0.549306."
+	   "ATANH(0.5) equals 0.549306. "
 	   "ATANH(0.8) equals 1.098612."
 	   "\n"
 	   "@SEEALSO=ATAN, TAN, SIN, COS, DEGREES, RADIANS")
@@ -432,7 +432,7 @@ gnumeric_atanh (FunctionEvalInfo *ei, Value **args)
 
 	t = value_get_as_float (args [0]);
 	if ((t <= -1.0) || (t >= 1.0))
-		return value_new_error (&ei->pos, gnumeric_err_NUM);
+		return value_new_error (ei->pos, gnumeric_err_NUM);
 
 	return value_new_float (atanh (value_get_as_float (args [0])));
 }
@@ -448,11 +448,11 @@ static char *help_atan2 = {
 	   "variables @b1 and @b2.  It is similar to calculating the arc "
 	   "tangent of @b2 / @b1, except that the signs of both arguments "
 	   "are used to determine the quadrant of the result. "
-	   "The result is in radians."
+	   "The result is in radians. "
 	   "This function is Excel compatible. "
 	   "\n"
 	   "@EXAMPLES=\n"
-	   "ATAN2(0.5,1.0) equals 1.107149."
+	   "ATAN2(0.5,1.0) equals 1.107149. "
 	   "ATAN2(-0.5,2.0) equals 1.815775."
 	   "\n"
 	   "@SEEALSO=ATAN, ATANH, COS, SIN, DEGREES, RADIANS")
@@ -476,8 +476,8 @@ static char *help_ceil = {
 	   "This function is Excel compatible. "
 	   "\n"
 	   "@EXAMPLES=\n"
-	   "CEIL(0.4) equals 1."
-	   "CEIL(-1.1) equals -1."
+	   "CEIL(0.4) equals 1. "
+	   "CEIL(-1.1) equals -1. "
 	   "CEIL(-2.9) equals -2."
 	   "\n"
 	   "@SEEALSO=ABS, FLOOR, INT")
@@ -529,7 +529,7 @@ gnumeric_countif (FunctionEvalInfo *ei, Value **argv)
 
 	if ((!VALUE_IS_NUMBER(argv[1]) && argv[1]->type != VALUE_STRING)
 	    || (range->type != VALUE_CELLRANGE))
-	        return value_new_error (&ei->pos, gnumeric_err_VALUE);
+	        return value_new_error (ei->pos, gnumeric_err_VALUE);
 
 	if (VALUE_IS_NUMBER(argv[1])) {
 	        items.fun = (criteria_test_fun_t) criteria_test_equal;
@@ -540,9 +540,9 @@ gnumeric_countif (FunctionEvalInfo *ei, Value **argv)
 		tmpval = items.test_value;
 	}
 
-	sheet = eval_sheet (range->v.cell_range.cell_a.sheet, ei->pos.sheet);
-	ret = sheet_cell_foreach_range (
-		sheet, TRUE,
+	sheet = eval_sheet (range->v.cell_range.cell_a.sheet, ei->pos->sheet);
+	ret = sheet_cell_foreach_range ( sheet,
+		FALSE, /* Ignore empty cells */
 		range->v.cell_range.cell_a.col,
 		range->v.cell_range.cell_a.row,
 		range->v.cell_range.cell_b.col,
@@ -554,7 +554,7 @@ gnumeric_countif (FunctionEvalInfo *ei, Value **argv)
 		value_release (tmpval);
 
 	if (ret != NULL)
-	        return value_new_error (&ei->pos, gnumeric_err_VALUE);
+	        return value_new_error (ei->pos, gnumeric_err_VALUE);
 
         list = items.list;
 
@@ -659,7 +659,7 @@ gnumeric_sumif (FunctionEvalInfo *ei, Value **argv)
 
 	if ((!VALUE_IS_NUMBER(argv[1]) && argv[1]->type != VALUE_STRING)
 	    || (range->type != VALUE_CELLRANGE))
-	        return value_new_error (&ei->pos, gnumeric_err_VALUE);
+	        return value_new_error (ei->pos, gnumeric_err_VALUE);
 
 	if (VALUE_IS_NUMBER(argv[1])) {
 	        items.fun = (criteria_test_fun_t) criteria_test_equal;
@@ -676,8 +676,10 @@ gnumeric_sumif (FunctionEvalInfo *ei, Value **argv)
 	        items.actual_range = FALSE;
 
 	ret = sheet_cell_foreach_range (
-		eval_sheet (ei->pos.sheet, ei->pos.sheet),
-		TRUE,
+		eval_sheet (range->v.cell_range.cell_a.sheet, ei->pos->sheet),
+		/* Do not ignore empty cells if there is an actual range */
+		actual_range == NULL,
+
 		range->v.cell_range.cell_a.col,
 		range->v.cell_range.cell_a.row,
 		range->v.cell_range.cell_b.col,
@@ -689,7 +691,7 @@ gnumeric_sumif (FunctionEvalInfo *ei, Value **argv)
 		value_release (tmpval);
 
 	if (ret != NULL)
-	        return value_new_error (&ei->pos, gnumeric_err_VALUE);
+	        return value_new_error (ei->pos, gnumeric_err_VALUE);
 
 	if (actual_range == NULL) {
 	        list = items.list;
@@ -707,14 +709,16 @@ gnumeric_sumif (FunctionEvalInfo *ei, Value **argv)
 	      items.current = items.list;
 	      items.sum = items.total_num = 0;
  	      ret = sheet_cell_foreach_range (
-		      eval_sheet (ei->pos.sheet, ei->pos.sheet),
-		      TRUE,
-		      actual_range->v.cell_range.cell_a.col,
-		      actual_range->v.cell_range.cell_a.row,
-		      actual_range->v.cell_range.cell_b.col,
-		      actual_range->v.cell_range.cell_b.row,
-		      callback_function_sumif,
-		      &items);
+			eval_sheet (actual_range->v.cell_range.cell_a.sheet, ei->pos->sheet),
+			/* Empty cells too.  Criteria and results must align */
+			FALSE,
+
+			actual_range->v.cell_range.cell_a.col,
+			actual_range->v.cell_range.cell_a.row,
+			actual_range->v.cell_range.cell_b.col,
+			actual_range->v.cell_range.cell_b.row,
+			callback_function_sumif,
+			&items);
 	      sum = items.sum;
 	}
 
@@ -740,7 +744,7 @@ static char *help_ceiling = {
 	   "This function is Excel compatible. "
 	   "\n"
 	   "@EXAMPLES=\n"
-	   "CEILING(2.43,1) equals 3."
+	   "CEILING(2.43,1) equals 3. "
 	   "CEILING(123.123,3) equals 126."
 	   "\n"
 	   "@SEEALSO=CEIL")
@@ -759,7 +763,7 @@ gnumeric_ceiling (FunctionEvalInfo *ei, Value **argv)
 	}
 
 	if (s == 0 || number / s < 0)
-		return value_new_error (&ei->pos, gnumeric_err_NUM);
+		return value_new_error (ei->pos, gnumeric_err_NUM);
 
 	return value_new_float (ceil (number / s) * s);
 }
@@ -801,7 +805,7 @@ static char *help_cosh = {
 	   "This function is Excel compatible. "
 	   "\n"
 	   "@EXAMPLES=\n"
-	   "COSH(0.5) equals 1.127626."
+	   "COSH(0.5) equals 1.127626. "
 	   "COSH(1) equals 1.543081."
 	   "\n"
 	   "@SEEALSO=COS, SIN, SINH, TAN, TANH, RADIANS, DEGREES, EXP")
@@ -820,7 +824,7 @@ static char *help_degrees = {
 	   "@SYNTAX=DEGREES(x)\n"
 
 	   "@DESCRIPTION="
-	   "DEGREES computes the number of degrees equivalent to @x radians."
+	   "DEGREES computes the number of degrees equivalent to @x radians. "
 	   "This function is Excel compatible. "
 	   "\n"
 	   "@EXAMPLES=\n"
@@ -890,11 +894,11 @@ gnumeric_fact (FunctionEvalInfo *ei, Value **argv)
 	gboolean x_is_integer;
 
 	if (!VALUE_IS_NUMBER (argv[0]))
-		return value_new_error (&ei->pos, gnumeric_err_NUM);
+		return value_new_error (ei->pos, gnumeric_err_NUM);
 
 	x = value_get_as_float (argv[0]);
 	if (x < 0)
-		return value_new_error (&ei->pos, gnumeric_err_NUM);
+		return value_new_error (ei->pos, gnumeric_err_NUM);
 
 	x_is_integer = (x == floor (x));
 
@@ -917,7 +921,7 @@ static char *help_combin = {
 	   "COMBIN computes the number of combinations. "
 	   "\n"
 	   "Performing this function on a non-integer or a negative number "
-           "returns an error.  Also if @n is less than @k returns an error."
+           "returns an error.  Also if @n is less than @k returns an error. "
 	   "This function is Excel compatible. "
 	   "\n"
 	   "@EXAMPLES=\n"
@@ -954,7 +958,7 @@ gnumeric_combin (FunctionEvalInfo *ei, Value **argv)
 	if (k >= 0 && n >= k)
 		return value_new_float (combin (n ,k));
 
-	return value_new_error (&ei->pos, gnumeric_err_NUM);
+	return value_new_error (ei->pos, gnumeric_err_NUM);
 }
 
 /***************************************************************************/
@@ -965,7 +969,7 @@ static char *help_floor = {
 
 	   "@DESCRIPTION="
 	   "FLOOR function rounds @x down to the next nearest multiple "
-	   "of @significance.  @significance defaults to 1."
+	   "of @significance.  @significance defaults to 1. "
 	   "This function is Excel compatible. "
 	   "\n"
 	   "@EXAMPLES=\n"
@@ -988,7 +992,7 @@ gnumeric_floor (FunctionEvalInfo *ei, Value **argv)
 	}
 
 	if (s == 0 || number / s < 0)
-		return value_new_error (&ei->pos, gnumeric_err_NUM);
+		return value_new_error (ei->pos, gnumeric_err_NUM);
 
 	return value_new_float (floor (number / s) * s);
 }
@@ -1046,11 +1050,14 @@ gnumeric_log (FunctionEvalInfo *ei, Value **argv)
 
 	if (argv[1] == NULL)
 	        base = 10;
-	else
+	else {
 	        base = value_get_as_float (argv[1]);
+		if (base <= 1)
+			return value_new_error (ei->pos, gnumeric_err_NUM);
+	}
 
 	if (t <= 0.0)
-		return value_new_error (&ei->pos, gnumeric_err_VALUE);
+		return value_new_error (ei->pos, gnumeric_err_NUM);
 
 	return value_new_float (log (t) / log (base));
 }
@@ -1080,7 +1087,7 @@ gnumeric_ln (FunctionEvalInfo *ei, Value **argv)
 	t = value_get_as_float (argv [0]);
 
 	if (t <= 0.0)
-		return value_new_error (&ei->pos, gnumeric_err_NUM);
+		return value_new_error (ei->pos, gnumeric_err_NUM);
 
 	return value_new_float (log (t));
 }
@@ -1092,7 +1099,7 @@ static char *help_power = {
 	   "@SYNTAX=POWER(x,y)\n"
 
 	   "@DESCRIPTION="
-	   "POWER returns the value of @x raised to the power @y."
+	   "POWER returns the value of @x raised to the power @y. "
 	   "This function is Excel compatible. "
 	   "\n"
 	   "@EXAMPLES=\n"
@@ -1113,8 +1120,10 @@ gnumeric_power (FunctionEvalInfo *ei, Value **argv)
 	if ((x > 0) || (x == 0 && y > 0) || (x < 0 && y == floor (y)))
 		return value_new_float (pow (x, y));
 
-	/* FIXME: What is supposed to happen for x=y=0?  */
-	return value_new_error (&ei->pos, gnumeric_err_VALUE);
+	if (x == 0 && y != 0)
+		return value_new_error (ei->pos, gnumeric_err_DIV0);
+	else
+		return value_new_error (ei->pos, gnumeric_err_NUM);
 }
 
 /***************************************************************************/
@@ -1140,7 +1149,7 @@ gnumeric_log2 (FunctionEvalInfo *ei, Value **argv)
 
 	t = value_get_as_float (argv [0]);
 	if (t <= 0.0)
-		return value_new_error (&ei->pos, gnumeric_err_NUM);
+		return value_new_error (ei->pos, gnumeric_err_NUM);
 
 	return value_new_float (log (t) / M_LN2);
 }
@@ -1169,7 +1178,7 @@ gnumeric_log10 (FunctionEvalInfo *ei, Value **argv)
 
 	t = value_get_as_float (argv [0]);
 	if (t <= 0.0)
-		return value_new_error (&ei->pos, gnumeric_err_NUM);
+		return value_new_error (ei->pos, gnumeric_err_NUM);
 
 	return value_new_float (log10 (t));
 }
@@ -1196,25 +1205,29 @@ static char *help_mod = {
 static Value *
 gnumeric_mod (FunctionEvalInfo *ei, Value **argv)
 {
-	int a,b;
+	int a, b;
 
 	a = value_get_as_int (argv[0]);
 	b = value_get_as_int (argv[1]);
 
 	if (b == 0)
-		return value_new_error (&ei->pos, gnumeric_err_NUM);
-
-	if (b < 0) {
-		a = -a;
+		return value_new_error (ei->pos, gnumeric_err_DIV0);
+	else if (b > 0) {
+		if (a >= 0)
+			return value_new_int (a % b);
+		else {
+			int c = (-a) % b;
+			return value_new_int (c ? b - c : 0);
+		}
+	} else {
 		b = -b;
-		/* FIXME: check for overflow.  */
-	}
-
-	if (a >= 0)
-		return value_new_int (a % b);
-	else {
-		int invres = (-a) % b;
-		return value_new_int (invres == 0 ? 0 : b - invres);
+		a = -a;
+		if (a >= 0)
+			return value_new_int (-(a % b));
+		else {
+			int c = (-a) % b;
+			return value_new_int (c ? c - b : 0);
+		}
 	}
 }
 
@@ -1271,7 +1284,7 @@ static char *help_sin = {
 
 	   "@DESCRIPTION="
 	   "SIN function returns the sine of @x, where @x is given "
-           "in radians."
+           "in radians. "
 	   "This function is Excel compatible. "
 	   "\n"
 	   "@EXAMPLES=\n"
@@ -1294,7 +1307,7 @@ static char *help_sinh = {
 
 	   "@DESCRIPTION="
 	   "SINH function returns the hyperbolic sine of @x, "
-	   "which is defined mathematically as (exp(@x) - exp(-@x)) / 2."
+	   "which is defined mathematically as (exp(@x) - exp(-@x)) / 2. "
 	   "This function is Excel compatible. "
 	   "\n"
 	   "@EXAMPLES=\n"
@@ -1316,7 +1329,7 @@ static char *help_sqrt = {
 	   "@SYNTAX=SQRT(x)\n"
 
 	   "@DESCRIPTION="
-	   "SQRT function returns the square root of @x."
+	   "SQRT function returns the square root of @x. "
 	   "This function is Excel compatible. "
 	   "\n"
 	   "If @x is negative, SQRT returns #NUM! error. "
@@ -1332,7 +1345,7 @@ gnumeric_sqrt (FunctionEvalInfo *ei, Value **argv)
 {
 	float_t x = value_get_as_float (argv[0]);
 	if (x < 0)
-		return value_new_error (&ei->pos, gnumeric_err_NUM);
+		return value_new_error (ei->pos, gnumeric_err_NUM);
 
 	return value_new_float (sqrt(x));
 }
@@ -1478,11 +1491,11 @@ gnumeric_multinomial (FunctionEvalInfo *ei, GList *nodes)
 	p.sum = 0;
 	p.product = 1;
 
-	if (function_iterate_argument_values (&ei->pos,
+	if (function_iterate_argument_values (ei->pos,
 					      callback_function_multinomial,
 					      &p, nodes,
 					      TRUE) != NULL)
-	        return value_new_error (&ei->pos, gnumeric_err_VALUE);
+	        return value_new_error (ei->pos, gnumeric_err_VALUE);
 
 	return value_new_float (fact(p.sum) / p.product);
 }
@@ -1496,16 +1509,53 @@ static char *help_product = {
 	   "@DESCRIPTION="
 	   "PRODUCT returns the product of all the values and cells "
 	   "referenced in the argument list. "
-	   "This function is Excel compatible. "
+	   "This function is Excel compatible.  In particular, this means "
+	   "that if all cells are empty, the result will be 0."
 	   "\n"
 	   "@EXAMPLES=\n"
 	   "PRODUCT(2,5,9) equals 90.\n"
 	   "\n"
+	   "@SEEALSO=SUM, COUNT, G_PRODUCT")
+};
+
+static int
+range_bogusproduct (const float_t *xs, int n, float_t *res)
+{
+	if (n == 0) {
+		*res = 0;  /* Severe Excel brain damange.  */
+		return 0;
+	} else
+		return range_product (xs, n, res);
+}
+
+static Value *
+gnumeric_product (FunctionEvalInfo *ei, GList *nodes)
+{
+	return float_range_function (nodes, ei,
+				     range_bogusproduct,
+				     COLLECT_IGNORE_STRINGS |
+				     COLLECT_IGNORE_BOOLS,
+				     gnumeric_err_VALUE);
+}
+
+
+static char *help_g_product = {
+	N_("@FUNCTION=G_PRODUCT\n"
+	   "@SYNTAX=G_PRODUCT(value1, value2, ...)\n"
+
+	   "@DESCRIPTION="
+	   "PRODUCT returns the product of all the values and cells "
+	   "referenced in the argument list.  Empty cells are ignored "
+	   "and the empty product in 1."
+	   "\n"
+	   "@EXAMPLES=\n"
+	   "G_PRODUCT(2,5,9) equals 90.\n"
+	   "\n"
 	   "@SEEALSO=SUM, COUNT")
 };
 
 static Value *
-gnumeric_product (FunctionEvalInfo *ei, GList *nodes)
+gnumeric_g_product (FunctionEvalInfo *ei, GList *nodes)
 {
 	return float_range_function (nodes, ei,
 				     range_product,
@@ -1522,7 +1572,7 @@ static char *help_tan = {
 
 	   "@DESCRIPTION="
 	   "TAN function returns the tangent of @x, where @x is "
-	   "given in radians."
+	   "given in radians. "
 	   "This function is Excel compatible. "
 	   "\n"
 	   "@EXAMPLES=\n"
@@ -1569,7 +1619,7 @@ static char *help_pi = {
 	   "@DESCRIPTION="
 	   "PI functions returns the value of Pi. "
 	   "\n"
-	   "This function is called with no arguments."
+	   "This function is called with no arguments. "
 	   "This function is Excel compatible. "
 	   "\n"
 	   "@EXAMPLES=\n"
@@ -1593,7 +1643,7 @@ static char *help_trunc = {
 	   "@DESCRIPTION="
 	   "TRUNC function returns the value of @number "
 	   "truncated to the number of digits specified. "
-	   "If @digits is omitted then @digits defaults to zero."
+	   "If @digits is omitted then @digits defaults to zero. "
 	   "This function is Excel compatible. "
 	   "\n"
 	   "@EXAMPLES=\n"
@@ -1728,7 +1778,7 @@ gnumeric_factdouble (FunctionEvalInfo *ei, Value **argv)
 
 	number = value_get_as_int (argv[0]);
 	if (number < 0)
-		return value_new_error (&ei->pos, gnumeric_err_NUM );
+		return value_new_error (ei->pos, gnumeric_err_NUM );
 
 	for (n = number; n > 0; n-= 2)
 	        product *= n;
@@ -1822,7 +1872,7 @@ gnumeric_sqrtpi (FunctionEvalInfo *ei, Value **argv)
 
 	n = value_get_as_float (argv[0]);
 	if (n < 0)
-		return value_new_error (&ei->pos, gnumeric_err_NUM);
+		return value_new_error (ei->pos, gnumeric_err_NUM);
 
 	return value_new_float (sqrt (M_PI * n));
 }
@@ -1855,7 +1905,7 @@ gnumeric_randbetween (FunctionEvalInfo *ei, Value **argv)
 	bottom = value_get_as_int (argv[0]);
 	top    = value_get_as_int (argv[1]);
 	if (bottom > top)
-		return value_new_error (&ei->pos, gnumeric_err_NUM );
+		return value_new_error (ei->pos, gnumeric_err_NUM );
 
 	r = bottom + floor ((top + 1.0 - bottom) * random_01 ());
 	return value_new_int ((int)r);
@@ -2034,7 +2084,7 @@ gnumeric_mround (FunctionEvalInfo *ei, Value **argv)
 
 	if ((number > 0 && multiple < 0)
 	    || (number < 0 && multiple > 0))
-		return value_new_error (&ei->pos, gnumeric_err_NUM);
+		return value_new_error (ei->pos, gnumeric_err_NUM);
 
 	if (number < 0) {
 	        sign = -1;
@@ -2100,13 +2150,13 @@ gnumeric_roman (FunctionEvalInfo *ei, Value **argv)
 	        form = value_get_as_int (argv[1]);
 
 	if (n < 0 || n > 3999)
-		return value_new_error (&ei->pos, gnumeric_err_VALUE );
+		return value_new_error (ei->pos, gnumeric_err_VALUE );
 
 	if (n == 0)
 		return value_new_string ("");
 
 	if (form < 0 || form > 4)
-		return value_new_error (&ei->pos, gnumeric_err_NUM );
+		return value_new_error (ei->pos, gnumeric_err_NUM );
 
 	for (i = j = 0; dec > 1; dec /= 10, j+=2){
 	        for (; n > 0; i++){
@@ -2363,7 +2413,7 @@ gnumeric_sumx2my2 (FunctionEvalInfo *ei, Value **argv)
 
         if (values_x->type == VALUE_CELLRANGE) {
 		ret = sheet_cell_foreach_range (
-			eval_sheet (ei->pos.sheet, ei->pos.sheet),
+			eval_sheet (ei->pos->sheet, ei->pos->sheet),
 			TRUE,
 			values_x->v.cell_range.cell_a.col,
 			values_x->v.cell_range.cell_a.row,
@@ -2373,14 +2423,14 @@ gnumeric_sumx2my2 (FunctionEvalInfo *ei, Value **argv)
 			&items_x);
 		
 		if (ret != NULL)
-		        return value_new_error (&ei->pos, gnumeric_err_VALUE);
+		        return value_new_error (ei->pos, gnumeric_err_VALUE);
 	} else
-		return value_new_error (&ei->pos,
+		return value_new_error (ei->pos,
 					_("Array version not implemented!"));
 
         if (values_y->type == VALUE_CELLRANGE) {
 		ret = sheet_cell_foreach_range (
-			eval_sheet (ei->pos.sheet, ei->pos.sheet), TRUE,
+			eval_sheet (ei->pos->sheet, ei->pos->sheet), TRUE,
 			values_y->v.cell_range.cell_a.col,
 			values_y->v.cell_range.cell_a.row,
 			values_y->v.cell_range.cell_b.col,
@@ -2388,13 +2438,13 @@ gnumeric_sumx2my2 (FunctionEvalInfo *ei, Value **argv)
 			callback_function_sumxy,
 			&items_y);
 		if (ret != NULL)
-		        return value_new_error (&ei->pos, gnumeric_err_VALUE);
+		        return value_new_error (ei->pos, gnumeric_err_VALUE);
 	} else
-		return value_new_error (&ei->pos,
+		return value_new_error (ei->pos,
 					_("Array version not implemented!"));
 
 	if (items_x.num != items_y.num)
-		return value_new_error (&ei->pos, gnumeric_err_NA);
+		return value_new_error (ei->pos, gnumeric_err_NA);
 
 	list1 = items_x.list;
 	list2 = items_y.list;
@@ -2462,7 +2512,7 @@ gnumeric_sumx2py2 (FunctionEvalInfo *ei, Value **argv)
 
         if (values_x->type == VALUE_CELLRANGE) {
 		ret = sheet_cell_foreach_range (
-			eval_sheet (ei->pos.sheet, ei->pos.sheet),
+			eval_sheet (ei->pos->sheet, ei->pos->sheet),
 			TRUE,
 			values_x->v.cell_range.cell_a.col,
 			values_x->v.cell_range.cell_a.row,
@@ -2471,14 +2521,14 @@ gnumeric_sumx2py2 (FunctionEvalInfo *ei, Value **argv)
 			callback_function_sumxy,
 			&items_x);
 		if (ret != NULL)
-		        return value_new_error (&ei->pos, gnumeric_err_VALUE);
+		        return value_new_error (ei->pos, gnumeric_err_VALUE);
 	} else
-		return value_new_error (&ei->pos,
+		return value_new_error (ei->pos,
 					_("Array version not implemented!"));
 
         if (values_y->type == VALUE_CELLRANGE) {
 		ret = sheet_cell_foreach_range (
-			eval_sheet (ei->pos.sheet, ei->pos.sheet), TRUE,
+			eval_sheet (ei->pos->sheet, ei->pos->sheet), TRUE,
 			values_y->v.cell_range.cell_a.col,
 			values_y->v.cell_range.cell_a.row,
 			values_y->v.cell_range.cell_b.col,
@@ -2486,13 +2536,13 @@ gnumeric_sumx2py2 (FunctionEvalInfo *ei, Value **argv)
 			callback_function_sumxy,
 			&items_y);
 		if (ret != NULL)
-		        return value_new_error (&ei->pos, gnumeric_err_VALUE);
+		        return value_new_error (ei->pos, gnumeric_err_VALUE);
 	} else
-		return value_new_error (&ei->pos,
+		return value_new_error (ei->pos,
 					_("Array version not implemented!"));
 
 	if (items_x.num != items_y.num)
-		return value_new_error (&ei->pos, gnumeric_err_NA);
+		return value_new_error (ei->pos, gnumeric_err_NA);
 
 	list1 = items_x.list;
 	list2 = items_y.list;
@@ -2558,7 +2608,7 @@ gnumeric_sumxmy2 (FunctionEvalInfo *ei, Value **argv)
 
         if (values_x->type == VALUE_CELLRANGE) {
 		ret = sheet_cell_foreach_range (
-			eval_sheet (ei->pos.sheet, ei->pos.sheet), TRUE,
+			eval_sheet (ei->pos->sheet, ei->pos->sheet), TRUE,
 			values_x->v.cell_range.cell_a.col,
 			values_x->v.cell_range.cell_a.row,
 			values_x->v.cell_range.cell_b.col,
@@ -2566,14 +2616,14 @@ gnumeric_sumxmy2 (FunctionEvalInfo *ei, Value **argv)
 			callback_function_sumxy,
 			&items_x);
 		if (ret != NULL)
-		        return value_new_error (&ei->pos, gnumeric_err_VALUE);
+		        return value_new_error (ei->pos, gnumeric_err_VALUE);
 	} else
-		return value_new_error (&ei->pos,
+		return value_new_error (ei->pos,
 					_("Array version not implemented!"));
 
         if (values_y->type == VALUE_CELLRANGE) {
 		ret = sheet_cell_foreach_range (
-			eval_sheet (ei->pos.sheet, ei->pos.sheet), TRUE,
+			eval_sheet (ei->pos->sheet, ei->pos->sheet), TRUE,
 			values_y->v.cell_range.cell_a.col,
 			values_y->v.cell_range.cell_a.row,
 			values_y->v.cell_range.cell_b.col,
@@ -2581,13 +2631,13 @@ gnumeric_sumxmy2 (FunctionEvalInfo *ei, Value **argv)
 			callback_function_sumxy,
 			&items_y);
 		if (ret != NULL)
-		        return value_new_error (&ei->pos, gnumeric_err_VALUE);
+		        return value_new_error (ei->pos, gnumeric_err_VALUE);
 	} else
-		return value_new_error (&ei->pos,
+		return value_new_error (ei->pos,
 					_("Array version not implemented!"));
 
 	if (items_x.num != items_y.num)
-	        return value_new_error (&ei->pos, gnumeric_err_NA);
+	        return value_new_error (ei->pos, gnumeric_err_NA);
 
 	list1 = items_x.list;
 	list2 = items_y.list;
@@ -2654,23 +2704,23 @@ gnumeric_subtotal (FunctionEvalInfo *ei, GList *expr_node_list)
 	int      fun_nbr;
 
 	if (expr_node_list == NULL)
-		return value_new_error (&ei->pos, gnumeric_err_NUM);
+		return value_new_error (ei->pos, gnumeric_err_NUM);
 
 	tree = (ExprTree *) expr_node_list->data;
 	if (tree == NULL)
-		return value_new_error (&ei->pos, gnumeric_err_NUM);
+		return value_new_error (ei->pos, gnumeric_err_NUM);
 
-	val = eval_expr (ei, tree);
+	val = eval_expr (ei->pos, tree);
 	if (!val) return NULL;
 	if (!VALUE_IS_NUMBER (val)) {
 		value_release (val);
-		return value_new_error (&ei->pos, gnumeric_err_VALUE);
+		return value_new_error (ei->pos, gnumeric_err_VALUE);
 	}
 
 	fun_nbr = value_get_as_int(val);
 	value_release (val);
 	if (fun_nbr < 1 || fun_nbr > 11)
-		return value_new_error (&ei->pos, gnumeric_err_NUM);
+		return value_new_error (ei->pos, gnumeric_err_NUM);
 
 	/* Skip the first node */
 	expr_node_list = expr_node_list->next;
@@ -2759,18 +2809,18 @@ gnumeric_seriessum (FunctionEvalInfo *ei, GList *nodes)
 	float_t          x, n, m;
 
 	if (nodes == NULL)
-		return value_new_error (&ei->pos, gnumeric_err_NUM);
+		return value_new_error (ei->pos, gnumeric_err_NUM);
 
 	/* Get x */
 	tree = (ExprTree *) nodes->data;
 	if (tree == NULL)
-		return value_new_error (&ei->pos, gnumeric_err_NUM);
+		return value_new_error (ei->pos, gnumeric_err_NUM);
 
-	val = eval_expr (ei, tree);
+	val = eval_expr (ei->pos, tree);
 	if (!val) return NULL;
 	if (! VALUE_IS_NUMBER(val)) {
 		value_release (val);
-		return value_new_error (&ei->pos, gnumeric_err_VALUE);
+		return value_new_error (ei->pos, gnumeric_err_VALUE);
 	}
 
 	x = value_get_as_float (val);
@@ -2780,13 +2830,13 @@ gnumeric_seriessum (FunctionEvalInfo *ei, GList *nodes)
 	/* Get n */
 	tree = (ExprTree *) nodes->data;
 	if (tree == NULL)
-		return value_new_error (&ei->pos, gnumeric_err_NUM);
+		return value_new_error (ei->pos, gnumeric_err_NUM);
 
-	val = eval_expr (ei, tree);
+	val = eval_expr (ei->pos, tree);
 	if (!val) return NULL;
 	if (! VALUE_IS_NUMBER(val)) {
 		value_release (val);
-		return value_new_error (&ei->pos, gnumeric_err_VALUE);
+		return value_new_error (ei->pos, gnumeric_err_VALUE);
 	}
 
 	n = value_get_as_int(val);
@@ -2794,18 +2844,18 @@ gnumeric_seriessum (FunctionEvalInfo *ei, GList *nodes)
 
 	nodes = nodes->next;
 	if (nodes == NULL)
-		return value_new_error (&ei->pos, gnumeric_err_NUM);
+		return value_new_error (ei->pos, gnumeric_err_NUM);
 
 	/* Get m */
 	tree = (ExprTree *) nodes->data;
 	if (tree == NULL)
-		return value_new_error (&ei->pos, gnumeric_err_NUM);
+		return value_new_error (ei->pos, gnumeric_err_NUM);
 
-	val = eval_expr (ei, tree);
+	val = eval_expr (ei->pos, tree);
 	if (!val) return NULL;
 	if (! VALUE_IS_NUMBER(val)) {
 		value_release (val);
-		return value_new_error (&ei->pos, gnumeric_err_VALUE);
+		return value_new_error (ei->pos, gnumeric_err_VALUE);
 	}
 
 	m = value_get_as_float(val);
@@ -2817,11 +2867,11 @@ gnumeric_seriessum (FunctionEvalInfo *ei, GList *nodes)
 	p.x = x;
 	p.sum = 0;
 
-	if (function_iterate_argument_values (&ei->pos,
+	if (function_iterate_argument_values (ei->pos,
 					      callback_function_seriessum,
 					      &p, nodes,
 					      TRUE) != NULL)
-		return value_new_error (&ei->pos, gnumeric_err_VALUE);
+		return value_new_error (ei->pos, gnumeric_err_VALUE);
 
 	return value_new_float (p.sum);
 }
@@ -2837,7 +2887,7 @@ static char *help_minverse = {
 	   "\n"
 	   "If the matrix cannot be inverted, MINVERSE returns #NUM! error. "
 	   "If the matrix does not contain equal number of columns and "
-	   "rows, MINVERSE returns #VALUE! error."
+	   "rows, MINVERSE returns #VALUE! error. "
 	   "This function is Excel compatible. "
 	   "\n"
 	   "@EXAMPLES=\n"
@@ -2956,7 +3006,7 @@ minverse(float_t *array, int cols, int rows)
 static Value *
 gnumeric_minverse (FunctionEvalInfo *ei, Value **argv)
 {
-	EvalPosition const * const ep = &ei->pos;
+	EvalPosition const * const ep = ei->pos;
 
 	int	r, rows;
 	int	c, cols;
@@ -2968,12 +3018,12 @@ gnumeric_minverse (FunctionEvalInfo *ei, Value **argv)
 
 	if (validate_range_numeric_matrix (ep, values, &rows, &cols,
 					   &error_string)) {
-		return value_new_error (&ei->pos, error_string);
+		return value_new_error (ei->pos, error_string);
 	}
 
 	/* Guarantee shape and non-zero size */
 	if (cols != rows || !rows || !cols)
-		return value_new_error (&ei->pos, gnumeric_err_VALUE);
+		return value_new_error (ei->pos, gnumeric_err_VALUE);
 
 	matrix = g_new (float_t, rows*cols*2);
 	for (c=0; c<cols; c++)
@@ -2989,7 +3039,7 @@ gnumeric_minverse (FunctionEvalInfo *ei, Value **argv)
 
 	if (minverse(matrix, cols*2, rows)) {
 	        g_free (matrix);
-		return value_new_error (&ei->pos, gnumeric_err_NUM);
+		return value_new_error (ei->pos, gnumeric_err_NUM);
 	}
 
 	res = g_new (Value, 1);
@@ -3021,7 +3071,7 @@ static char *help_mmult = {
 	   "@DESCRIPTION="
 	   "MMULT function returns the matrix product of two arrays. The "
 	   "result is an array with the same number of rows as @array1 and the "
-	   "same number of columns as @array2."
+	   "same number of columns as @array2. "
 	   "This function is Excel compatible. "
 	   "\n"
 	   "@EXAMPLES=\n"
@@ -3033,7 +3083,7 @@ static char *help_mmult = {
 static Value *
 gnumeric_mmult (FunctionEvalInfo *ei, Value **argv)
 {
-	EvalPosition const * const ep = &ei->pos;
+	EvalPosition const * const ep = ei->pos;
 	int	r, rows_a, rows_b, i;
 	int	c, cols_a, cols_b;
         Value *res;
@@ -3045,12 +3095,12 @@ gnumeric_mmult (FunctionEvalInfo *ei, Value **argv)
 					   &error_string) ||
 	    validate_range_numeric_matrix (ep, values_b, &rows_b, &cols_b,
 					   &error_string)){
-		return value_new_error (&ei->pos, error_string);
+		return value_new_error (ei->pos, error_string);
 	}
 
 	/* Guarantee shape and non-zero size */
 	if (cols_a != rows_b || !rows_a || !rows_b || !cols_a || !cols_b)
-		return value_new_error (&ei->pos, gnumeric_err_VALUE);
+		return value_new_error (ei->pos, gnumeric_err_VALUE);
 
 	res = g_new (Value, 1);
 	res->type = VALUE_ARRAY;
@@ -3088,7 +3138,7 @@ static char *help_mdeterm = {
 	   "MDETERM function returns the determinant of a given matrix. "
 	   "\n"
 	   "If the matrix does not contain equal number of columns and "
-	   "rows, MDETERM returns #VALUE! error."
+	   "rows, MDETERM returns #VALUE! error. "
 	   "This function is Excel compatible. "
 	   "\n"
 	   "@EXAMPLES=\n"
@@ -3171,7 +3221,7 @@ mdeterm(float_t *A, int cols)
 static Value *
 gnumeric_mdeterm (FunctionEvalInfo *ei, Value **argv)
 {
-	EvalPosition const * const ep = &ei->pos;
+	EvalPosition const * const ep = ei->pos;
 
 	int	r, rows;
 	int	c, cols;
@@ -3183,12 +3233,12 @@ gnumeric_mdeterm (FunctionEvalInfo *ei, Value **argv)
 
 	if (validate_range_numeric_matrix (ep, values, &rows, &cols,
 					   &error_string)) {
-		return value_new_error (&ei->pos, error_string);
+		return value_new_error (ei->pos, error_string);
 	}
 
 	/* Guarantee shape and non-zero size */
 	if (cols != rows || !rows || !cols)
-		return value_new_error (&ei->pos, gnumeric_err_VALUE);
+		return value_new_error (ei->pos, gnumeric_err_VALUE);
 
 	matrix = g_new (float_t, rows*cols);
 	for (c=0; c<cols; c++)
@@ -3270,7 +3320,7 @@ gnumeric_sumproduct (FunctionEvalInfo *ei, GList *expr_node_list)
 	Value            *result=NULL;
 
 	if (expr_node_list == NULL)
-		return value_new_error (&ei->pos, gnumeric_err_NUM);
+		return value_new_error (ei->pos, gnumeric_err_NUM);
 
 	p.components = NULL;
 	p.first      = TRUE;
@@ -3280,12 +3330,12 @@ gnumeric_sumproduct (FunctionEvalInfo *ei, GList *expr_node_list)
 		ExprTree *tree = (ExprTree *) expr_node_list->data;
 		Value    *val;
 
-		val = eval_expr (ei, tree);
+		val = eval_expr (ei->pos, tree);
 
 		if (val) {
 		        p.current = p.components;
 			result = function_iterate_do_value (
-				&ei->pos, callback_function_sumproduct, &p,
+				ei->pos, callback_function_sumproduct, &p,
 				val, TRUE);
 
 			value_release (val);
@@ -3304,7 +3354,7 @@ gnumeric_sumproduct (FunctionEvalInfo *ei, GList *expr_node_list)
 	g_slist_free(p.components);
 
 	if (expr_node_list)
-		return value_new_error (&ei->pos, gnumeric_err_VALUE);
+		return value_new_error (ei->pos, gnumeric_err_VALUE);
 
 	return value_new_float (sum);
 }
@@ -3314,10 +3364,13 @@ gnumeric_sumproduct (FunctionEvalInfo *ei, GList *expr_node_list)
 void
 math_functions_init (void)
 {
+	FunctionDefinition *def;
 	FunctionCategory *cat = function_get_category (_("Maths / Trig."));
 
-	function_add_args  (cat, "abs",     "f",
-			    "number",    &help_abs,      gnumeric_abs);
+	def = function_add_args  (cat, "abs",     "f",
+				  "number",    &help_abs,      gnumeric_abs);
+	auto_format_function_result (def, AF_FIRST_ARG_FORMAT);
+
 	function_add_args  (cat, "acos",    "f",
 			    "number",    &help_acos,     gnumeric_acos);
 	function_add_args  (cat, "acosh",   "f",
@@ -3339,11 +3392,16 @@ math_functions_init (void)
 	function_add_args  (cat, "countif", "r?",
 			    "range,criteria",
 			    &help_countif,     gnumeric_countif);
-	function_add_args  (cat, "ceil",    "f",
-			    "number",    &help_ceil,     gnumeric_ceil);
-	function_add_args  (cat, "ceiling", "ff",
-			    "number,significance",
-			    &help_ceiling,     gnumeric_ceiling);
+
+	def = function_add_args  (cat, "ceil",    "f",
+				  "number",    &help_ceil,     gnumeric_ceil);
+	auto_format_function_result (def, AF_FIRST_ARG_FORMAT);
+
+	def = function_add_args  (cat, "ceiling", "ff",
+				  "number,significance",
+				  &help_ceiling,     gnumeric_ceiling);
+	auto_format_function_result (def, AF_FIRST_ARG_FORMAT);
+
 	function_add_args  (cat, "degrees", "f",
 			    "number",    &help_degrees,  gnumeric_degrees);
 	function_add_args  (cat, "even",    "f",
@@ -3357,13 +3415,19 @@ math_functions_init (void)
 			    &help_factdouble,  gnumeric_factdouble);
 	function_add_args  (cat, "combin",  "ff",
 			    "n,k",       &help_combin,   gnumeric_combin);
-	function_add_args  (cat, "floor",   "f|f",
-			    "number",    &help_floor,    gnumeric_floor);
+
+	def = function_add_args  (cat, "floor",   "f|f",
+				  "number",    &help_floor,    gnumeric_floor);
+	auto_format_function_result (def, AF_FIRST_ARG_FORMAT);
+
 	function_add_nodes (cat, "gcd",     "ff",
 			    "number1,number2",
 			    &help_gcd,         gnumeric_gcd);
-	function_add_args  (cat, "int",     "f",
-			    "number",    &help_int,      gnumeric_int);
+
+	def = function_add_args  (cat, "int",     "f",
+				  "number",    &help_int,      gnumeric_int);
+	auto_format_function_result (def, AF_FIRST_ARG_FORMAT);
+
 	function_add_nodes (cat, "lcm",     0,
 			    "",          &help_lcm,      gnumeric_lcm);
 	function_add_args  (cat, "ln",      "f",
@@ -3377,9 +3441,12 @@ math_functions_init (void)
 			    "number",    &help_log10,    gnumeric_log10);
 	function_add_args  (cat, "mod",     "ff",
 			    "num,denom", &help_mod,      gnumeric_mod);
-	function_add_args  (cat, "mround",  "ff",
-			    "number,multiple",
-			    &help_mround,      gnumeric_mround);
+
+	def = function_add_args  (cat, "mround",  "ff",
+				  "number,multiple",
+				  &help_mround,      gnumeric_mround);
+	auto_format_function_result (def, AF_FIRST_ARG_FORMAT);
+
 	function_add_nodes (cat, "multinomial", 0,
 			    "",          
 			    &help_multinomial, gnumeric_multinomial);
@@ -3389,6 +3456,8 @@ math_functions_init (void)
 			    "x,y",       &help_power,    gnumeric_power);
 	function_add_nodes (cat, "product", 0,
 			    "number",    &help_product,  gnumeric_product);
+	function_add_nodes (cat, "g_product", 0,
+			    "number",    &help_g_product,  gnumeric_g_product);
 	function_add_args  (cat, "quotient" , "ff",
 			    "num,den",   &help_quotient, gnumeric_quotient);
 	function_add_args  (cat, "radians", "f",
@@ -3402,15 +3471,22 @@ math_functions_init (void)
 	function_add_args  (cat, "roman",      "f|f",
 			    "number[,type]",
 			    &help_roman,       gnumeric_roman);
-	function_add_args  (cat, "round",      "f|f",
-			    "number[,digits]",
-			    &help_round,       gnumeric_round);
-	function_add_args  (cat, "rounddown",  "f|f",
-			    "number,digits",
-			    &help_rounddown,   gnumeric_rounddown);
-	function_add_args  (cat, "roundup",    "f|f",
-			    "number,digits",
-			    &help_roundup,     gnumeric_roundup);
+
+	def = function_add_args  (cat, "round",      "f|f",
+				  "number[,digits]",
+				  &help_round,       gnumeric_round);
+	auto_format_function_result (def, AF_FIRST_ARG_FORMAT);
+
+	def = function_add_args  (cat, "rounddown",  "f|f",
+				  "number,digits",
+				  &help_rounddown,   gnumeric_rounddown);
+	auto_format_function_result (def, AF_FIRST_ARG_FORMAT);
+
+	def = function_add_args  (cat, "roundup",    "f|f",
+				  "number,digits",
+				  &help_roundup,     gnumeric_roundup);
+	auto_format_function_result (def, AF_FIRST_ARG_FORMAT);
+
 	function_add_nodes (cat, "seriessum", 0,
 			    "x,n,m,coefficients",
 			    &help_seriessum,   gnumeric_seriessum);
@@ -3427,12 +3503,17 @@ math_functions_init (void)
 	function_add_nodes (cat, "subtotal", 0,
 			    "function_nbr,ref1,ref2,...",
 			    &help_subtotal,    gnumeric_subtotal);
-	function_add_nodes (cat, "sum",     0,
-			    "number1,number2,...",
-			    &help_sum,	       gnumeric_sum);
-	function_add_nodes (cat, "suma",    0,
-			    "number1,number2,...",
-			    &help_suma,	       gnumeric_suma);
+
+	def = function_add_nodes (cat, "sum",     0,
+				  "number1,number2,...",
+				  &help_sum,	       gnumeric_sum);
+	auto_format_function_result (def, AF_FIRST_ARG_FORMAT);
+
+	def = function_add_nodes (cat, "suma",    0,
+				  "number1,number2,...",
+				  &help_suma,	       gnumeric_suma);
+	auto_format_function_result (def, AF_FIRST_ARG_FORMAT);
+
 	function_add_args  (cat, "sumif",   "r?|r",
 			    "range,criteria[,actual_range]",
 			    &help_sumif,       gnumeric_sumif);
@@ -3454,9 +3535,12 @@ math_functions_init (void)
 			    "number",    &help_tan,      gnumeric_tan);
 	function_add_args  (cat, "tanh",    "f",
 			    "number",    &help_tanh,     gnumeric_tanh);
-	function_add_args  (cat, "trunc",   "f|f",
-			    "number,digits",
-			    &help_trunc,       gnumeric_trunc);
+
+	def = function_add_args  (cat, "trunc",   "f|f",
+				  "number,digits",
+				  &help_trunc,       gnumeric_trunc);
+	auto_format_function_result (def, AF_FIRST_ARG_FORMAT);
+
 	function_add_args  (cat, "pi",      "",
 			    "",          &help_pi,       gnumeric_pi);
 	function_add_args  (cat, "mmult",   "AA",
