@@ -19,7 +19,7 @@ static char *help_selection = {
 
 	   "@DESCRIPTION="
 	   "The SELECTION function returns a list with the values in the current selection. "
-	   "This is usually used to implement on-the-fly computation of values."
+	   "This is usually used to implement on-the-fly computation of values. "
 	   "If @permit_intersection is TRUE the user specifed selection "
 	   "ranges are returned, EVEN IF THEY OVERLAP.  If @permit_intersection is FALSE "
 	   "a distict set of regions is returned, however, there may be more of them than "
@@ -72,7 +72,7 @@ accumulate_regions (Sheet *sheet,
 static Value *
 gnumeric_selection (FunctionEvalInfo *ei, Value *argv [])
 {
-	Sheet * const sheet = ei->pos.sheet;
+	Sheet * const sheet = ei->pos->sheet;
 	gboolean const permit_intersection = argv [0]->v.v_bool;
 	Value * res;
 	int i;
