@@ -12,6 +12,7 @@
 #include <string.h>
 #include "gnumeric.h"
 #include "gnumeric-sheet.h"
+#include "gnumeric-util.h"
 #include "sheet-object.h"
 #include "color.h"
 #include "cursors.h"
@@ -21,6 +22,7 @@
 #include "application.h"
 #include "workbook-view.h"
 #include "workbook.h"
+#include "commands.h"
 
 #undef DEBUG_POSITIONS
 
@@ -154,13 +156,27 @@ static void
 move_cursor (GnumericSheet *gsheet, int col, int row, gboolean clear_selection)
 {
 	Sheet *sheet = gsheet->sheet_view->sheet;
+
+	/*
+	 * Please note that the order here is important, as
+	 * the sheet_make_cell_visible call might scroll the
+	 * canvas, you should do all of your screen changes
+	 * in an atomic fashion.
+	 *
+	 * The code at some point did do the selection change
+	 * after the sheet moved, causing flicker -mig
+	 *
+	 * If you dont know what this means, just mail me.
+	 */
+	 
+	if (clear_selection)
+		sheet_selection_reset_only (sheet);
+
 	sheet_cursor_set (sheet, col, row, col, row, col, row);
 	sheet_make_cell_visible (sheet, col, row);
 
-	if (clear_selection) {
-		sheet_selection_reset_only (sheet);
+	if (clear_selection)
 		sheet_selection_append (sheet, col, row);
-	}
 }
 
 void
@@ -169,7 +185,7 @@ gnumeric_sheet_move_cursor (GnumericSheet *gsheet, int col, int row)
 	g_return_if_fail (gsheet != NULL);
 	g_return_if_fail (GNUMERIC_IS_SHEET (gsheet));
 
-	move_cursor (gsheet, col, row, 1);
+	move_cursor (gsheet, col, row, TRUE);
 }
 
 void
@@ -266,7 +282,7 @@ gnumeric_sheet_can_move_cursor (GnumericSheet *gsheet)
 	entry = GTK_ENTRY (gsheet->entry);
 	cursor_pos = GTK_EDITABLE (entry)->current_pos;
 
-	if (entry->text [0] != '=')
+	if (!gnumeric_char_start_expr_p (entry->text[0]))
 		return FALSE;
 	if (cursor_pos == 0)
 		return FALSE;
@@ -747,16 +763,20 @@ gnumeric_sheet_key_mode_sheet (GnumericSheet *gsheet, GdkEventKey *event)
 	case GDK_Page_Up:
 		if ((event->state & GDK_CONTROL_MASK) != 0)
 			gtk_notebook_prev_page (GTK_NOTEBOOK (wb->notebook));
-		else
+		else if ((event->state & GDK_MOD1_MASK) == 0)
 			(*movefn_vertical)(gsheet, -(gsheet->last_visible_row-gsheet->top_row), FALSE);
+		else
+			(*movefn_horizontal)(gsheet, -(gsheet->last_visible_col-gsheet->left_col), FALSE);
 		break;
 
 	case GDK_KP_Page_Down:
 	case GDK_Page_Down:
 		if ((event->state & GDK_CONTROL_MASK) != 0)
 			gtk_notebook_next_page (GTK_NOTEBOOK (wb->notebook));
-		else
+		else if ((event->state & GDK_MOD1_MASK) == 0)
 			(*movefn_vertical)(gsheet, gsheet->last_visible_row-gsheet->top_row, FALSE);
+		else
+			(*movefn_horizontal)(gsheet, gsheet->last_visible_col-gsheet->left_col, FALSE);
 		break;
 
 	case GDK_KP_Home:
@@ -771,7 +791,7 @@ gnumeric_sheet_key_mode_sheet (GnumericSheet *gsheet, GdkEventKey *event)
 
 	case GDK_KP_Delete:
 	case GDK_Delete:
-		sheet_selection_clear_content (workbook_command_context_gui (wb), sheet);
+		cmd_clear_selection (workbook_command_context_gui (wb), sheet, CLEAR_VALUES);
 		break;
 
 	case GDK_KP_Enter:
@@ -784,6 +804,12 @@ gnumeric_sheet_key_mode_sheet (GnumericSheet *gsheet, GdkEventKey *event)
 				    (event->state & GDK_SHIFT_MASK);
 
 				sheet_accept_pending_input (sheet);
+				/* An undo record is placed on the stack
+				 * for the pseudo SetText associated with the
+				 * corner cell.
+				 */
+				command_list_pop_top_undo (wb);
+
 				cell = sheet_cell_get (sheet,
 						       sheet->cursor_col,
 						       sheet->cursor_row);
@@ -796,8 +822,8 @@ gnumeric_sheet_key_mode_sheet (GnumericSheet *gsheet, GdkEventKey *event)
 				 */
 				g_return_val_if_fail (cell != NULL, 1);
 				text = cell_get_text (cell);
-				sheet_fill_selection_with (sheet, text,
-							   is_array);
+				sheet_fill_selection_with (workbook_command_context_gui (wb),
+							   sheet, text, is_array);
 				g_free (text);
 			} else {
 				gtk_widget_grab_focus (gsheet->entry);
@@ -1056,25 +1082,25 @@ gnumeric_sheet_compute_visible_ranges (GnumericSheet *gsheet)
 	width = GTK_WIDGET (canvas)->allocation.width;
 
 	do {
-		ColRowInfo *ci;
-		int cb;
+		ColRowInfo const * const ci = sheet_col_get_info (gsheet->sheet_view->sheet, col);
+		int const tmp = ci->pixels;
+		if (tmp > 0) {
+			int const cb = pixels + ci->pixels;
 
-		ci = sheet_col_get_info (gsheet->sheet_view->sheet, col);
-		cb = pixels + ci->pixels;
-
-		if (cb == width){
-			gsheet->last_visible_col = col;
-			gsheet->last_full_col = col;
-			break;
-		} if (cb > width){
-			gsheet->last_visible_col = col;
-			if (col == gsheet->left_col)
-				gsheet->last_full_col = gsheet->left_col;
-			else
-				gsheet->last_full_col = col - 1;
-			break;
+			if (cb == width){
+				gsheet->last_visible_col = col;
+				gsheet->last_full_col = col;
+				break;
+			} if (cb > width){
+				gsheet->last_visible_col = col;
+				if (col == gsheet->left_col)
+					gsheet->last_full_col = gsheet->left_col;
+				else
+					gsheet->last_full_col = col - 1;
+				break;
+			}
+			pixels = cb;
 		}
-		pixels = cb;
 		++col;
 	} while (pixels < width && col < SHEET_MAX_COLS);
 
@@ -1088,25 +1114,25 @@ gnumeric_sheet_compute_visible_ranges (GnumericSheet *gsheet)
 	row = gsheet->top_row;
 	height = GTK_WIDGET (canvas)->allocation.height;
 	do {
-		ColRowInfo *ri;
-		int cb;
+		ColRowInfo const * const ri = sheet_row_get_info (gsheet->sheet_view->sheet, row);
+		int const tmp = ri->pixels;
+		if (tmp > 0) {
+			int const cb = pixels + ri->pixels;
 
-		ri = sheet_row_get_info (gsheet->sheet_view->sheet, row);
-		cb = pixels + ri->pixels;
-
-		if (cb == height){
-			gsheet->last_visible_row = row;
-			gsheet->last_full_row = row;
-			break;
-		} if (cb > height){
-			gsheet->last_visible_row = row;
-			if (row == gsheet->top_row)
-				gsheet->last_full_row = gsheet->top_row;
-			else
-				gsheet->last_full_row = row - 1;
-			break;
+			if (cb == height){
+				gsheet->last_visible_row = row;
+				gsheet->last_full_row = row;
+				break;
+			} if (cb > height){
+				gsheet->last_visible_row = row;
+				if (row == gsheet->top_row)
+					gsheet->last_full_row = gsheet->top_row;
+				else
+					gsheet->last_full_row = row - 1;
+				break;
+			}
+			pixels = cb;
 		}
-		pixels = cb;
 		row++;
 	} while (pixels < height && row < SHEET_MAX_ROWS);
 
@@ -1219,17 +1245,18 @@ gnumeric_sheet_make_cell_visible (GnumericSheet *gsheet, int col, int row)
 	if (col < gsheet->left_col){
 		new_left_col = col;
 	} else if (col > gsheet->last_full_col){
-		ColRowInfo *ci;
 		int width = GTK_WIDGET (canvas)->allocation.width;
 		int allocated = 0;
 		int first_col;
 
 		for (first_col = col; first_col > 0; first_col--){
-			ci = sheet_col_get_info (sheet, first_col);
-
-			if (allocated + ci->pixels > width)
-				break;
-			allocated += ci->pixels;
+			ColRowInfo const * const ci = sheet_col_get_info (sheet, first_col);
+			int const tmp = ci->pixels;
+			if (tmp > 0) {
+				if (allocated + ci->pixels > width)
+					break;
+				allocated += ci->pixels;
+			}
 		}
 		new_left_col = first_col+1;
 	} else
@@ -1239,17 +1266,18 @@ gnumeric_sheet_make_cell_visible (GnumericSheet *gsheet, int col, int row)
 	if (row < gsheet->top_row){
 		new_top_row = row;
 	} else if (row > gsheet->last_full_row){
-		ColRowInfo *ri;
 		int height = GTK_WIDGET (canvas)->allocation.height;
 		int allocated = 0;
 		int first_row;
 
 		for (first_row = row; first_row > 0; first_row--){
-			ri = sheet_row_get_info (sheet, first_row);
-
-			if (allocated + ri->pixels > height)
-				break;
-			allocated += ri->pixels;
+			ColRowInfo const * const ri = sheet_row_get_info (sheet, first_row);
+			int const tmp = ri->pixels;
+			if (tmp > 0) {
+				if (allocated + ri->pixels > height)
+					break;
+				allocated += ri->pixels;
+			}
 		}
 		new_top_row = first_row+1;
 	} else
