@@ -6,6 +6,8 @@
 void       gnumeric_notice (Workbook *wb, const char *type, const char *str);
 void       gnumeric_no_modify_array_notice (Workbook *wb);
 
+gint       gnumeric_dialog_run (Workbook *wb, GnomeDialog *dialog);
+
 /* A simple routine for making a popup menu */
 int        run_popup_menu  (GdkEvent *event, int button, char **strings);
 
@@ -24,9 +26,12 @@ void      gnumeric_auto_kill_popup_menu_on_hide (GtkMenu *menu);
 void      gnumeric_popup_menu                   (GtkMenu *menu, GdkEventButton *event);
 
 /*
- * Some tool tip support code.
+ * Pseudo-tool-tip support code.
  */
 void        gnumeric_position_tooltip (GtkWidget *tip, int horizontal);
 GtkWidget * gnumeric_create_tooltip (void);
 
+/* Is this character the start of an expression */
+gboolean    gnumeric_char_start_expr_p (char const c);
+
 #endif /* GNUMERIC_GNUMERIC_UTIL_H */
