@@ -39,11 +39,12 @@ enum {
 	ARG_STYLE,              /* The style type */
 };
 
-static void
+static int
 item_cursor_animation_callback (ItemCursor *item_cursor)
 {
 	item_cursor->state = !item_cursor->state;
 	item_cursor_request_redraw (item_cursor);
+	return TRUE;
 }
 
 static void
@@ -379,9 +380,12 @@ item_cursor_request_redraw (ItemCursor *item_cursor)
 {
 	GnomeCanvas *canvas = GNOME_CANVAS_ITEM (item_cursor)->canvas;
 	int x, y, w, h;
-
 	item_cursor_get_pixel_coords (item_cursor, &x, &y, &w, &h);
-	gnome_canvas_request_redraw (canvas, x - 2, y - 2, x + w + 5, y + h + 5);
+
+	gnome_canvas_request_redraw (canvas, x - 2, y - 2, x + 2, y + h + 2);
+	gnome_canvas_request_redraw (canvas, x - 2, y - 2, x + w + 2, y + 2);
+	gnome_canvas_request_redraw (canvas, x + w - 2, y - 2, x + w + 5, y + h + 5);
+	gnome_canvas_request_redraw (canvas, x - 2, y + h - 2, x + w + 5, y + h + 5);
 }
 
 void
@@ -585,7 +589,7 @@ item_cursor_target_region_ok (ItemCursor *item_cursor)
 		NULL);
 	window = gtk_widget_get_toplevel (GTK_WIDGET (canvas));
 	gnome_dialog_set_parent (GNOME_DIALOG (message), GTK_WINDOW (window));
-	v = gnome_dialog_run (GNOME_DIALOG (message));
+	v = gnumeric_dialog_run (item_cursor->sheet->workbook, GNOME_DIALOG (message));
 
 	if (v == 0)
 		return TRUE;
