@@ -23,6 +23,7 @@
 #include "workbook-cmd-format.h"
 #include "pattern.h"
 #include "workbook-view.h"
+#include "commands.h"
 
 static GnomeCanvasItemClass *item_grid_parent_class;
 
@@ -76,7 +77,7 @@ item_grid_realize (GnomeCanvasItem *item)
 	item_grid->fill_gc = gdk_gc_new (window);
 	item_grid->gc = gdk_gc_new (window);
 	item_grid->empty_gc = gdk_gc_new (window);
-		
+
 	/* Allocate the default colors */
 	item_grid->background = gs_white;
 	item_grid->grid_color = gs_light_gray;
@@ -153,14 +154,18 @@ item_grid_find_col (ItemGrid *item_grid, int x, int *col_origin)
 
 	do {
 		ColRowInfo *ci = sheet_col_get_info (item_grid->sheet, col);
+		int const tmp = ci->pixels;
 
-		if (x >= pixel && x <= pixel + ci->pixels) {
-			if (col_origin)
-				*col_origin = pixel;
-			return col;
+		if (tmp > 0) {
+			if (x >= pixel && x <= pixel + tmp) {
+				if (col_origin)
+					*col_origin = pixel;
+				return col;
+			}
+			pixel += tmp;
 		}
+
 		col++;
-		pixel += ci->pixels;
 	} while (col < SHEET_MAX_COLS);
 
 	return SHEET_MAX_COLS-1;
@@ -184,14 +189,17 @@ item_grid_find_row (ItemGrid *item_grid, int y, int *row_origin)
 
 	do {
 		ColRowInfo *ri = sheet_row_get_info (item_grid->sheet, row);
+		int const tmp = ri->pixels;
 
-		if (y >= pixel && y <= pixel + ri->pixels) {
-			if (row_origin)
-				*row_origin = pixel;
-			return row;
+		if (tmp > 0) {
+			if (y >= pixel && y <= pixel + tmp) {
+				if (row_origin)
+					*row_origin = pixel;
+				return row;
+			}
+			pixel += tmp;
 		}
 		row++;
-		pixel += ri->pixels;
 	} while (row < SHEET_MAX_ROWS);
 	return SHEET_MAX_ROWS-1;
 }
@@ -242,19 +250,19 @@ item_grid_draw_border (GdkDrawable *drawable, MStyle *mstyle,
 				   x, y, x + w, y);
 	if (!extended_left &&
 	    mstyle_is_element_set (mstyle, MSTYLE_BORDER_LEFT))
-		style_border_draw (drawable, 
+		style_border_draw (drawable,
 				   mstyle_get_border (mstyle, MSTYLE_BORDER_LEFT),
 				   x, y, x, y + h);
 	if (mstyle_is_element_set (mstyle, MSTYLE_BORDER_BOTTOM))
-		style_border_draw (drawable, 
+		style_border_draw (drawable,
 				   mstyle_get_border (mstyle, MSTYLE_BORDER_BOTTOM),
 				   x, y + h, x + w, y + h);
 	if (!extended_right &&
 	    mstyle_is_element_set (mstyle, MSTYLE_BORDER_RIGHT))
-		style_border_draw (drawable, 
+		style_border_draw (drawable,
 				   mstyle_get_border (mstyle, MSTYLE_BORDER_RIGHT),
 				   x + w, y, x + w, y + h);
-	
+
 	if (mstyle_is_element_set (mstyle, MSTYLE_BORDER_DIAGONAL))
 		style_border_draw (drawable,
 				   mstyle_get_border (mstyle, MSTYLE_BORDER_DIAGONAL),
@@ -275,10 +283,14 @@ item_grid_draw_cell (GdkDrawable *drawable, ItemGrid *item_grid, Cell *cell, int
 {
 	GdkGC      *gc     = item_grid->gc;
 	GdkColor   *col;
-	int         count;
-	int         w, h;
+	int         count  = 1;
 	MStyle     *mstyle;
-	
+	int const w = cell->col->pixels;
+	int const h = cell->row->pixels;
+
+	if (w <= 0 || h <= 0)
+		return 0;
+
 	mstyle = sheet_style_compute (item_grid->sheet, cell->col->pos,
 				      cell->row->pos);
 
@@ -297,11 +309,9 @@ item_grid_draw_cell (GdkDrawable *drawable, ItemGrid *item_grid, Cell *cell, int
 	} else
 		gdk_gc_set_background (gc, &item_grid->background);
 
-	w = cell->col->pixels;
-	h = cell->row->pixels;
-
 	/* Draw cell contents BEFORE border */
-	count = cell_draw (cell, item_grid->sheet_view, gc, drawable, x1, y1);
+	if (cell->sheet->display_zero || !cell_is_zero (cell))
+		count = cell_draw (cell, item_grid->sheet_view, gc, drawable, x1, y1);
 
 	item_grid_draw_border (drawable, mstyle, x1, y1, w, h, count > 1, FALSE);
 
@@ -312,22 +322,28 @@ item_grid_draw_cell (GdkDrawable *drawable, ItemGrid *item_grid, Cell *cell, int
 
 static void
 item_grid_paint_empty_cell (GdkDrawable *drawable, ItemGrid *item_grid,
-			    ColRowInfo *ci, ColRowInfo *ri, int col, int row,
+			    ColRowInfo const * const ci, ColRowInfo const * const ri,
+			    int col, int row,
 			    int x, int y,
 			    int const span_count)
 {
 	MStyle *mstyle;
 	GdkGC  *gc = item_grid->empty_gc;
-	
+	int const w = ci->pixels;
+	int const h = ri->pixels;
+
+	if (w <= 0 || h <= 0)
+		return;
+
 	mstyle = sheet_style_compute (item_grid->sheet, col, row);
 
 	if (gnumeric_background_set_gc (mstyle, gc, item_grid->canvas_item.canvas))
 		/* Ignore margins. Fill the entire cell (including the right
 		 * hand divider) */
 		gdk_draw_rectangle (drawable, gc, TRUE,
-				    x, y, ci->pixels+1, ri->pixels+1);
+				    x, y, w+1, h+1);
 
-	item_grid_draw_border (drawable, mstyle, x, y, ci->pixels, ri->pixels,
+	item_grid_draw_border (drawable, mstyle, x, y, w, h,
 			       span_count > 1,
 			       span_count > 0);
 
@@ -340,77 +356,64 @@ item_grid_draw (GnomeCanvasItem *item, GdkDrawable *drawable, int x, int y, int
 	ItemGrid *item_grid = ITEM_GRID (item);
 	Sheet *sheet   = item_grid->sheet;
 	GdkGC *grid_gc = item_grid->grid_gc;
-	Cell  *cell;
-	int end_x, end_y;
-	int paint_col, paint_row, max_paint_col, max_paint_row;
-	int col, row;
-	int x_paint, y_paint, real_x;
-	int diff_x, diff_y;
+	int col, row, real_x;
 	int span_count = 0;
 
-	if (x < 0){
-		g_warning ("x < 0\n");
-		return;
-	}
+	int x_paint, y_paint;
+	int const paint_col = item_grid_find_col (item_grid, x, &x_paint);
+	int const paint_row = item_grid_find_row (item_grid, y, &y_paint);
+	int const diff_x = x - x_paint;
+	int const end_x  = width + diff_x;
+	int const diff_y = y - y_paint;
+	int const end_y  = height + diff_y;
 
-	if (y < 0){
-		g_warning ("y < 0\n");
-		return;
-	}
-
-	max_paint_col = max_paint_row = 0;
-	paint_col = item_grid_find_col (item_grid, x, &x_paint);
-	paint_row = item_grid_find_row (item_grid, y, &y_paint);
-
-	col = paint_col;
-
-	diff_x = x - x_paint;
-	end_x = width + diff_x;
-	diff_y = y - y_paint;
-	end_y = height + diff_y;
+	/* We can relax this eventually. See comment in item_grid_find_col */
+	g_return_if_fail (x >= 0);
+	g_return_if_fail (y >= 0);
 
 	/* 1. The default background */
 	gdk_draw_rectangle (drawable, item_grid->fill_gc, TRUE,
 			    0, 0, width, height);
 
 	/* 2. the grids */
-	for (x_paint = -diff_x; x_paint < end_x && col < SHEET_MAX_COLS; col++){
-		ColRowInfo *ci;
+	if (sheet->show_grid) {
+		col = paint_col;
+		for (x_paint = -diff_x; x_paint < end_x && col < SHEET_MAX_COLS; ++col){
+			ColRowInfo const * const ci = sheet_col_get_info (sheet, col);
+			int const tmp = ci->pixels;
 
-		ci = sheet_col_get_info (sheet, col);
-		g_assert (ci->pixels != 0);
+			if (tmp > 0) {
+				gdk_draw_line (drawable, grid_gc, x_paint, 0, x_paint, height);
+				x_paint += ci->pixels;
+			}
+		}
 
-		gdk_draw_line (drawable, grid_gc, x_paint, 0, x_paint, height);
+		row = paint_row;
+		for (y_paint = -diff_y; y_paint < end_y && row < SHEET_MAX_ROWS; ++row){
+			ColRowInfo const * const ri = sheet_row_get_info (sheet, row);
+			int const tmp = ri->pixels;
 
-		x_paint += ci->pixels;
-		max_paint_col = col;
-	}
-
-	row = paint_row;
-	for (y_paint = -diff_y; y_paint < end_y && row < SHEET_MAX_ROWS; row++){
-		ColRowInfo *ri;
-
-		ri = sheet_row_get_info (sheet, row);
-		gdk_draw_line (drawable, grid_gc, 0, y_paint, width, y_paint);
-		y_paint += ri->pixels;
-		max_paint_row = row;
+			if (tmp > 0) {
+				gdk_draw_line (drawable, grid_gc, 0, y_paint, width, y_paint);
+				y_paint += ri->pixels;
+			}
+		}
 	}
 
 	gdk_gc_set_function (item_grid->gc, GDK_COPY);
 
 	row = paint_row;
 	for (y_paint = -diff_y; y_paint < end_y && row < SHEET_MAX_ROWS; row++) {
-		ColRowInfo *ri;
+		ColRowInfo const * const ri = sheet_row_get_info (sheet, row);
+		if (ri->pixels <= 0)
+			continue;
 
-		ri = sheet_row_get_info (sheet, row);
 		col = paint_col;
-
 		for (x_paint = -diff_x; x_paint < end_x && col < SHEET_MAX_COLS; ++col) {
-			ColRowInfo *ci;
-			
-			ci = sheet_col_get_info (sheet, col);
-
-			cell = sheet_cell_get (sheet, col, row);
+			ColRowInfo const * const ci = sheet_col_get_info (sheet, col);
+			Cell *cell = sheet_cell_get (sheet, col, row);
+			if (ci->pixels <= 0)
+				continue;
 
 			/* If the cell does not exist paint it as an empty cell */
 			if (cell == NULL) {
@@ -445,21 +448,20 @@ item_grid_draw (GnomeCanvasItem *item, GdkDrawable *drawable, int x, int y, int
 					 * Either adjust the left part
 					 */
 					for (i = cell->col->pos; i < col; i++) {
-						ColRowInfo *tci;
-
-						tci = sheet_col_get_info (sheet, i);
-						real_x -= tci->pixels;
+						ColRowInfo const * const tci = sheet_col_get_info (sheet, i);
+						int const tmp = tci->pixels;
+						if (tmp > 0)
+							real_x -= tci->pixels;
 					}
 
 					/*
 					 * Or adjust the right part
 					 */
 					for (i = col; i < cell->col->pos; i++) {
-						ColRowInfo *tci;
-
-						tci = sheet_col_get_info (
-							sheet, i);
-						real_x += tci->pixels;
+						ColRowInfo const * const tci = sheet_col_get_info (sheet, i);
+						int const tmp = tci->pixels;
+						if (tmp > 0)
+							real_x += tci->pixels;
 					}
 
 					/*
@@ -477,12 +479,10 @@ item_grid_draw (GnomeCanvasItem *item, GdkDrawable *drawable, int x, int y, int
 					end_col = cell->col->pos + count;
 
 					for (i = col+1; i < end_col; i++, col++) {
-						ColRowInfo *tci;
-
-						tci = sheet_col_get_info (
-							sheet, i);
-
-						x_paint += tci->pixels;
+						ColRowInfo const * const tci = sheet_col_get_info (sheet, i);
+						int const tmp = tci->pixels;
+						if (tmp > 0)
+							x_paint += tci->pixels;
 					}
 				} /* if cell */
 			}
@@ -496,25 +496,23 @@ item_grid_draw (GnomeCanvasItem *item, GdkDrawable *drawable, int x, int y, int
 
 	row = paint_row;
 	for (y_paint = -diff_y; y_paint < end_y && row < SHEET_MAX_ROWS; row++) {
-		ColRowInfo *ri, *ci;
+		ColRowInfo const * const ri = sheet_row_get_info (sheet, row);
+		if (ri->pixels <= 0)
+			continue;
 
-		ri = sheet_row_get_info (sheet, row);
 		col = paint_col;
-
-		for (x_paint = -diff_x;
-		     x_paint < end_x && col < SHEET_MAX_COLS;
-		     ++col, x_paint += ci->pixels){
-			ci = sheet_col_get_info (sheet, col);
-
-			if (sheet->cursor_col == col && sheet->cursor_row == row)
+		for (x_paint = -diff_x; x_paint < end_x && col < SHEET_MAX_COLS; ++col) {
+			ColRowInfo const * const ci = sheet_col_get_info (sheet, col);
+			if (ci->pixels <= 0)
 				continue;
 
-			if (!sheet_selection_is_cell_selected (sheet, col, row))
-				continue;
-
-			gdk_draw_rectangle (drawable, item_grid->gc, TRUE,
-					    x_paint+1, y_paint+1,
-					    ci->pixels-1, ri->pixels-1);
+			if (!(sheet->cursor_col == col && sheet->cursor_row == row) &&
+			    sheet_selection_is_cell_selected (sheet, col, row)) {
+				gdk_draw_rectangle (drawable, item_grid->gc, TRUE,
+						    x_paint+1, y_paint+1,
+						    ci->pixels-1, ri->pixels-1);
+			}
+			x_paint += ci->pixels;
 		}
 		y_paint += ri->pixels;
 	}
@@ -597,7 +595,7 @@ context_delete_cmd (GtkWidget *widget, Sheet *sheet)
 static void
 context_clear_cmd (GtkWidget *widget, Sheet *sheet)
 {
-	sheet_selection_clear_content (workbook_command_context_gui (sheet->workbook), sheet);
+	cmd_clear_selection (workbook_command_context_gui (sheet->workbook), sheet, CLEAR_VALUES);
 	context_destroy_menu (widget);
 }
 
@@ -622,10 +620,38 @@ context_row_height (GtkWidget *widget, Sheet *sheet)
 	context_destroy_menu (widget);
 }
 
+static void
+context_col_hide (GtkWidget *widget, Sheet *sheet)
+{
+	cmd_hide_selection_rows_cols (workbook_command_context_gui (sheet->workbook),
+				      sheet, TRUE, FALSE);
+	context_destroy_menu (widget);
+}
+static void
+context_col_unhide (GtkWidget *widget, Sheet *sheet)
+{
+	cmd_hide_selection_rows_cols (workbook_command_context_gui (sheet->workbook),
+				      sheet, TRUE, TRUE);
+	context_destroy_menu (widget);
+}
+static void
+context_row_hide (GtkWidget *widget, Sheet *sheet)
+{
+	cmd_hide_selection_rows_cols (workbook_command_context_gui (sheet->workbook),
+				      sheet, FALSE, FALSE);
+	context_destroy_menu (widget);
+}
+static void
+context_row_unhide (GtkWidget *widget, Sheet *sheet)
+{
+	cmd_hide_selection_rows_cols (workbook_command_context_gui (sheet->workbook),
+				      sheet, FALSE, TRUE);
+	context_destroy_menu (widget);
+}
+
 typedef enum {
 	IG_ALWAYS,
 	IG_SEPARATOR,
-	IG_PASTE,
 	IG_PASTE_SPECIAL,
 	IG_ROW	   = 0x8,
 	IG_COLUMN  = 0x10,
@@ -643,7 +669,7 @@ static struct {
 	{ N_("_Copy"),          GNOME_STOCK_PIXMAP_COPY,
 	    &context_copy_cmd,          IG_ALWAYS },
 	{ N_("_Paste"),         GNOME_STOCK_PIXMAP_PASTE,
-	    &context_paste_cmd,         IG_PASTE },
+	    &context_paste_cmd,         IG_ALWAYS },
 	{ N_("Paste _Special"),	NULL,
 	    &context_paste_special_cmd, IG_PASTE_SPECIAL },
 
@@ -665,23 +691,22 @@ static struct {
 	{ N_("Column _Width..."),NULL,
 	    &context_column_width, IG_COLUMN },
 	{ N_("_Hide"),		 NULL,
-	    NULL,   IG_COLUMN },
+	    &context_col_hide,   IG_COLUMN },
 	{ N_("_Unhide"),	 NULL,
-	    NULL,   IG_COLUMN },
+	    &context_col_unhide,   IG_COLUMN },
 
 	/* Row specific functions (Note some of the labels are duplicated */
 	{ N_("_Row Height..."),	 NULL,
 	    &context_row_height,   IG_ROW },
 	{ N_("_Hide"),		 NULL,
-	    NULL,   IG_ROW },
+	    &context_row_hide,   IG_ROW },
 	{ N_("_Unhide"),	 NULL,
-	    NULL,   IG_ROW },
+	    &context_row_unhide,   IG_ROW },
 	{ NULL, NULL, NULL, 0 }
 };
 
 static GtkWidget *
 create_popup_menu (Sheet *sheet,
-		   gboolean const include_paste,
 		   gboolean const include_paste_special,
 		   gboolean const is_col,
 		   gboolean const is_row)
@@ -707,7 +732,7 @@ create_popup_menu (Sheet *sheet,
 			break;
 
 		/* Desesitize later */
-		case IG_PASTE : case IG_PASTE_SPECIAL :
+		case IG_PASTE_SPECIAL :
 		case IG_ROW :	case IG_COLUMN :
 		case IG_ALWAYS:
 		{
@@ -740,8 +765,7 @@ create_popup_menu (Sheet *sheet,
 			g_warning ("Never reached");
 		}
 
-		if ((type == IG_PASTE && !include_paste) ||
-		    (type == IG_PASTE_SPECIAL && !include_paste_special) ||
+		if ((type == IG_PASTE_SPECIAL && !include_paste_special) ||
 		    item_grid_context_menu [i].fn == NULL)
 			gtk_widget_set_sensitive (GTK_WIDGET (item), FALSE);
 
@@ -754,7 +778,6 @@ create_popup_menu (Sheet *sheet,
 				GTK_PIXMAP_MENU_ITEM (item),
 				pixmap);
 		}
-
 		if (item_grid_context_menu [i].fn)
 			gtk_signal_connect (
 				GTK_OBJECT (item), "activate",
@@ -775,13 +798,16 @@ item_grid_popup_menu (Sheet *sheet, GdkEvent *event, int col, int row,
 {
 	GtkWidget *menu;
 
-	/* We can paste if there is something in the clipboard */
-	gboolean const show_paste = !application_clipboard_is_empty ();
-	/* Paste special only applies to copied cells not cut */
-	gboolean const show_paste_special = show_paste &&
+	/*
+	 * Paste special does not apply to cut cells.  Enable
+	 * when there is nothing in the local clipboard, or when
+	 * the clipboard has the results of a copy.
+	 */
+	gboolean const show_paste_special =
+	    application_clipboard_is_empty () ||
 	    (application_clipboard_contents_get () != NULL);
 
-	menu = create_popup_menu (sheet, show_paste, show_paste_special,
+	menu = create_popup_menu (sheet, show_paste_special,
 				  is_col, is_row);
 
 	gnumeric_popup_menu (GTK_MENU (menu), (GdkEventButton *) event);
@@ -812,7 +838,7 @@ item_grid_button_1 (Sheet *sheet, GdkEvent *event, ItemGrid *item_grid, int col,
 		gnumeric_sheet_selection_cursor_base (gsheet, col, row);
 		return 1;
 	}
-	
+
 	/*
 	 * If the user is editing a formula (gnumeric_sheet_can_move_cursor)
 	 * then we enable the dynamic cell selection mode.
@@ -975,7 +1001,7 @@ drag_start (GtkWidget *widget, GdkEvent *event, Sheet *sheet)
 	static GtkTargetEntry drag_types [] = {
 		{ "bonobo/moniker", 0, 1 },
 	};
-	
+
         list = gtk_target_list_new (drag_types, 1);
 
         context = gtk_drag_begin (widget, list,
@@ -1123,7 +1149,7 @@ item_grid_event (GnomeCanvasItem *item, GdkEvent *event)
 			g_warning ("This is here just for demo purposes");
 			drag_start (GTK_WIDGET (item->canvas), event, sheet);
 			return 1;
-			
+
 		case 3:
 			item_grid_popup_menu (item_grid->sheet,
 					      event, col, row,
