@@ -24,10 +24,12 @@
 /*
  * Gnumeric Parser
  *
- * (C) 1998 the Free Software Foundation
+ * (C) 1998-2000 the Free Software Foundation
  *
  * Author:
  *    Miguel de Icaza (miguel@gnu.org)
+ *    Jody Goldberg (jgoldberg@home.com)
+ *    Morten Welinder (terra@diku.dk)
  */
 #include <config.h>
 #include <ctype.h>
@@ -40,40 +42,147 @@
 #include "expr.h"
 #include "sheet.h"
 #include "utils.h"
-	
-/* Allocation with disposal-on-error */ 
-static void *alloc_buffer    (int size);
-static void  register_symbol (Symbol *sym);
-static void  alloc_clean     (void);
-static void  alloc_glist     (GList *l); 
-static void  forget_glist    (GList *list);
-static void  forget_array    (GList *array);
-static void  forget_tree     (ExprTree *tree);
-static void  release_const_tree (ExprTree *tree);
-static void  alloc_list_free (void); 
-static Value*v_new (void);
-static void  v_forget (Value *v);
-	
+#include "auto-format.h"
+
+/* ------------------------------------------------------------------------- */
+/* Allocation with disposal-on-error */
+
+/*
+ * Defined: the stack itself will be kept in use.  This isn't much, btw.
+ *   This setting is good for speed.
+ *
+ * Not defined: memory will be freed.  The is good for finding leaks in the
+ * program.  (Here and elsewhere.)
+ */
+#define KEEP_DEALLOCATION_STACK_BETWEEN_CALLS
+
+/*
+ * If some dork enters "=1+2+2*(1+" we have already allocated space for
+ * "1+2", "2", and "1" before the parser sees the syntax error and warps
+ * us to the error production in the "line" non-terminal.
+ *
+ * To make sure we can clean up, we register every allocation.  On success,
+ * nothing should be left (except the final expression which is unregistered),
+ * but on failure we must free everything allocated.
+ *
+ * Note: there is some room left for optimisation here.  Talk to terra@diku.dk
+ * before you set out to do it.
+ */
+
+static void
+free_expr_list (GList *list)
+{
+	GList *l;
+	for (l = list; l; l = l->next)
+		expr_tree_unref (l->data);
+	g_list_free (list);
+}
+
+static void
+free_expr_list_list (GList *list)
+{
+	GList *l;
+	for (l = list; l; l = l->next)
+		free_expr_list (l->data);
+	g_list_free (list);
+}
+
+typedef void (*ParseDeallocator) (void *);
+static GPtrArray *deallocate_stack;
+
+static void
+deallocate_init (void)
+{
+	deallocate_stack = g_ptr_array_new ();
+}
+
+#ifndef KEEP_DEALLOCATION_STACK_BETWEEN_CALLS
+static void
+deallocate_uninit (void)
+{
+	g_ptr_array_free (deallocate_stack, TRUE);
+	deallocate_stack = NULL;
+}
+#endif
+
+static void
+deallocate_all (void)
+{
+	int i;
+
+	for (i = 0; i < deallocate_stack->len; i += 2) {
+		ParseDeallocator freer = g_ptr_array_index (deallocate_stack, i + 1);
+		freer (g_ptr_array_index (deallocate_stack, i));
+	}
+
+	g_ptr_array_set_size (deallocate_stack, 0);
+}
+
+static void
+deallocate_assert_empty (void)
+{
+	if (deallocate_stack->len == 0)
+		return;
+
+	g_warning ("deallocate_stack not empty as expected.");
+	deallocate_all ();
+}
+
+static void *
+register_allocation (void *data, ParseDeallocator freer)
+{
+	/* It's handy to be able to register and unregister NULLs.  */
+	if (data) {
+		int len;
+		/*
+		 * There are really only a few different freers, so we
+		 * could encode the freer in the lower bits of the data
+		 * pointer.  Unfortunately, no-one can predict how high
+		 * Miguel would jump when he found out.
+		 */
+		len = deallocate_stack->len;
+		g_ptr_array_set_size (deallocate_stack, len + 2);
+		g_ptr_array_index (deallocate_stack, len) = data;
+		g_ptr_array_index (deallocate_stack, len + 1) = freer;
+	}
+
+	/* Returning the pointer here improved readability of the caller.  */
+	return data;
+}
+
+#define register_expr_allocation(expr) \
+  register_allocation ((expr), (ParseDeallocator)&expr_tree_unref)
+
+#define register_expr_list_allocation(list) \
+  register_allocation ((list), (ParseDeallocator)&free_expr_list)
+
+#define register_expr_list_list_allocation(list) \
+  register_allocation ((list), (ParseDeallocator)&free_expr_list_list)
+
+static void
+unregister_allocation (const void *data)
+{
+	int pos;
+
+	/* It's handy to be able to register and unregister NULLs.  */
+	if (!data)
+		return;
+
+	pos = deallocate_stack->len - 2;
+	if (pos < 0 ||
+	    data != g_ptr_array_index (deallocate_stack, pos)) {
+		g_warning ("Unbalanced allocation registration");
+		return;
+	}
+
+	g_ptr_array_set_size (deallocate_stack, pos);
+}
+
+/* ------------------------------------------------------------------------- */
+
 #define ERROR -1
- 
-/* Types of items we know how to dispose on error */ 
-typedef enum {
-	ALLOC_SYMBOL,
-	ALLOC_VALUE,
-	ALLOC_BUFFER,
-	ALLOC_LIST
-} AllocType;
 
-/* How we keep track of them */ 
-typedef struct {
-	AllocType type;
-	void      *data;
-} AllocRec;
-
-/* This keeps a list of  AllocRecs */
-static GList *alloc_list;
- 
-/* Bison/Yacc internals */ 
+/* Bison/Yacc internals */
 static int  yylex (void);
 static int  yyerror (char *s);
 
@@ -90,36 +199,28 @@ static int parser_col, parser_row;
 static Workbook *parser_wb;
 
 /* The suggested format to use for this expression */
-static const char **parser_desired_format;
+static char **parser_desired_format;
 
 /* Locale info.  */
 static char parser_decimal_point;
 static char parser_separator;
 static char parser_array_col_separator;
- 
-static ExprTree **parser_result;
 
-#define p_new(type) ((type *) alloc_buffer ((unsigned) sizeof (type)))
+static ExprTree **parser_result;
 
 static ExprTree *
 build_unary_op (Operation op, ExprTree *expr)
 {
-	ExprTree *res = p_new (ExprTree);
-	res->ref_count = 1;
-	res->oper = op;
-	res->u.value = expr;
-	return res;
+	unregister_allocation (expr);
+	return register_expr_allocation (expr_tree_new_unary (op, expr));
 }
 
 static ExprTree *
 build_binop (ExprTree *l, Operation op, ExprTree *r)
 {
-	ExprTree *res = p_new (ExprTree);
-	res->ref_count = 1;
-	res->u.binary.value_a = l;
-	res->oper = op;
-	res->u.binary.value_b = r;
-	return res;
+	unregister_allocation (r);
+	unregister_allocation (l);
+	return register_expr_allocation (expr_tree_new_binary (l, op, r));
 }
 
 static ExprTree *
@@ -127,19 +228,15 @@ build_array_formula (ExprTree *func, int cols, int rows, int x, int y)
 {
 	ExprTree *res = expr_tree_array_formula (x, y, rows, cols);
 
+	/*
+	 * Note: for a non-corner cell, caller must arrange to have the
+	 * inner expression ("func") unref'ed.  This happens in
+	 * cell_set_formula.
+	 */
 	res->u.array.corner.func.expr = func;
 	return res;
 }
 
-static void
-release_const_tree (ExprTree *tree)
-{
-	g_assert (tree->oper == OPER_CONSTANT);
-
-	g_free (tree->u.constant);
-	g_free (tree);
-}
-
 static ExprTree *
 build_array (GList *cols)
 {
@@ -171,10 +268,8 @@ build_array (GList *cols)
 
 			g_assert (expr->oper == OPER_CONSTANT);
 
-			value_array_set (array, x, y, v);
-			v_forget (v);
-			g_free (expr);
-			
+			value_array_set (array, x, y, value_duplicate (v));
+
 			x++;
 			row = g_list_next (row);
 		}
@@ -187,14 +282,14 @@ build_array (GList *cols)
 		cols = g_list_next (cols);
 	}
 
-	return expr_tree_new_constant (array);
+	return register_expr_allocation (expr_tree_new_constant (array));
 }
 
 /* Make byacc happier */
 int yyparse(void);
 
 
-#line 176 "parser.y"
+#line 271 "parser.y"
 typedef union {
 	ExprTree *tree;
 	CellRef  *cell;
@@ -252,9 +347,8 @@ static const char yytranslate[] = {     0,
 static const short yyprhs[] = {     0,
      0,     2,    17,    19,    21,    23,    25,    27,    31,    35,
     39,    43,    47,    51,    55,    59,    63,    67,    71,    75,
-    79,    82,    85,    88,    91,    95,    99,   104,   106,   110,
-   117,   119,   123,   124,   126,   128,   130,   134,   138,   139,
-   141,   145
+    79,    82,    85,    88,    92,    96,   101,   103,   107,   114,
+   116,   120,   121,   123,   125,   127,   131,   135,   136,   138
 };
 
 static const short yyrhs[] = {    38,
@@ -266,24 +360,23 @@ static const short yyrhs[] = {    38,
     38,    17,    38,     0,    38,    15,    38,     0,    38,    16,
     38,     0,    38,     8,    38,     0,    38,    10,    38,     0,
     38,     9,    38,     0,    29,    38,    30,     0,    38,    26,
-     0,    18,    38,     0,    19,    38,     0,    19,    38,     0,
-    27,    43,    28,     0,    39,    33,    39,     0,     5,    29,
-    40,    30,     0,     7,     0,    11,    24,     7,     0,    31,
-    12,    32,    11,    24,     7,     0,    38,     0,    38,    13,
-    40,     0,     0,     3,     0,     4,     0,    41,     0,    41,
-    34,    42,     0,    41,    35,    42,     0,     0,    42,     0,
-    42,    36,    43,     0,     0
+     0,    18,    38,     0,    19,    38,     0,    27,    43,    28,
+     0,    39,    33,    39,     0,     5,    29,    40,    30,     0,
+     7,     0,    11,    24,     7,     0,    31,    12,    32,    11,
+    24,     7,     0,    38,     0,    38,    13,    40,     0,     0,
+     3,     0,     4,     0,    41,     0,    41,    34,    42,     0,
+    41,    35,    42,     0,     0,    42,     0,    42,    36,    43,
+     0
 };
 
 #endif
 
 #if YYDEBUG != 0
 static const short yyrline[] = { 0,
-   201,   203,   221,   224,   225,   226,   227,   228,   229,   230,
-   231,   232,   233,   234,   235,   236,   237,   238,   239,   240,
-   242,   243,   244,   246,   250,   255,   269,   275,   279,   284,
-   290,   294,   299,   302,   303,   306,   310,   318,   326,   329,
-   333,   338
+   296,   301,   324,   327,   328,   329,   330,   331,   332,   333,
+   334,   335,   336,   337,   338,   339,   340,   341,   342,   343,
+   345,   346,   347,   349,   355,   365,   372,   376,   381,   388,
+   393,   399,   402,   403,   406,   411,   420,   429,   432,   437
 };
 #endif
 
@@ -301,28 +394,26 @@ static const char * const yytname[] = {   "$","error","$undefined.","NUMBER",
 static const short yyr1[] = {     0,
     37,    37,    37,    38,    38,    38,    38,    38,    38,    38,
     38,    38,    38,    38,    38,    38,    38,    38,    38,    38,
-    38,    38,    38,    38,    38,    38,    38,    39,    39,    39,
-    40,    40,    40,    41,    41,    42,    42,    42,    42,    43,
-    43,    43
+    38,    38,    38,    38,    38,    38,    39,    39,    39,    40,
+    40,    40,    41,    41,    42,    42,    42,    42,    43,    43
 };
 
 static const short yyr2[] = {     0,
      1,    14,     1,     1,     1,     1,     1,     3,     3,     3,
      3,     3,     3,     3,     3,     3,     3,     3,     3,     3,
-     2,     2,     2,     2,     3,     3,     4,     1,     3,     6,
-     1,     3,     0,     1,     1,     1,     3,     3,     0,     1,
-     3,     0
+     2,     2,     2,     3,     3,     4,     1,     3,     6,     1,
+     3,     0,     1,     1,     1,     3,     3,     0,     1,     3
 };
 
 static const short yydefact[] = {     0,
-     3,     4,     5,     0,     7,    28,     0,     0,     0,    39,
-     0,     0,     1,     6,    33,     0,    39,    22,    23,     4,
-     5,     0,    36,    40,     0,     0,     0,     0,     0,     0,
+     3,     4,     5,     0,     7,    27,     0,     0,     0,    38,
+     0,     0,     1,     6,    32,     0,    38,    22,    23,     4,
+     5,     0,    35,    39,     0,     0,     0,     0,     0,     0,
      0,     0,     0,     0,     0,     0,     0,     0,     0,    21,
-     0,    31,     0,    29,    34,    35,     0,    39,    39,    39,
-    25,    20,     0,    17,    19,    18,    13,    15,    16,    14,
-     9,     8,    10,    11,    12,    26,    33,    27,     0,    37,
-    38,    41,     0,    32,     0,     0,     0,    30,     0,     0,
+     0,    30,     0,    28,    33,    34,     0,    38,    38,    38,
+    24,    20,     0,    17,    19,    18,    13,    15,    16,    14,
+     9,     8,    10,    11,    12,    25,    32,    26,     0,    36,
+    37,    40,     0,    31,     0,     0,     0,    29,     0,     0,
      0,     0,     0,     0,     0,     2,     0,     0,     0
 };
 
@@ -351,11 +442,11 @@ static const short yypgoto[] = {-32768,
 
 
 static const short yytable[] = {    13,
-     6,   -34,   -34,   -34,     7,    45,    46,    18,    19,    22,
+     6,   -33,   -33,   -33,     7,    45,    46,    18,    19,    22,
     26,    15,    35,    36,    37,    38,    37,    38,    27,    39,
     40,    39,    40,    41,    12,    39,    40,    54,    55,    56,
     57,    58,    59,    60,    61,    62,    63,    64,    65,     1,
-    44,     2,     3,     4,     5,     6,   -35,   -35,   -35,     7,
+    44,     2,     3,     4,     5,     6,   -34,   -34,   -34,     7,
     48,    49,    70,    71,    16,    50,     8,     9,     2,     3,
      4,     5,     6,    51,    53,    10,     7,    11,    69,    12,
     68,    73,    75,     8,     9,    20,    21,     4,     5,     6,
@@ -894,247 +985,252 @@ yyreduce:
   switch (yyn) {
 
 case 1:
-#line 201 "parser.y"
-{ *parser_result = yyvsp[0].tree; ;
+#line 296 "parser.y"
+{
+		unregister_allocation (yyvsp[0].tree);
+		*parser_result = yyvsp[0].tree;
+	;
     break;}
 case 2:
-#line 203 "parser.y"
+#line 301 "parser.y"
 {
 		const int num_cols = expr_tree_get_const_int (yyvsp[-7].tree);
 		const int num_rows = expr_tree_get_const_int (yyvsp[-9].tree);
 		const int x = expr_tree_get_const_int (yyvsp[-1].tree);
 		const int y = expr_tree_get_const_int (yyvsp[-4].tree);
 
-		*parser_result = build_array_formula (yyvsp[-12].tree, num_cols, num_rows, x, y);
-
 		/*
 		 * Notice that we have no use for the ExprTrees for the NUMBERS,
 		 * so we deallocate them.
 		 */
-		release_const_tree (yyvsp[-7].tree);
-		release_const_tree (yyvsp[-9].tree);
-		release_const_tree (yyvsp[-1].tree);
-		release_const_tree (yyvsp[-4].tree);
+		unregister_allocation (yyvsp[-1].tree);
+		expr_tree_unref (yyvsp[-1].tree);
+		unregister_allocation (yyvsp[-4].tree);
+		expr_tree_unref (yyvsp[-4].tree);
+		unregister_allocation (yyvsp[-7].tree);
+		expr_tree_unref (yyvsp[-7].tree);
+		unregister_allocation (yyvsp[-9].tree);
+		expr_tree_unref (yyvsp[-9].tree);
+
+		unregister_allocation (yyvsp[-12].tree);
+		*parser_result = build_array_formula (yyvsp[-12].tree, num_cols, num_rows, x, y);
 	;
     break;}
 case 3:
-#line 221 "parser.y"
+#line 324 "parser.y"
 { parser_error = PARSE_ERR_SYNTAX; ;
     break;}
 case 4:
-#line 224 "parser.y"
+#line 327 "parser.y"
 { yyval.tree = yyvsp[0].tree; ;
     break;}
 case 5:
-#line 225 "parser.y"
+#line 328 "parser.y"
 { yyval.tree = yyvsp[0].tree; ;
     break;}
 case 6:
-#line 226 "parser.y"
+#line 329 "parser.y"
 { yyval.tree = yyvsp[0].tree; ;
     break;}
 case 7:
-#line 227 "parser.y"
+#line 330 "parser.y"
 { yyval.tree = yyvsp[0].tree; ;
     break;}
 case 8:
-#line 228 "parser.y"
+#line 331 "parser.y"
 { yyval.tree = build_binop (yyvsp[-2].tree, OPER_ADD,       yyvsp[0].tree); ;
     break;}
 case 9:
-#line 229 "parser.y"
+#line 332 "parser.y"
 { yyval.tree = build_binop (yyvsp[-2].tree, OPER_SUB,       yyvsp[0].tree); ;
     break;}
 case 10:
-#line 230 "parser.y"
+#line 333 "parser.y"
 { yyval.tree = build_binop (yyvsp[-2].tree, OPER_MULT,      yyvsp[0].tree); ;
     break;}
 case 11:
-#line 231 "parser.y"
+#line 334 "parser.y"
 { yyval.tree = build_binop (yyvsp[-2].tree, OPER_DIV,       yyvsp[0].tree); ;
     break;}
 case 12:
-#line 232 "parser.y"
+#line 335 "parser.y"
 { yyval.tree = build_binop (yyvsp[-2].tree, OPER_EXP,       yyvsp[0].tree); ;
     break;}
 case 13:
-#line 233 "parser.y"
+#line 336 "parser.y"
 { yyval.tree = build_binop (yyvsp[-2].tree, OPER_CONCAT,    yyvsp[0].tree); ;
     break;}
 case 14:
-#line 234 "parser.y"
+#line 337 "parser.y"
 { yyval.tree = build_binop (yyvsp[-2].tree, OPER_EQUAL,     yyvsp[0].tree); ;
     break;}
 case 15:
-#line 235 "parser.y"
+#line 338 "parser.y"
 { yyval.tree = build_binop (yyvsp[-2].tree, OPER_LT,        yyvsp[0].tree); ;
     break;}
 case 16:
-#line 236 "parser.y"
+#line 339 "parser.y"
 { yyval.tree = build_binop (yyvsp[-2].tree, OPER_GT,        yyvsp[0].tree); ;
     break;}
 case 17:
-#line 237 "parser.y"
+#line 340 "parser.y"
 { yyval.tree = build_binop (yyvsp[-2].tree, OPER_GTE,       yyvsp[0].tree); ;
     break;}
 case 18:
-#line 238 "parser.y"
+#line 341 "parser.y"
 { yyval.tree = build_binop (yyvsp[-2].tree, OPER_NOT_EQUAL, yyvsp[0].tree); ;
     break;}
 case 19:
-#line 239 "parser.y"
+#line 342 "parser.y"
 { yyval.tree = build_binop (yyvsp[-2].tree, OPER_LTE,       yyvsp[0].tree); ;
     break;}
 case 20:
-#line 240 "parser.y"
+#line 343 "parser.y"
 { yyval.tree = yyvsp[-1].tree; ;
     break;}
 case 21:
-#line 242 "parser.y"
+#line 345 "parser.y"
 { yyval.tree = build_unary_op (OPER_PERCENT, yyvsp[-1].tree); ;
     break;}
 case 22:
-#line 243 "parser.y"
-{ yyval.tree = build_unary_op (OPER_NEG, yyvsp[0].tree); ;
+#line 346 "parser.y"
+{ yyval.tree = build_unary_op (OPER_UNARY_NEG, yyvsp[0].tree); ;
     break;}
 case 23:
-#line 244 "parser.y"
-{ yyval.tree = yyvsp[0].tree; ;
+#line 347 "parser.y"
+{ yyval.tree = build_unary_op (OPER_UNARY_PLUS, yyvsp[0].tree); ;
     break;}
 case 24:
-#line 246 "parser.y"
+#line 349 "parser.y"
 {
-		yyval.tree = yyvsp[0].tree;
+		unregister_allocation (yyvsp[-1].list);
+		yyval.tree = build_array (yyvsp[-1].list);
+		free_expr_list_list (yyvsp[-1].list);
 	;
     break;}
 case 25:
-#line 250 "parser.y"
+#line 355 "parser.y"
 {
-		yyval.tree = build_array (yyvsp[-1].list);
-		forget_array (yyvsp[-1].list);
+		unregister_allocation (yyvsp[0].tree);
+		unregister_allocation (yyvsp[-2].tree);
+		yyval.tree = register_expr_allocation
+			(expr_tree_new_constant
+			 (value_new_cellrange (&(yyvsp[-2].tree->u.ref), &(yyvsp[0].tree->u.ref))));
+		expr_tree_unref (yyvsp[0].tree);
+		expr_tree_unref (yyvsp[-2].tree);
 	;
     break;}
 case 26:
-#line 255 "parser.y"
-{
-		yyval.tree = p_new (ExprTree);
-		yyval.tree->ref_count  = 1;
-		yyval.tree->oper       = OPER_CONSTANT;
-		yyval.tree->u.constant = v_new ();
-		yyval.tree->u.constant->type = VALUE_CELLRANGE;
-		yyval.tree->u.constant->v.cell_range.cell_a = yyvsp[-2].tree->u.ref;
-		yyval.tree->u.constant->v.cell_range.cell_b = yyvsp[0].tree->u.ref;
-
-		forget_tree (yyvsp[-2].tree);
-		forget_tree (yyvsp[0].tree);
-	;
-    break;}
-case 27:
-#line 269 "parser.y"
+#line 365 "parser.y"
 {
+		unregister_allocation (yyvsp[-1].list);
 		yyval.tree = yyvsp[-3].tree;
 		yyval.tree->u.function.arg_list = yyvsp[-1].list;
 	;
     break;}
-case 28:
-#line 275 "parser.y"
+case 27:
+#line 372 "parser.y"
 {
 		yyval.tree = yyvsp[0].tree;
 	;
     break;}
-case 29:
-#line 279 "parser.y"
+case 28:
+#line 376 "parser.y"
 {
 	        yyval.tree = yyvsp[0].tree;
 		yyval.tree->u.ref.sheet = yyvsp[-2].sheetref;
 	;
     break;}
+case 29:
+#line 381 "parser.y"
+{
+		/* FIXME: what about $2?  -- MW.  */
+	        yyval.tree = yyvsp[0].tree;
+		yyval.tree->u.ref.sheet = yyvsp[-2].sheetref;
+	;
+    break;}
 case 30:
-#line 284 "parser.y"
-{
-	        yyval.tree = yyvsp[0].tree;
-		yyval.tree->u.ref.sheet = yyvsp[-2].sheetref;
-	;
-    break;}
-case 31:
-#line 290 "parser.y"
+#line 388 "parser.y"
 {
+		unregister_allocation (yyvsp[0].tree);
 		yyval.list = g_list_prepend (NULL, yyvsp[0].tree);
-		alloc_glist (yyval.list);
+		register_expr_list_allocation (yyval.list);
         ;
     break;}
-case 32:
-#line 294 "parser.y"
+case 31:
+#line 393 "parser.y"
 {
-		forget_glist (yyvsp[0].list);
+		unregister_allocation (yyvsp[0].list);
+		unregister_allocation (yyvsp[-2].tree);
 		yyval.list = g_list_prepend (yyvsp[0].list, yyvsp[-2].tree);
-		alloc_glist (yyval.list);
+		register_expr_list_allocation (yyval.list);
 	;
     break;}
-case 33:
-#line 299 "parser.y"
+case 32:
+#line 399 "parser.y"
 { yyval.list = NULL; ;
     break;}
+case 33:
+#line 402 "parser.y"
+{ yyval.tree = yyvsp[0].tree; ;
+    break;}
 case 34:
-#line 302 "parser.y"
+#line 403 "parser.y"
 { yyval.tree = yyvsp[0].tree; ;
     break;}
 case 35:
-#line 303 "parser.y"
-{ yyval.tree = yyvsp[0].tree; ;
-    break;}
-case 36:
-#line 306 "parser.y"
+#line 406 "parser.y"
 {
+		unregister_allocation (yyvsp[0].tree);
 		yyval.list = g_list_prepend (NULL, yyvsp[0].tree);
-		alloc_glist (yyval.list);
+		register_expr_list_allocation (yyval.list);
         ;
     break;}
-case 37:
-#line 310 "parser.y"
+case 36:
+#line 411 "parser.y"
 {
 		if (parser_array_col_separator == ',') {
-			forget_glist (yyvsp[0].list);
+			unregister_allocation (yyvsp[0].list);
+			unregister_allocation (yyvsp[-2].tree);
 			yyval.list = g_list_prepend (yyvsp[0].list, yyvsp[-2].tree);
-			alloc_glist  (yyval.list);
+			register_expr_list_allocation (yyval.list);
+		} else
+			parser_error = PARSE_ERR_SYNTAX;
+	;
+    break;}
+case 37:
+#line 420 "parser.y"
+{
+		if (parser_array_col_separator == '\\') {
+			unregister_allocation (yyvsp[0].list);
+			unregister_allocation (yyvsp[-2].tree);
+			yyval.list = g_list_prepend (yyvsp[0].list, yyvsp[-2].tree);
+			register_expr_list_allocation (yyval.list);
 		} else
 			parser_error = PARSE_ERR_SYNTAX;
 	;
     break;}
 case 38:
-#line 318 "parser.y"
-{
-		if (parser_array_col_separator == '\\') {
-			forget_glist (yyvsp[0].list);
-			yyval.list = g_list_prepend (yyvsp[0].list, yyvsp[-2].tree);
-			alloc_glist  (yyval.list);
-		} else
-			parser_error = PARSE_ERR_SYNTAX;
-	;
+#line 429 "parser.y"
+{ yyval.list = NULL; ;
     break;}
 case 39:
-#line 326 "parser.y"
-{ yyval.list = NULL; ;
-    break;}
-case 40:
-#line 329 "parser.y"
+#line 432 "parser.y"
 {
+		unregister_allocation (yyvsp[0].list);
 		yyval.list = g_list_prepend (NULL, yyvsp[0].list);
-		alloc_glist (yyval.list);
+		register_expr_list_list_allocation (yyval.list);
         ;
     break;}
-case 41:
-#line 333 "parser.y"
+case 40:
+#line 437 "parser.y"
 {
-		forget_glist (yyvsp[0].list);
+		unregister_allocation (yyvsp[0].list);
+		unregister_allocation (yyvsp[-2].list);
 		yyval.list = g_list_prepend (yyvsp[0].list, yyvsp[-2].list);
-		alloc_glist  (yyval.list);
+		register_expr_list_list_allocation (yyval.list);
 	;
     break;}
-case 42:
-#line 338 "parser.y"
-{ yyval.list = NULL; ;
-    break;}
 }
    /* the action file gets copied in in place of this dollarsign */
 #line 498 "/usr/lib/bison.simple"
@@ -1333,28 +1429,19 @@ yyerrhandle:
   yystate = yyn;
   goto yynewstate;
 }
-#line 340 "parser.y"
+#line 444 "parser.y"
 
 
 static int
 return_cellref (char *p)
 {
 	CellRef   ref;
-	ExprTree *e;
 
-	if (!cellref_get (&ref, p, parser_col, parser_row))
+	if (cellref_get (&ref, p, parser_col, parser_row)) {
+		yylval.tree = register_expr_allocation (expr_tree_new_var (&ref));
+		return CELLREF;
+	} else
 		return 0;
-
-	/*  Ok, parsed successfully, create the return value */
-	e = p_new (ExprTree);
-	e->ref_count = 1;
-
-	e->oper  = OPER_VAR;
-	e->u.ref = ref;
-
-	yylval.tree = e;
-
-	return CELLREF;
 }
 
 static int
@@ -1367,15 +1454,10 @@ return_sheetref (Sheet *sheet)
 static int
 make_string_return (char const *string, gboolean const possible_number)
 {
-	ExprTree *e;
 	Value *v;
 	double fv;
 	char *format;
 
-	e = p_new (ExprTree);
-	e->ref_count = 1;
-
-	v = v_new ();
 	/*
 	 * Try to match the entered text against any
 	 * of the known number formating codes, if this
@@ -1386,20 +1468,13 @@ make_string_return (char const *string, gboolean const possible_number)
 	 */
 	if (possible_number && string[0] != '\0' &&
 	    format_match (string, &fv, &format)){
-		v->type = VALUE_FLOAT;
-		v->v.v_float = fv;
+		v = value_new_float (fv);
 		if (parser_desired_format && *parser_desired_format == NULL)
-			*parser_desired_format = format;
-	} else {
-		v->v.str = string_get (string);
-		v->type = VALUE_STRING;
-	}
-	
-	e->oper = OPER_CONSTANT;
-	e->u.constant = v;
-	
-	yylval.tree = e;
+			*parser_desired_format = g_strdup (format);
+	} else
+		v = value_new_string (string);
 
+	yylval.tree = register_expr_allocation (expr_tree_new_constant (v));
 	return STRING;
 }
 
@@ -1408,54 +1483,33 @@ return_symbol (Symbol *sym)
 {
 	ExprTree *e = NULL;
 	int type = STRING;
-	
-	symbol_ref (sym);
-	
+
 	switch (sym->type){
 	case SYMBOL_FUNCTION:
-		e = p_new (ExprTree);
-		e->ref_count = 1;
-		e->oper = OPER_FUNCALL;
+		symbol_ref (sym);
+		e = expr_tree_new_funcall (sym, NULL);
 		type = FUNCALL;
-		e->u.function.symbol = sym;
-		e->u.function.arg_list = NULL;
 		break;
-		
+
 	case SYMBOL_VALUE:
 	case SYMBOL_STRING: {
-		Value *v, *dv;
-		
-		/* Make a copy of the value */
-		dv = (Value *) sym->data;
-		v = v_new ();
-		value_copy_to (v, dv);
-		
-		e = p_new (ExprTree);
-		e->ref_count = 1;
-		e->oper = OPER_CONSTANT;
-		e->u.constant = v;
+		e = expr_tree_new_constant (value_duplicate (sym->data));
 		type = CONSTANT;
 		break;
 	}
-	
+
+	default:
+		g_assert_not_reached ();
 	} /* switch */
 
-	register_symbol (sym);
-	yylval.tree = e;
-
+	yylval.tree = register_expr_allocation (e);
 	return type;
 }
 
 static int
 return_name (ExprName *exprn)
 {
-	ExprTree *e = p_new (ExprTree);
-	e->ref_count = 1;
-
-	e->oper = OPER_NAME;
-	e->u.name = exprn;
-	yylval.tree = e;
-
+	yylval.tree = register_expr_allocation (expr_tree_new_name (exprn));
 	return CONSTANT;
 }
 
@@ -1490,7 +1544,7 @@ try_symbol (char *string, gboolean try_cellref_and_number)
 		if (sheet)
 			return return_sheetref (sheet);
 	}
-	
+
 	{ /* Name ? */
 		/*
 		 * FIXME: we need a good bit of work to get sheet
@@ -1505,7 +1559,8 @@ try_symbol (char *string, gboolean try_cellref_and_number)
 	return make_string_return (string, try_cellref_and_number);
 }
 
-int yylex (void)
+int
+yylex (void)
 {
 	int c;
 	const char *p, *tmp;
@@ -1520,7 +1575,7 @@ int yylex (void)
 
 	if (c == parser_separator)
 		return SEPARATOR;
-	
+
 	/* Translate locale's decimal marker into a dot.  */
 	if (c == parser_decimal_point)
 		c = '.';
@@ -1528,10 +1583,8 @@ int yylex (void)
 	switch (c){
         case '0': case '1': case '2': case '3': case '4': case '5':
 	case '6': case '7': case '8': case '9': case '.': {
-		ExprTree *e = p_new (ExprTree);
-		Value *v = v_new ();
+		Value *v;
 
-		e->ref_count = 1;
 		is_float = c == '.';
 		p = parser_expr-1;
 		tmp = parser_expr;
@@ -1557,27 +1610,39 @@ int yylex (void)
 
 		/* Ok, we have skipped over a number, now load its value */
 		if (is_float) {
-			v->type = VALUE_FLOAT;
-			float_get_from_range (p, tmp, &v->v.v_float);
+			float_t f;
+			float_get_from_range (p, tmp, &f);
+			v = value_new_float (f);
 		} else {
-			v->type = VALUE_INTEGER;
-			int_get_from_range (p, tmp, &v->v.v_int);
+			int i;
+			int_get_from_range (p, tmp, &i);
+			v = value_new_int (i);
 		}
 
 		/* Return the value to the parser */
-		e->oper = OPER_CONSTANT;
-		e->u.constant = v;
-		yylval.tree = e;
-
+		yylval.tree = register_expr_allocation (expr_tree_new_constant (v));
 		parser_expr = tmp;
 		return NUMBER;
 	}
+
+	case '#' :
+	{
+		int offset = 0;
+		/* we already took the leading '#' off */
+		Value *err = value_is_error (parser_expr-1, &offset);
+		if (err != NULL) {
+			yylval.tree = register_expr_allocation (expr_tree_new_constant (err));
+			parser_expr += offset - 1;
+			return CONSTANT;
+		}
+	}
+	break;
+
 	case '\'':
 	case '"': {
 		char *string, *s;
-		int v;
 		char quotes_end = c;
-		
+
                 p = parser_expr;
                 while(*parser_expr && *parser_expr != quotes_end) {
                         if (*parser_expr == '\\' && parser_expr [1])
@@ -1588,7 +1653,7 @@ int yylex (void)
                         parser_error = PARSE_ERR_NO_QUOTE;
                         return ERROR;
                 }
-		
+
 		s = string = (char *) alloca (1 + parser_expr - p);
 		while (p != parser_expr){
 			if (*p== '\\'){
@@ -1596,21 +1661,21 @@ int yylex (void)
 				*s++ = *p++;
 			} else
 				*s++ = *p++;
-			
+
 		}
 		*s = 0;
 		parser_expr++;
 
-		v = try_symbol (string, FALSE);
-		return v;
+		return make_string_return (string, FALSE);
 	}
+
 	}
-	
+
 	if (isalpha ((unsigned char)c) || c == '_' || c == '$'){
 		const char *start = parser_expr - 1;
 		char *str;
 		int  len;
-		
+
 		while (isalnum ((unsigned char)*parser_expr) || *parser_expr == '_' ||
 		       *parser_expr == '$' || *parser_expr == '.')
 			parser_expr++;
@@ -1636,7 +1701,7 @@ int yylex (void)
 		}
 		return c;
 	}
-	
+
 	if (c == '>'){
 		if (*parser_expr == '='){
 			parser_expr++;
@@ -1644,7 +1709,7 @@ int yylex (void)
 		}
 		return c;
 	}
-	
+
 	return c;
 }
 
@@ -1657,153 +1722,9 @@ yyerror (char *s)
 	return 0;
 }
 
-static void
-alloc_register (AllocRec *a_info)
-{
-	alloc_list = g_list_prepend (alloc_list, a_info);
-}
-
-static void
-register_symbol (Symbol *sym)
-{
-	AllocRec *a_info = g_new (AllocRec, 1);
-
-	a_info->type = ALLOC_SYMBOL;
-	a_info->data = sym;
-	alloc_register (a_info);
-}
-
-void *
-alloc_buffer (int size)
-{
-	AllocRec *a_info = g_new (AllocRec, 1);
-	void *res = g_malloc (size);
-
-	a_info->type = ALLOC_BUFFER;
-	a_info->data = res;
-	alloc_register (a_info);
-
-	return res;
-}
-
-static Value *
-v_new (void)
-{
-	AllocRec *a_info = g_new (AllocRec, 1);
-	Value *res = g_new (Value, 1);
-
-	a_info->type = ALLOC_VALUE;
-	a_info->data = res;
-	alloc_register (a_info);
-
-	return res;
-}
-
-static void
-alloc_clean (void)
-{
-	GList *l = alloc_list;
-	
-	for (; l; l = l->next) {
-		AllocRec *rec = l->data;
-
-		switch (rec->type){
-		case ALLOC_BUFFER:
-			g_free (rec->data);
-			break;
-			
-		case ALLOC_SYMBOL:
-			symbol_unref ((Symbol *)rec->data);
-			break;
-
-		case ALLOC_VALUE:
-			value_release ((Value *)rec->data);
-			break;
-			
-		case ALLOC_LIST:
-			g_list_free ((GList *) rec->data);
-			break;
-		}
-		g_free (rec);
-	}
-
-	g_list_free (alloc_list);
-	alloc_list = NULL;
-}
-
-static void
-alloc_list_free (void)
-{
-	GList *l = alloc_list;
-	
-	for (; l; l = l->next)
-		g_free (l->data);
-
-	g_list_free (alloc_list);
-	alloc_list = NULL;
-}
-
-static void
-alloc_glist (GList *list)
-{
-	AllocRec *a_info = g_new (AllocRec, 1);
-
-	a_info->type = ALLOC_LIST;
-	a_info->data = list;
-	alloc_register (a_info);
-}
-
-static void
-forget (AllocType type, void *data)
-{
-	GList *l;
-
-	for (l = alloc_list; l; l = l->next) {
-		AllocRec *a_info = (AllocRec *) l->data;
-
-		if (a_info->type == type && a_info->data == data) {
-			alloc_list = g_list_remove_link (alloc_list, l);
-			g_list_free_1 (l);
-			g_free (a_info);
-			return;
-		}
-	}
-}
-
-static void
-forget_glist (GList *list)
-{
-	forget (ALLOC_LIST, list);
-}
-
-static void
-v_forget (Value *v)
-{
-	forget (ALLOC_VALUE, v);
-}
-
-static void
-forget_tree (ExprTree *tree)
-{
-	expr_tree_unref (tree);
-	forget (ALLOC_BUFFER, tree);
-}
-
-static void
-forget_array (GList *array)
-{
-	GList *l = array;
-
-	while (l) {
-		forget_glist (l->data);
-		l = g_list_next (l);
-	}
-	forget_glist (array);
-}
-
 ParseErr
 gnumeric_expr_parser (const char *expr, const ParsePosition *pp,
-		      const char **desired_format, ExprTree **result)
+		      char **desired_format, ExprTree **result)
 {
 	struct lconv *locinfo;
 
@@ -1837,15 +1758,48 @@ gnumeric_expr_parser (const char *expr, const ParsePosition *pp,
 		parser_array_col_separator = ',';
 	}
 
+	if (deallocate_stack == NULL)
+		deallocate_init ();
+
 	yyparse ();
 
-	if (parser_error == PARSE_OK)
-		alloc_list_free ();
-	else {
-		printf ("Parser error!\n");
-		alloc_clean ();
+	if (parser_error == PARSE_OK) {
+		deallocate_assert_empty ();
+		if (desired_format) {
+			char *format;
+			EvalPosition pos;
+
+			/*
+			 * FIXME: we *need* the sheet here, even if occasionally
+			 * it is NULL.
+			 */			   
+			pos.sheet = NULL;
+			pos.eval.col = pp->col;
+			pos.eval.row = pp->row;
+			format = auto_format_suggest (*parser_result, &pos);
+			if (format) {
+				/*
+				 * Override the format that came from a
+				 * constant somewhere inside.
+				 */
+				if (*desired_format)
+					g_free (*desired_format);
+				*desired_format = format;
+			}
+		}
+	} else {
+		fprintf (stderr, "Unable to parse '%s'\n", expr);
+		deallocate_all ();
 		*parser_result = NULL;
+		if (desired_format && *desired_format) {
+			g_free (*desired_format);
+			*desired_format = NULL;
+		}
 	}
 
+#ifndef KEEP_DEALLOCATION_STACK_BETWEEN_CALLS
+	deallocate_uninit ();
+#endif
+
 	return parser_error;
 }
