@@ -14,6 +14,7 @@
 #include "print-info.h"
 #include "format.h"
 #include "func.h"
+#include "datetime.h"
 
 static PrintHF predefined_formats [] = {
 	{ "",                 "",                             "" },
@@ -649,9 +650,9 @@ HFRenderInfo *
 hf_render_info_new (void)
 {
 	HFRenderInfo *hfi;
-	
+
 	hfi = g_new0 (HFRenderInfo, 1);
-	hfi->date_time = gnumeric_return_current_time ();
+	hfi->date_time = value_new_float (datetime_timet_to_serial_raw (time (NULL)));
 
 	return hfi;
 }
