@@ -166,6 +166,8 @@ sheet_selection_append (Sheet *sheet, int col, int row)
 static void
 sheet_selection_change (Sheet *sheet, SheetSelection *old, SheetSelection *new)
 {
+	GList *ranges, *l;
+	
 	if (range_equal (&old->user, &new->user))
 		return;
 
@@ -174,9 +176,24 @@ sheet_selection_change (Sheet *sheet, SheetSelection *old, SheetSelection *new)
 	sheet_set_selection (sheet, new);
 	sheet_selection_changed_hook (sheet);
 
-	sheet_redraw_selection (sheet, old);
-	sheet_redraw_selection (sheet, new);
-
+	if (range_overlap (&old->user, &new->user)){
+		/*
+		 * Compute the blocks that need to be repainted: those that
+		 * are in the complement of the intersection.
+		 */
+		ranges = range_fragment (&old->user, &new->user);
+		
+		for (l = ranges->next; l; l = l->next){
+			Range *range = l->data;
+			
+			sheet_redraw_range (sheet, range);
+		}
+		range_fragment_free (ranges);
+	} else {
+		sheet_redraw_selection (sheet, old);
+		sheet_redraw_selection (sheet, new);
+	}
+	
 	if (new->user.start.col != old->user.start.col ||
 	    new->user.end.col != old->user.end.col ||
 	    ((new->user.start.row == 0 && new->user.end.row == SHEET_MAX_ROWS-1) ^
@@ -289,6 +306,7 @@ sheet_selection_extend_horizontal (Sheet *sheet, int n, gboolean jump_to_boundar
 {
 	SheetSelection *ss;
 	SheetSelection old_selection;
+	int row, col;
 
 	g_return_if_fail (sheet != NULL);
 	g_return_if_fail (IS_SHEET (sheet));
@@ -297,19 +315,19 @@ sheet_selection_extend_horizontal (Sheet *sheet, int n, gboolean jump_to_boundar
 	old_selection = *ss;
 
 	if (ss->base.col < ss->user.end.col)
-		ss->user.end.col =
+		col = ss->user.end.col =
 		    sheet_find_boundary_horizontal (sheet,
-						    ss->user.end.col, ss->user.end.row,
+						    ss->user.end.col, row = ss->user.end.row,
 						    n, jump_to_boundaries);
 	else if (ss->base.col > ss->user.start.col || n < 0)
-		ss->user.start.col =
+		col = ss->user.start.col =
 		    sheet_find_boundary_horizontal (sheet,
-						    ss->user.start.col, ss->user.start.row,
+						    ss->user.start.col, row = ss->user.start.row,
 						    n, jump_to_boundaries);
 	else
-		ss->user.end.col =
+		col = ss->user.end.col =
 		    sheet_find_boundary_horizontal (sheet,
-						    ss->user.end.col,  ss->user.end.row,
+						    ss->user.end.col,  row = ss->user.end.row,
 						    n, jump_to_boundaries);
 
 	if (ss->user.end.col < ss->user.start.col) {
@@ -318,6 +336,7 @@ sheet_selection_extend_horizontal (Sheet *sheet, int n, gboolean jump_to_boundar
 		ss->user.end.col = tmp;
 	}
 	sheet_selection_change (sheet, &old_selection, ss);
+	sheet_make_cell_visible (sheet, col, row);
 }
 
 /*
@@ -331,6 +350,7 @@ sheet_selection_extend_vertical (Sheet *sheet, int n, gboolean jump_to_boundarie
 {
 	SheetSelection *ss;
 	SheetSelection old_selection;
+	int row, col;
 
 	g_return_if_fail (sheet != NULL);
 	g_return_if_fail (IS_SHEET (sheet));
@@ -339,19 +359,19 @@ sheet_selection_extend_vertical (Sheet *sheet, int n, gboolean jump_to_boundarie
 	old_selection = *ss;
 
 	if (ss->base.row < ss->user.end.row)
-		ss->user.end.row =
+		row = ss->user.end.row =
 		    sheet_find_boundary_vertical (sheet,
-						  ss->user.end.col, ss->user.end.row,
+						  col = ss->user.end.col, ss->user.end.row,
 						  n, jump_to_boundaries);
 	else if (ss->base.row > ss->user.start.row || n < 0)
-		ss->user.start.row =
+		row = ss->user.start.row =
 		    sheet_find_boundary_vertical (sheet,
-						  ss->user.start.col, ss->user.start.row,
+						  col = ss->user.start.col, ss->user.start.row,
 						  n, jump_to_boundaries);
 	else
-		ss->user.end.row =
+		row = ss->user.end.row =
 		    sheet_find_boundary_vertical (sheet,
-						  ss->user.end.col, ss->user.end.row,
+						  col = ss->user.end.col, ss->user.end.row,
 						  n, jump_to_boundaries);
 
 	if (ss->user.end.row < ss->user.start.row) {
@@ -360,6 +380,7 @@ sheet_selection_extend_vertical (Sheet *sheet, int n, gboolean jump_to_boundarie
 		ss->user.end.row = tmp;
 	}
 	sheet_selection_change (sheet, &old_selection, ss);
+	sheet_make_cell_visible (sheet, col, row);
 }
 
 void
@@ -648,7 +669,7 @@ sheet_selection_cut (CommandContext *context, Sheet *sheet)
 }
 
 static void
-sheet_selection_move (struct expr_relocate_info *rinfo)
+sheet_selection_move (ExprRelocateInfo *rinfo)
 {
 	Sheet * const sheet = rinfo->target_sheet;
 	Range r = rinfo->origin;
@@ -684,9 +705,9 @@ sheet_selection_paste (CommandContext *context, Sheet *sheet,
 	area = application_clipboard_area_get ();
 	content = application_clipboard_contents_get ();
 
-	/* If contents are null this was a cut */
 	if (content == NULL && area != NULL) {
-		struct expr_relocate_info rinfo;
+		/* Pasting a Cut */
+		ExprRelocateInfo rinfo;
 		Sheet * src_sheet = application_clipboard_sheet_get ();
 		Range const *sel = selection_first_range (sheet, FALSE);
 
@@ -718,119 +739,29 @@ sheet_selection_paste (CommandContext *context, Sheet *sheet,
 		rinfo.origin_sheet = src_sheet;
 		rinfo.target_sheet = sheet;
 
-		sheet_move_range      (NULL, &rinfo);
+		sheet_move_range      (context, &rinfo);
 		sheet_selection_move  (&rinfo);
 		application_clipboard_clear ();
-	} else
+	} else {
+		/* Pasting a Copy or from the X selection */
 		clipboard_paste_region (context, content,
 					sheet, dest_col, dest_row,
 					paste_flags, time);
-}
 
-static void
-cb_sheet_selection_clear (Sheet *sheet, 
-			  int start_col, int start_row,
-			  int end_col,   int end_row,
-			  void *context)
-{
-	sheet_clear_region (context, sheet, start_col, start_row,
-			    end_col,  end_row, TRUE);
-}
-
-/**
- * sheet_selection_clear:
- * @sheet:  The sheet where we operate
- *
- * Removes the contents and styles.
- **/
-void
-sheet_selection_clear (CommandContext *context, Sheet *sheet)
-{
-	selection_apply (sheet, &cb_sheet_selection_clear, TRUE, NULL);
-}
-
-static void
-cb_sheet_selection_clear_content (Sheet *sheet, 
-				  int start_col, int start_row,
-				  int end_col,   int end_row,
-				  void *context)
-{
-	sheet_clear_region_content (context, sheet, 
-				    start_col, start_row,
-				    end_col,   end_row);
-}
-
-/**
- * sheet_selection_clear_content:
- * @sheet:  The sheet where we operate
- *
- * Removes the contents of all the cells in the current selection.
- **/
-void
-sheet_selection_clear_content (CommandContext *context, Sheet *sheet)
-{
-	selection_apply (sheet, &cb_sheet_selection_clear_content,
-			 TRUE, context);
-	sheet_load_cell_val(sheet);
-}
-
-/**
- * sheet_selection_clear_comments:
- * @sheet:  The sheet where we operate
- *
- * Removes all of the comments on the range of selected cells.
- **/
-void
-sheet_selection_clear_comments (CommandContext *context, Sheet *sheet)
-{
-	selection_apply (sheet, &sheet_clear_region_comments, TRUE, NULL);
-}
-
-/**
- * sheet_selection_clear_formats:
- * @sheet:  The sheet where we operate
- *
- * Removes all formating
- **/
-void
-sheet_selection_clear_formats (CommandContext *context, Sheet *sheet)
-{
-	selection_apply (sheet, &sheet_clear_region_formats, TRUE, NULL);
-}
-
-/**
- * selection_apply:
- * @sheet: the sheet.
- * @func:  The function to apply.
- * @allow_intersection : Call the routine for the non-intersecting subregions.
- * @closure : A parameter to pass to each invocation of @func.
- *
- * Applies the specified function for all ranges in the selection.  Optionally
- * select whether to use the high level potentially over lapped ranges, rather
- * than the smaller system created non-intersection regions.
- */
-void
-selection_apply (Sheet *sheet, SelectionApplyFunc const func,
-		 gboolean allow_intersection,
-		 void * closure)
-{
-	GList *l;
-	GSList *proposed = NULL;
-
-	g_return_if_fail (sheet != NULL);
-	g_return_if_fail (IS_SHEET (sheet));
-
-	if (allow_intersection) {
-		for (l = sheet->selections; l != NULL; l = l->next) {
-			SheetSelection const *ss = l->data;
-
-			(*func) (sheet,
-				 ss->user.start.col, ss->user.start.row,
-				 ss->user.end.col, ss->user.end.row,
-				 closure);
-		}
-		return;
+		workbook_recalc (sheet->workbook);
 	}
+}
+
+/**
+ * selection_get_ranges:
+ * @sheet: the sheet.
+ * @allow_intersection : Divide the selection into nonoverlappign subranges.
+ */
+GSList *
+selection_get_ranges (Sheet * sheet, gboolean const allow_intersection)
+{
+	GList  *l;
+	GSList *proposed = NULL;
 
 #undef DEBUG_SELECTION
 #ifdef DEBUG_SELECTION
@@ -880,10 +811,14 @@ selection_apply (Sheet *sheet, SelectionApplyFunc const func,
 						    b->start.col, b->end.col);
 
 #ifdef DEBUG_SELECTION
-			fprintf (stderr, "col = %d\na = %d -> %d\nb = %d -> %d\n",
-				col_intersect,
-				a->start.col, a->end.col,
-				b->start.col, b->end.col);
+			fprintf (stderr, "col = %d\na = %s", col_intersect, col_name(a->start.col));
+			if (a->start.col != a->end.col)
+				fprintf (stderr, " -> %s", col_name(a->end.col));
+			fprintf (stderr, "\nb = %s", col_name(b->start.col));
+			if (b->start.col != b->end.col)
+				fprintf (stderr, " -> %s\n", col_name(b->end.col));
+			else
+				fputc ('\n', stderr);
 #endif
 
 			/* No intersection */
@@ -896,10 +831,14 @@ selection_apply (Sheet *sheet, SelectionApplyFunc const func,
 				segments_intersect (a->start.row, a->end.row,
 						    b->start.row, b->end.row);
 #ifdef DEBUG_SELECTION
-			fprintf (stderr, "row = %d\na = %d -> %d\nb = %d -> %d\n",
-				row_intersect,
-				a->start.row, a->end.row,
-				b->start.row, b->end.row);
+			fprintf (stderr, "row = %d\na = %d", row_intersect, a->start.row +1);
+			if (a->start.row != a->end.row)
+				fprintf (stderr, " -> %d", a->end.row +1);
+			fprintf (stderr, "\nb = %d", b->start.row +1);
+			if (b->start.row != b->end.row)
+				fprintf (stderr, " -> %d\n", b->end.row +1);
+			else
+				fputc ('\n', stderr);
 #endif
 
 			/* No intersection */
@@ -914,9 +853,13 @@ selection_apply (Sheet *sheet, SelectionApplyFunc const func,
 					row_intersect = 4;
 				if (row_intersect == 4 || row_intersect == 2)
 					col_intersect = row_intersect;
+				else
+					col_intersect = 4;
 			} else if (row_intersect == 5) {
 				if (col_intersect == 4 || col_intersect == 2)
 					row_intersect = col_intersect;
+				else
+					row_intersect = 4;
 			}
 
 			/* Cross product of intersection cases */
@@ -933,7 +876,7 @@ selection_apply (Sheet *sheet, SelectionApplyFunc const func,
 
 				case 3 : /* overlap top */
 					/* Shrink existing range */
-					b->end.row = a->start.row - 1;
+					b->start.row = a->end.row + 1;
 					break;
 
 				case 2 : /* b contains a */
@@ -941,19 +884,20 @@ selection_apply (Sheet *sheet, SelectionApplyFunc const func,
 						/* Shrink existing range */
 						a->end.col = b->start.col - 1;
 						break;
-					} else if (a->start.col != b->start.col &&
-						   b->start.col > 0) {
+					}
+					if (a->start.col != b->start.col) {
 						/* Split existing range */
 						tmp = range_copy (a);
 						tmp->end.col = b->start.col - 1;
-						clear = g_slist_prepend (clear,
-									 tmp);
+						clear = g_slist_prepend (clear, tmp);
 					}
-					/* Fall through to do bottom segment */
+					/* Shrink existing range */
+					a->start.col = b->end.col + 1;
+					break;
 
 				case 1 : /* overlap bottom */
 					/* Shrink existing range */
-					a->start.col = b->end.col + 1;
+					a->start.row = b->end.row + 1;
 					break;
 
 				default :
@@ -974,8 +918,7 @@ selection_apply (Sheet *sheet, SelectionApplyFunc const func,
 						tmp = range_copy (a);
 						tmp->start.col = b->start.col;
 						tmp->end.row = b->start.row - 1;
-						clear = g_slist_prepend (clear,
-									 tmp);
+						clear = g_slist_prepend (clear, tmp);
 					}
 					/* fall through */
 
@@ -990,8 +933,7 @@ selection_apply (Sheet *sheet, SelectionApplyFunc const func,
 						tmp = range_copy (a);
 						tmp->start.col = b->start.col;
 						tmp->start.row = b->end.row + 1;
-						clear = g_slist_prepend (clear,
-									 tmp);
+						clear = g_slist_prepend (clear, tmp);
 					}
 
 					/* shrink the left segment */
@@ -1026,8 +968,8 @@ selection_apply (Sheet *sheet, SelectionApplyFunc const func,
 						/* Shrink existing range */
 						a->end.row = b->start.row - 1;
 						break;
-					} else if (a->start.row != b->start.row &&
-						   b->start.row > 0) {
+					}
+					if (a->start.row != b->start.row) {
 						/* Split region */
 						tmp = range_copy (a);
 						tmp->end.row = b->start.row - 1;
@@ -1057,6 +999,7 @@ selection_apply (Sheet *sheet, SelectionApplyFunc const func,
 					tmp = range_copy (a);
 					tmp->end.col = b->end.col;
 					tmp->end.row = b->start.row - 1;
+					clear = g_slist_prepend (clear, tmp);
 					/* fall through */
 
 				case 2 : /* b contains a */
@@ -1090,20 +1033,57 @@ selection_apply (Sheet *sheet, SelectionApplyFunc const func,
 		proposed = (b != NULL) ? g_slist_prepend (clear, b) : clear;
 	}
 
-	while (proposed != NULL) {
-		/* pop the 1st element off the list */
-		Range *r = proposed->data;
-		proposed = g_slist_remove (proposed, r);
+	return proposed;
+}
+
+/**
+ * selection_apply:
+ * @sheet: the sheet.
+ * @func:  The function to apply.
+ * @allow_intersection : Call the routine for the non-intersecting subregions.
+ * @closure : A parameter to pass to each invocation of @func.
+ *
+ * Applies the specified function for all ranges in the selection.  Optionally
+ * select whether to use the high level potentially over lapped ranges, rather
+ * than the smaller system created non-intersection regions.
+ */
+void
+selection_apply (Sheet *sheet, SelectionApplyFunc const func,
+		 gboolean allow_intersection,
+		 void * closure)
+{
+	GList *l;
+	GSList *proposed = NULL;
+
+	g_return_if_fail (sheet != NULL);
+	g_return_if_fail (IS_SHEET (sheet));
+
+	if (allow_intersection) {
+		for (l = sheet->selections; l != NULL; l = l->next) {
+			SheetSelection const *ss = l->data;
+
+			(*func) (sheet,
+				 ss->user.start.col, ss->user.start.row,
+				 ss->user.end.col, ss->user.end.row,
+				 closure);
+		}
+	} else {
+		proposed = selection_get_ranges (sheet, allow_intersection);
+		while (proposed != NULL) {
+			/* pop the 1st element off the list */
+			Range *r = proposed->data;
+			proposed = g_slist_remove (proposed, r);
 
 #ifdef DEBUG_SELECTION
-		range_dump (r);
+			range_dump (r);
 #endif
 
-		(*func) (sheet,
-			 r->start.col, r->start.row,
-			 r->end.col, r->end.row,
-			 closure);
-		g_free (r);
+			(*func) (sheet,
+				 r->start.col, r->start.row,
+				 r->end.col, r->end.row,
+				 closure);
+			g_free (r);
+		}
 	}
 }
 
@@ -1256,4 +1236,3 @@ sheet_selection_height_update (Sheet *sheet)
 				      &cb_set_row_height, NULL);
 	}
 }
-
