@@ -23,10 +23,6 @@ void        sheet_cell_list_free         (CellList *cell_list);
 char       *sheet_selection_to_string    (Sheet *sheet, gboolean include_sheet_name_prefix);
 
 /* Operations on the selection */
-void        sheet_selection_clear             (CommandContext *context, Sheet *sheet);
-void        sheet_selection_clear_content     (CommandContext *context, Sheet *sheet);
-void        sheet_selection_clear_comments    (CommandContext *context, Sheet *sheet);
-void        sheet_selection_clear_formats     (CommandContext *context, Sheet *sheet);
 void        sheet_selection_height_update     (Sheet *sheet);
 
 /* Cut/Copy/Paste on the workbook selection */
@@ -58,6 +54,7 @@ typedef void (*SelectionApplyFunc) (Sheet *sheet,
 void selection_apply (Sheet *sheet, SelectionApplyFunc const func,
 		      gboolean allow_intersection,
 		      void *closure);
+GSList * selection_get_ranges (Sheet * sheet, gboolean const allow_intersection);
 
 /* export the selection */
 CellList   *selection_to_list      (Sheet *sheet, gboolean allow_intersection);
