@@ -839,7 +839,7 @@ sheet_style_insert_colrow (Sheet *sheet, int pos, int count,
 }
 
 void
-sheet_style_relocate (const struct expr_relocate_info *rinfo)
+sheet_style_relocate (const ExprRelocateInfo *rinfo)
 {
 	GList *stored_regions = NULL;
 	GList *l, *next;
@@ -1469,7 +1469,7 @@ sheet_style_get_extent (Range *r, Sheet const *sheet)
 {
 	GList   *l;
 
-	for (l = STYLE_LIST (sheet); l; l = g_list_next (l)) {
+	for (l = STYLE_LIST (sheet); l && l->next; l = g_list_next (l)) {
 		Range *sr = (Range *)l->data;
 
 		if (!range_is_infinite (sr))
