@@ -23,6 +23,7 @@
 #include "mstyle.h"
 #include "application.h"
 #include "command-context.h"
+#include "commands.h"
 #ifdef ENABLE_BONOBO
 #    include <libgnorba/gnorba.h>
 #endif
@@ -38,6 +39,18 @@
 #define COLROW_GET_SEGMENT(seg_array, i) \
 	(g_ptr_array_index ((seg_array)->info, COLROW_SEGMENT_INDEX(i)))
 
+void
+sheet_adjust_preferences (Sheet const *sheet)
+{
+	GList *l;
+
+	for (l = sheet->sheet_views; l; l = l->next){
+		SheetView *sheet_view = l->data;
+
+		sheet_view_adjust_preferences (sheet_view);
+	}
+}
+
 void
 sheet_redraw_all (Sheet const *sheet)
 {
@@ -86,6 +99,7 @@ col_row_info_init (ColRowInfo *cri, double points)
 	cri->margin_a = 0;
 	cri->margin_b = 0;
 
+	cri->hard_size = FALSE;
 	cri->spans = NULL;
 }
 
@@ -197,8 +211,9 @@ sheet_new (Workbook *wb, const char *name)
 			      COLROW_SEGMENT_INDEX (SHEET_MAX_ROWS-1)+1);
 	sheet->print_info = print_info_new ();
 
-	sheet->cell_hash = g_hash_table_new (cell_hash,
-					     (GCompareFunc)&cell_compare);
+	sheet->cell_hash  = g_hash_table_new (cell_hash,
+					      (GCompareFunc)&cell_compare);
+	sheet->deps       = dependency_data_new ();
 
 	mstyle = mstyle_new_default ();
 	sheet_style_attach (sheet, sheet_get_full_range (), mstyle);
@@ -222,6 +237,13 @@ sheet_new (Workbook *wb, const char *name)
 	sheet->pristine = TRUE;
 	sheet->modified = FALSE;
 
+	/* Init preferences */
+	sheet->display_formulas = FALSE;
+	sheet->display_zero = TRUE;
+	sheet->show_grid = TRUE;
+	sheet->show_col_header = TRUE;
+	sheet->show_row_header = TRUE;
+
 	return sheet;
 }
 
@@ -269,9 +291,12 @@ sheet_foreach_colrow (Sheet *sheet, ColRowCollection *infos,
 static gboolean
 sheet_compute_col_row_new_size (Sheet *sheet, ColRowInfo *ci, void *data)
 {
-	double pix_per_unit = sheet->last_zoom_factor_used;
+	double const pix_per_unit = sheet->last_zoom_factor_used;
+	gboolean const hidden = (ci->pixels < 0);
 
 	ci->pixels = (ci->units + ci->margin_a_pt + ci->margin_b_pt) * pix_per_unit;
+	if (hidden)
+		ci->pixels *= -1;
 
 	/*
 	 * Ensure that there is at least 1 pixel around every cell so that we
@@ -474,9 +499,10 @@ static void
 colrow_set_units (Sheet *sheet, ColRowInfo *info)
 {
 	double const pix = sheet->last_zoom_factor_used;
+	int p = info->pixels;
+	if (p < 0) p = -p;
 
-	info->units = (info->pixels -
-		       (info->margin_a + info->margin_b - 1)) / pix;
+	info->units = (p - (info->margin_a + info->margin_b - 1)) / pix;
 }
 
 static void
@@ -566,19 +592,19 @@ sheet_get_extent_cb (gpointer key, gpointer value, gpointer data)
 {
 	Cell *cell = (Cell *) value;
 	
-	if ( cell && !cell_is_blank (cell)) {
+	if (cell && !cell_is_blank (cell)) {
 		Range *range = (Range *)data;
 
-		if ( cell->row->pos < range->start.row)
+		if (cell->row->pos < range->start.row)
 			range->start.row = cell->row->pos;
 
-		if ( cell->row->pos > range->end.row)
+		if (cell->row->pos > range->end.row)
 			range->end.row = cell->row->pos;
 
-		if ( cell->col->pos < range->start.col)
+		if (cell->col->pos < range->start.col)
 			range->start.col = cell->col->pos;
 
-		if ( cell->col->pos > range->end.col)
+		if (cell->col->pos > range->end.col)
 			range->end.col = cell->col->pos;
 	}
 }
@@ -624,11 +650,15 @@ sheet_get_extent (Sheet const *sheet)
 		r.end.col   = sheet->cols.max_used;
 	if (r.start.col > r.end.col)
 		r.start.col = r.end.col;
+	if (r.start.col < 0)
+		r.start.col = 0;
 
 	if (r.end.row   > sheet->rows.max_used)
 		r.end.row   = sheet->rows.max_used;
 	if (r.start.row > r.end.row)
 		r.start.row = r.end.row;
+	if (r.start.row < 0)
+		r.start.row = 0;
 
 	return r;
 }
@@ -643,7 +673,7 @@ sheet_row_info_set_height (Sheet *sheet, ColRowInfo *ri, int height, gboolean he
 	if (height_set_by_user)
 		ri->hard_size = TRUE;
 
-	ri->pixels = height;
+	ri->pixels = (ri->pixels >= 0) ? height : -height;
 	colrow_set_units (sheet, ri);
 
 	sheet_compute_visible_ranges (sheet);
@@ -698,6 +728,7 @@ void
 sheet_row_set_internal_height (Sheet *sheet, ColRowInfo *ri, double height)
 {
 	double pix;
+	gboolean hidden;
 
 	g_return_if_fail (sheet != NULL);
 	g_return_if_fail (IS_SHEET (sheet));
@@ -708,8 +739,11 @@ sheet_row_set_internal_height (Sheet *sheet, ColRowInfo *ri, double height)
 	if (ri->units == height)
 		return;
 
+	hidden = ri->pixels < 0;
 	ri->units = height;
 	ri->pixels = (ri->units * pix) + (ri->margin_a + ri->margin_b - 1);
+	if (hidden)
+		ri->pixels *= -1;
 
 	sheet_compute_visible_ranges (sheet);
 	sheet_reposition_comments_from_row (sheet, ri->pos);
@@ -729,6 +763,7 @@ void
 sheet_col_set_internal_width (Sheet *sheet, ColRowInfo *ci, double width)
 {
 	double pix;
+	gboolean hidden;
 
 	g_return_if_fail (sheet != NULL);
 	g_return_if_fail (IS_SHEET (sheet));
@@ -739,8 +774,11 @@ sheet_col_set_internal_width (Sheet *sheet, ColRowInfo *ci, double width)
 	if (ci->units == width)
 		return;
 
+	hidden = ci->pixels < 0;
 	ci->units = width;
 	ci->pixels = (ci->units * pix) + (ci->margin_a + ci->margin_b - 1);
+	if (hidden)
+		ci->pixels *= -1;
 
 	sheet_compute_visible_ranges (sheet);
 	sheet_reposition_comments_from_col (sheet, ci->pos);
@@ -748,7 +786,8 @@ sheet_col_set_internal_width (Sheet *sheet, ColRowInfo *ci, double width)
 }
 
 void
-sheet_row_set_height_units (Sheet *sheet, int row, double height, gboolean height_set_by_user)
+sheet_row_set_height_units (Sheet *sheet, int row, double height,
+			    gboolean set_by_user)
 {
 	ColRowInfo *ri;
 	
@@ -763,14 +802,15 @@ sheet_row_set_height_units (Sheet *sheet, int row, double height, gboolean heigh
 		sheet_row_add (sheet, ri);
 	}
 
-	if (height_set_by_user)
+	if (set_by_user)
 		ri->hard_size = TRUE;
 	
 	sheet_row_set_internal_height (sheet, ri, height);
 }
 
 void
-sheet_col_set_width_units (Sheet *sheet, int col, double width)
+sheet_col_set_width_units (Sheet *sheet, int col, double width,
+			   gboolean set_by_user)
 {
 	ColRowInfo *ci;
 	
@@ -785,6 +825,8 @@ sheet_col_set_width_units (Sheet *sheet, int col, double width)
 		sheet_col_add (sheet, ci);
 	}
 
+	if (set_by_user)
+		ci->hard_size = TRUE;
 	sheet_col_set_internal_width (sheet, ci, width);
 }
 
@@ -963,7 +1005,7 @@ sheet_col_info_set_width (Sheet *sheet, ColRowInfo *ci, int width)
 	g_return_if_fail (ci != NULL);
 
 
-	ci->pixels = width;
+	ci->pixels = (ci->pixels >= 0) ? width : -width;
 	colrow_set_units (sheet, ci);
 
 	sheet_compute_visible_ranges (sheet);
@@ -1017,7 +1059,9 @@ sheet_col_get_distance (Sheet const *sheet, int from, int to)
 	/* Do not use sheet_foreach_colrow, it ignores empties */
 	for (i = from ; i < to ; ++i) {
 		ColRowInfo const *ci = sheet_col_get_info (sheet, i);
-		pixels += ci->pixels;
+		int const tmp = ci->pixels;
+		if (tmp > 0)
+			pixels += ci->pixels;
 	}
 
 	return pixels;
@@ -1109,7 +1153,9 @@ sheet_row_get_distance (Sheet const *sheet, int from, int to)
 	/* Do not use sheet_foreach_colrow, it ignores empties */
 	for (i = from ; i < to ; ++i) {
 		ColRowInfo const *ri = sheet_row_get_info (sheet, i);
-		pixels += ri->pixels;
+		int const tmp = ri->pixels;
+		if (tmp > 0)
+			pixels += ri->pixels;
 	}
 
 	return pixels;
@@ -1132,6 +1178,7 @@ sheet_col_get_unit_distance (Sheet const *sheet, int from, int to)
 	/* Do not use sheet_foreach_colrow, it ignores empties */
 	for (i = from ; i < to ; ++i) {
 		ColRowInfo const *ci = sheet_col_get_info (sheet, i);
+		int const tmp = ci->pixels;
 
 		/* I do not know why we subtract 1, but it is required to get
 		 * things to match.
@@ -1139,7 +1186,8 @@ sheet_col_get_unit_distance (Sheet const *sheet, int from, int to)
 		 * We take to floor of the width to get capture the rounding the
 		 * effect of rounding to pixels.
 		 */
-		units += (int)(ci->units + ci->margin_a_pt + ci->margin_b_pt) - 1;
+		if (tmp > 0)
+			units += (int)(ci->units + ci->margin_a_pt + ci->margin_b_pt) - 1;
 	}
 	
 	return units;
@@ -1162,6 +1210,7 @@ sheet_row_get_unit_distance (Sheet const *sheet, int from, int to)
 	/* Do not use sheet_foreach_colrow, it ignores empties */
 	for (i = from ; i < to ; ++i) {
 		ColRowInfo const *ri = sheet_row_get_info (sheet, i);
+		int const tmp = ri->pixels;
 
 		/* I do not know why we subtract 1, but it is required to get
 		 * things to match.
@@ -1169,7 +1218,8 @@ sheet_row_get_unit_distance (Sheet const *sheet, int from, int to)
 		 * We take to floor of the width to get capture the rounding the
 		 * effect of rounding to pixels.
 		 */
-		units += (int)(ri->units + ri->margin_a_pt + ri->margin_b_pt) - 1;
+		if (tmp > 0)
+			units += (int)(ri->units + ri->margin_a_pt + ri->margin_b_pt) - 1;
 	}
 	
 	return units;
@@ -1180,27 +1230,27 @@ sheet_update_auto_expr (Sheet *sheet)
 {
 	Value *v;
 	Workbook *wb = sheet->workbook;
-	FunctionEvalInfo ei;
 
 	g_return_if_fail (sheet != NULL);
 	g_return_if_fail (IS_SHEET (sheet));
 
 	/* defaults */
 	v = NULL;
-	func_eval_info_init (&ei, sheet, 0, 0);
 
-	if (wb->auto_expr)
-		v = eval_expr (&ei, wb->auto_expr);
+	if (wb->auto_expr) {
+		EvalPosition pos;
+		eval_pos_init (&pos, sheet, 0, 0);
+		v = eval_expr (&pos, wb->auto_expr);
+		if (v) {
+			char *s;
 
-	if (v) {
-		char *s;
-
-		s = value_get_as_string (v);
-		workbook_auto_expr_label_set (wb, s);
-		g_free (s);
-		value_release (v);
-	} else
-		workbook_auto_expr_label_set (wb, _("Internal ERROR"));
+			s = value_get_as_string (v);
+			workbook_auto_expr_label_set (wb, s);
+			g_free (s);
+			value_release (v);
+		} else
+			workbook_auto_expr_label_set (wb, _("Internal ERROR"));
+	}
 }
 
 /*
@@ -1255,12 +1305,8 @@ sheet_set_text (Sheet *sheet, char const *text, Range const * r)
 	g_return_if_fail (sheet != NULL);
 	g_return_if_fail (IS_SHEET (sheet));
 
-	/*
-	 * Figure out if a format matches, and for sanity compare that to
-	 * a rendered version of the text, if they compare equally, then
-	 * use that.
-	 */
-	if (*text != '=') {
+	/* If its not a formula see if there is a prefered format. */
+	if (!gnumeric_char_start_expr_p (*text) || text[1] == '\0') {
 		closure_set_cell_value	closure;
 		char *end;
 
@@ -1298,7 +1344,8 @@ sheet_set_current_value (Sheet *sheet)
 	str = gtk_entry_get_text (GTK_ENTRY (sheet->workbook->ea_input));
 	g_return_if_fail (str != NULL);
 
-	if (*str == '@'){
+	/* A hack to accept Lotus 123 style formula entries */
+	if (gnumeric_char_start_expr_p (*str) && *str != '=' && str[1] != '\0') {
 		char *new_text = g_strdup (str);
 
 		*new_text = *str = '=';
@@ -1308,7 +1355,11 @@ sheet_set_current_value (Sheet *sheet)
 
 	r.start.col = r.end.col = sheet->cursor_col;
 	r.start.row = r.end.row = sheet->cursor_row;
-	sheet_set_text (sheet, str, &r);
+
+	/* TODO : Get a context */
+	/* Store the old value for undo */
+	if (!cmd_set_text (NULL, sheet, &r.start, str, sheet->editing_saved_text))
+		sheet_set_text (sheet, str, &r);
 
 	for (l = sheet->sheet_views; l; l = l->next){
 		GnumericSheet *gsheet = GNUMERIC_SHEET_VIEW (l->data);
@@ -1575,15 +1626,21 @@ sheet_redraw_cell_region (Sheet const *sheet,
 }
 
 void
-sheet_redraw_selection (Sheet const *sheet, SheetSelection const *ss)
+sheet_redraw_range (Sheet const *sheet, Range const *range)
 {
 	g_return_if_fail (sheet != NULL);
 	g_return_if_fail (IS_SHEET (sheet));
-	g_return_if_fail (ss != NULL);
+	g_return_if_fail (range != NULL);
 
 	sheet_redraw_cell_region (sheet,
-				  ss->user.start.col, ss->user.start.row,
-				  ss->user.end.col, ss->user.end.row);
+				  range->start.col, range->start.row,
+				  range->end.col, range->end.row);
+}
+
+void
+sheet_redraw_selection (Sheet const *sheet, SheetSelection const *ss)
+{
+	sheet_redraw_range (sheet, &ss->user);
 }
 
 int
@@ -1612,6 +1669,13 @@ sheet_col_check_bound (int col, int diff)
 	return new_val;
 }
 
+static gboolean
+sheet_col_is_hidden (Sheet *sheet, int const col)
+{
+	ColRowInfo const * const res = sheet_col_get (sheet, col);
+	return (res != NULL && res->pixels < 0);
+}
+
 /*
  * sheet_find_boundary_horizontal
  *
@@ -1645,10 +1709,10 @@ sheet_find_boundary_horizontal (Sheet *sheet, int start_col, int row,
 		keep_looking = FALSE;
 
 		if (new_col < 0)
-			new_col = 0;
-		else if (new_col >= SHEET_MAX_COLS)
-			new_col = SHEET_MAX_COLS-1;
-		else if (jump_to_boundaries) {
+			return 0;
+		if (new_col >= SHEET_MAX_COLS)
+			return SHEET_MAX_COLS-1;
+		if (jump_to_boundaries) {
 			keep_looking = (cell_is_blank (sheet_cell_get (sheet, new_col, row)) == find_first);
 			if (keep_looking)
 				prev_col = new_col;
@@ -1663,11 +1727,18 @@ sheet_find_boundary_horizontal (Sheet *sheet, int start_col, int row,
 					new_col = prev_col;
 			}
 		}
-	} while (keep_looking);
+	} while (keep_looking || sheet_col_is_hidden (sheet, new_col));
 
 	return new_col;
 }
 
+static gboolean
+sheet_row_is_hidden (Sheet *sheet, int const row)
+{
+	ColRowInfo const * const res = sheet_row_get (sheet, row);
+	return (res != NULL && res->pixels < 0);
+}
+
 /*
  * sheet_find_boundary_vertical
  *
@@ -1700,10 +1771,10 @@ sheet_find_boundary_vertical (Sheet *sheet, int col, int start_row,
 		keep_looking = FALSE;
 
 		if (new_row < 0)
-			new_row = 0;
-		else if (new_row > SHEET_MAX_ROWS-1)
-			new_row = SHEET_MAX_ROWS-1;
-		else if (jump_to_boundaries) {
+			return 0;
+		if (new_row > SHEET_MAX_ROWS-1)
+			return SHEET_MAX_ROWS-1;
+		if (jump_to_boundaries) {
 			keep_looking = (cell_is_blank (sheet_cell_get (sheet, col, new_row)) == find_first);
 			if (keep_looking)
 				prev_row = new_row;
@@ -1718,7 +1789,7 @@ sheet_find_boundary_vertical (Sheet *sheet, int col, int start_row,
 					new_row = prev_row;
 			}
 		}
-	} while (keep_looking);
+	} while (keep_looking || sheet_row_is_hidden (sheet, new_row));
 
 	return new_row;
 }
@@ -1999,7 +2070,7 @@ sheet_row_get (Sheet const *sheet, int const pos)
 }
 
 /**
- * sheet_row_get:
+ * sheet_row_fetch:
  *
  * Returns an allocated row:  either an existing one, or a fresh copy
  */
@@ -2173,9 +2244,11 @@ sheet_cell_add (Sheet *sheet, Cell *cell, int col, int row)
 	cell->row   = sheet_row_fetch (sheet, row);
 
 	cell_realize (cell);
-	cell_calc_dimensions (cell);
+	cell_calc_dimensions  (cell);
 
 	sheet_cell_add_to_hash (sheet, cell);
+
+	cell_add_dependencies (cell);
 }
 
 Cell *
@@ -2201,7 +2274,9 @@ sheet_cell_remove_from_hash (Sheet *sheet, Cell *cell)
 	cellpos.col = cell->col->pos;
 	cellpos.row = cell->row->pos;
 
-	cell_unregister_span (cell);
+	cell_unregister_span   (cell);
+	cell_drop_dependencies (cell);
+
 	if (g_hash_table_lookup_extended (sheet->cell_hash, &cellpos, &original_key, NULL)) {
 		g_hash_table_remove (sheet->cell_hash, &cellpos);
 		g_free (original_key);
@@ -2475,12 +2550,12 @@ sheet_destroy (Sheet *sheet)
 
 	sheet_destroy_contents (sheet);
 
-	if (sheet->dependency_hash != NULL) {
-		if (g_hash_table_size (sheet->dependency_hash) != 0)
-			g_warning ("Dangling dependencies");
-		g_hash_table_destroy (sheet->dependency_hash);
-		sheet->dependency_hash = NULL;
-	}
+	/* Clear the cliboard to avoid dangling references to the deleted sheet */
+	if (sheet == application_clipboard_sheet_get ())
+		application_clipboard_clear ();
+
+	dependency_data_destroy (sheet);
+	sheet->deps = NULL;
 
 	sheet_destroy_styles (sheet);
 
@@ -2488,13 +2563,11 @@ sheet_destroy (Sheet *sheet)
 
 	expr_name_clean_sheet (sheet);
 
-	if (sheet->dependency_hash)
-		g_hash_table_destroy (sheet->dependency_hash);
-
 	sheet->signature = 0;
 	g_free (sheet);
 }
 
+/*****************************************************************************/
 
 struct sheet_clear_region_callback_data
 {
@@ -2537,141 +2610,82 @@ assemble_clear_cell_list (Sheet *sheet, int col, int row, Cell *cell,
 	return NULL;
 }
 
-/**
- * sheet_clear_region:
- *
- * Clears are region of cells
- *
- * @clearStyles : If this is TRUE then styles are erased.
- *
- * We assemble a list of cells to destroy, since we will be making changes
- * to the structure being manipulated by the sheet_cell_foreach_range routine
- */
-void
-sheet_clear_region (CommandContext *context, Sheet *sheet,
-		    int start_col, int start_row, int end_col, int end_row,
-		    gboolean const clearStyles)
-{
-	struct sheet_clear_region_callback_data cb;
-	GList *l;
-
-	g_return_if_fail (sheet != NULL);
-	g_return_if_fail (IS_SHEET (sheet));
-	g_return_if_fail (start_col <= end_col);
-	g_return_if_fail (start_row <= end_row);
-
-	/* Queue a redraw for the cells being removed */
-	sheet_redraw_cell_region (sheet, start_col, start_row, end_col, end_row);
-
-	cb.r.start.col = start_col;
-	cb.r.start.row = start_row;
-	cb.r.end.col = end_col;
-	cb.r.end.row = end_row;
-	cb.l = NULL;
-
-	/* Clear the style in the region (new_default will ref the style for us). */
-	if (clearStyles)
-		sheet_style_attach (sheet, cb.r, mstyle_new_default ());
-
-	if (sheet_cell_foreach_range (sheet, TRUE,
-				       start_col, start_row, end_col, end_row,
-				       assemble_clear_cell_list, &cb) == NULL) {
-		cb.l = g_list_reverse (cb.l);
-		cell_freeze_redraws ();
-
-		for (l = cb.l; l; l = l->next){
-			Cell *cell = l->data;
-
-			sheet_cell_remove (sheet, cell);
-			cell_destroy (cell);
-		}
-		cell_thaw_redraws ();
-		workbook_recalc (sheet->workbook);
-	} else 
-		gnumeric_error_splits_array (context);
-	g_list_free (cb.l);
-}
-
-/**
- * sheet_clear_region_content:
- * @sheet:     The sheet on which we operate
- * @start_col: starting column
- * @start_row: starting row
- * @end_col:   end column
- * @end_row:   end row
- *
- * Clears the contents in a region of cells
- */
-void
-sheet_clear_region_content (CommandContext *context, Sheet *sheet,
-			    int start_col, int start_row,
-			    int end_col, int end_row)
-{
-	sheet_clear_region (context, sheet,
-			    start_col, start_row,
-			    end_col, end_row, FALSE);
-}
-
 static Value *
-clear_cell_comments (Sheet *sheet, int col, int row, Cell *cell, void *user_data)
+cb_clear_cell_comments (Sheet *sheet, int col, int row, Cell *cell, void *user_data)
 {
 	cell_comment_destroy (cell);
 	return NULL;
 }
 
 /**
- * sheet_clear_region_comments:
- * @sheet:     The sheet on which we operate
- * @start_col: starting column
- * @start_row: starting row
- * @end_col:   end column
- * @end_row:   end row
+ * sheet_clear_region:
  *
- * Removes all of the comments in the cells in the specified range.
- **/
+ * Clears are region of cells
+ *
+ * @clear_flags : If this is TRUE then styles are erased.
+ *
+ * We assemble a list of cells to destroy, since we will be making changes
+ * to the structure being manipulated by the sheet_cell_foreach_range routine
+ */
 void
-sheet_clear_region_comments (Sheet *sheet, int start_col, int start_row, int end_col, int end_row,
-			     void *closure)
+sheet_clear_region (CommandContext *context, Sheet *sheet,
+		    int const start_col, int const start_row,
+		    int const end_col, int const end_row,
+		    int const clear_flags)
 {
+	struct sheet_clear_region_callback_data cb;
+
 	g_return_if_fail (sheet != NULL);
 	g_return_if_fail (IS_SHEET (sheet));
 	g_return_if_fail (start_col <= end_col);
 	g_return_if_fail (start_row <= end_row);
 
-	/* Queue a redraw for the region being redrawn */
-	sheet_redraw_cell_region (sheet, start_col, start_row, end_col, end_row);
+	cb.r.start.col = start_col;
+	cb.r.start.row = start_row;
+	cb.r.end.col = end_col;
+	cb.r.end.row = end_row;
 
-	sheet_cell_foreach_range (
-		sheet, TRUE,
-		start_col, start_row,
-		end_col,   end_row,
-		clear_cell_comments, NULL);
+	/* Queue a redraw for cells being modified */
+	if (clear_flags & (CLEAR_VALUES|CLEAR_FORMATS))
+		sheet_redraw_cell_region (sheet,
+					  start_col, start_row,
+					  end_col, end_row);
+
+	/* Clear the style in the region (new_default will ref the style for us). */
+	if (clear_flags & CLEAR_FORMATS)
+		sheet_style_attach (sheet, cb.r, mstyle_new_default ());
+
+	if (clear_flags & CLEAR_COMMENTS)
+		sheet_cell_foreach_range (sheet, TRUE,
+					  start_col, start_row,
+					  end_col,   end_row,
+					  cb_clear_cell_comments, NULL);
+
+	if (clear_flags & CLEAR_VALUES) {
+		GList *l;
+		cb.l = NULL;
+
+		if (sheet_cell_foreach_range (sheet, TRUE,
+					      start_col, start_row, end_col, end_row,
+					      assemble_clear_cell_list, &cb) == NULL) {
+			cb.l = g_list_reverse (cb.l);
+			cell_freeze_redraws ();
+
+			for (l = cb.l; l; l = l->next){
+				Cell *cell = l->data;
+
+				sheet_cell_remove (sheet, cell);
+				cell_destroy (cell);
+			}
+			cell_thaw_redraws ();
+		} else 
+		    gnumeric_error_splits_array (context);
+
+		g_list_free (cb.l);
+	}
 }
 
-static Value *
-clear_cell_format (Sheet *sheet, int col, int row, Cell *cell, void *user_data)
-{
-	cell_set_format (cell, "General");
-	return NULL;
-}
-
-void
-sheet_clear_region_formats (Sheet *sheet, int start_col, int start_row, int end_col, int end_row,
-			    void *closure)
-{
-	g_return_if_fail (sheet != NULL);
-	g_return_if_fail (IS_SHEET (sheet));
-	g_return_if_fail (start_col <= end_col);
-	g_return_if_fail (start_row <= end_row);
-
-	/* Queue a draw for the region being modified */
-	sheet_redraw_cell_region (sheet, start_col, start_row, end_col, end_row);
-	sheet_cell_foreach_range (
-		sheet, TRUE,
-		start_col, start_row,
-		end_col, end_row,
-		clear_cell_format, NULL);
-}
+/*****************************************************************************/
 
 void
 sheet_make_cell_visible (Sheet *sheet, int col, int row)
@@ -2772,8 +2786,8 @@ sheet_cursor_set (Sheet *sheet, int base_col, int base_row, int start_col, int s
  * an array-formula.
  */
 void
-sheet_fill_selection_with (Sheet *sheet, const char *str,
-			   gboolean const is_array)
+sheet_fill_selection_with (CommandContext *context, Sheet *sheet,
+			   const char *str, gboolean const is_array)
 {
 	GList *l;
 
@@ -2788,11 +2802,7 @@ sheet_fill_selection_with (Sheet *sheet, const char *str,
 		if (!sheet_check_for_partial_array (sheet,
 						    ss->user.start.row,ss->user.start.col,
 						    ss->user.end.row, ss->user.end.col)) {
-#if 0
 			gnumeric_error_splits_array (context);
-#else
-			gnumeric_no_modify_array_notice (sheet->workbook);
-#endif
 			return;
 		}
 	}
@@ -3255,7 +3265,7 @@ colrow_move (Sheet *sheet,
 		segment[COLROW_SUB_INDEX(old_pos)] : NULL;
 
 	GList *cells = NULL;
-	Cell *cell;
+	Cell  *cell;
 
 	g_return_if_fail (old_pos >= 0);
 	g_return_if_fail (new_pos >= 0);
@@ -3294,26 +3304,29 @@ colrow_move (Sheet *sheet,
 }
 
 /**
- * sheet_insert_col:
+ * sheet_insert_cols:
  * @sheet   The sheet
  * @col     At which position we want to insert
  * @count   The number of columns to be inserted
  */
-void
+gboolean
 sheet_insert_cols (CommandContext *context, Sheet *sheet,
-		   int col, int count)
+		   int col, int count, GSList **reloc_storage)
 {
-	struct expr_relocate_info reloc_info;
-	GList *deps;
+	ExprRelocateInfo reloc_info;
 	int   i;
 
-	g_return_if_fail (sheet != NULL);
-	g_return_if_fail (IS_SHEET (sheet));
-	g_return_if_fail (count != 0);
+	g_return_val_if_fail (reloc_storage != NULL, TRUE);
+
+	*reloc_storage = NULL;
+
+	g_return_val_if_fail (sheet != NULL, TRUE);
+	g_return_val_if_fail (IS_SHEET (sheet), TRUE);
+	g_return_val_if_fail (count != 0, TRUE);
 
 	/* Is there any work to do? */
 	if (sheet->cols.max_used < 0)
-		return;
+		return FALSE;
 
 	/* 0. Walk cells in displaced col and ensure arrays aren't divided. */
 	if (col > 0)	/* No need to test leftmost column */
@@ -3322,12 +3335,17 @@ sheet_insert_cols (CommandContext *context, Sheet *sheet,
 					      &avoid_dividing_array_horizontal,
 					      NULL) != NULL){
 			gnumeric_error_splits_array (context);
-			return;
+			return TRUE;
 		}
 
-	/* TODO TODO : Walk the right edge to make sure nothing is split
-	 * due to over run.
-	 */
+	/* Walk the right edge to make sure nothing is split due to over run.  */
+	if (sheet_cell_foreach_range (sheet, TRUE, SHEET_MAX_COLS-count, 0,
+				      SHEET_MAX_COLS-1, SHEET_MAX_ROWS-1,
+				      &avoid_dividing_array_horizontal,
+				      NULL) != NULL){
+		gnumeric_error_splits_array (context);
+		return TRUE;
+	}
 
 	/* 1. Delete all columns (and their cells) that will fall off the end */
 	for (i = sheet->cols.max_used; i >= SHEET_MAX_COLS - count ; --i)
@@ -3341,7 +3359,7 @@ sheet_insert_cols (CommandContext *context, Sheet *sheet,
 	reloc_info.origin_sheet = reloc_info.target_sheet = sheet;
 	reloc_info.col_offset = count;
 	reloc_info.row_offset = 0;
-	workbook_expr_relocate (sheet->workbook, &reloc_info);
+	*reloc_storage = workbook_expr_relocate (sheet->workbook, &reloc_info);
 
 	/* 3. Move the columns to their new location (From right to left) */
 	for (i = sheet->cols.max_used; i >= col ; --i)
@@ -3352,14 +3370,9 @@ sheet_insert_cols (CommandContext *context, Sheet *sheet,
 	sheet_style_insert_colrow (sheet, col, count, TRUE);
 
 	/* 5. Recompute dependencies */
-	deps = region_get_dependencies (sheet,
-					col, 0,
-					SHEET_MAX_COLS-1, SHEET_MAX_ROWS-1);
-	cell_queue_recalc_list (deps, TRUE);
-	workbook_recalc (sheet->workbook);
+	sheet_recalc_dependencies (sheet);
 
-	/* 6. Redraw */
-	sheet_redraw_all (sheet);
+	return FALSE;
 }
 
 /*
@@ -3368,28 +3381,31 @@ sheet_insert_cols (CommandContext *context, Sheet *sheet,
  * @col     At which position we want to start deleting columns
  * @count   The number of columns to be deleted
  */
-void
+gboolean
 sheet_delete_cols (CommandContext *context, Sheet *sheet,
-		   int col, int count)
+		   int col, int count, GSList **reloc_storage)
 {
-	struct expr_relocate_info reloc_info;
-	GList *deps;
+	ExprRelocateInfo reloc_info;
 	int i;
 
-	g_return_if_fail (sheet != NULL);
-	g_return_if_fail (IS_SHEET (sheet));
-	g_return_if_fail (count != 0);
+	g_return_val_if_fail (reloc_storage != NULL, TRUE);
+	
+	*reloc_storage = NULL;
+
+	g_return_val_if_fail (sheet != NULL, TRUE);
+	g_return_val_if_fail (IS_SHEET (sheet), TRUE);
+	g_return_val_if_fail (count != 0, TRUE);
 
 	/* Is there any work to do? */
 	if (sheet->cols.max_used < 0)
-		return;
+		return FALSE;
 
 	/* 0. Walk cells in deleted cols and ensure arrays aren't divided. */
 	if (!sheet_check_for_partial_array (sheet, 0, col, 
 					    SHEET_MAX_ROWS-1, col+count-1))
 	{
 		gnumeric_error_splits_array (context);
-		return;
+		return TRUE;
 	}
 
 	/* 1. Delete all columns (and their cells) that will fall off the end */
@@ -3404,14 +3420,16 @@ sheet_delete_cols (CommandContext *context, Sheet *sheet,
 	reloc_info.origin_sheet = reloc_info.target_sheet = sheet;
 	reloc_info.col_offset = SHEET_MAX_COLS; /* send them to infinity */
 	reloc_info.row_offset = SHEET_MAX_ROWS; /*   to force invalidation */
-	workbook_expr_relocate (sheet->workbook, &reloc_info);
+	*reloc_storage = workbook_expr_relocate (sheet->workbook, &reloc_info);
 
 	/* 3. Fix references to and from the cells which are moving */
 	reloc_info.origin.start.col = col+count;
 	reloc_info.origin.end.col = SHEET_MAX_COLS-1;
 	reloc_info.col_offset = -count;
 	reloc_info.row_offset = 0;
-	workbook_expr_relocate (sheet->workbook, &reloc_info);
+	*reloc_storage = g_slist_concat (*reloc_storage,
+					 workbook_expr_relocate (sheet->workbook,
+								 &reloc_info));
 
 	/* 4. Move the columns to their new location (from left to right) */
 	for (i = col + count ; i <= sheet->cols.max_used; ++i)
@@ -3422,14 +3440,9 @@ sheet_delete_cols (CommandContext *context, Sheet *sheet,
 	sheet_style_delete_colrow (sheet, col, count, TRUE);
 
 	/* 6. Recompute dependencies */
-	deps = region_get_dependencies (sheet,
-					col, 0,
-					SHEET_MAX_COLS-1, SHEET_MAX_ROWS-1);
-	cell_queue_recalc_list (deps, TRUE);
-	workbook_recalc (sheet->workbook);
+	sheet_recalc_dependencies (sheet);
 
-	/* 7. Redraw */
-	sheet_redraw_all (sheet);
+	return FALSE;
 }
 
 /**
@@ -3438,21 +3451,24 @@ sheet_delete_cols (CommandContext *context, Sheet *sheet,
  * @row     At which position we want to insert
  * @count   The number of rows to be inserted
  */
-void
+gboolean
 sheet_insert_rows (CommandContext *context, Sheet *sheet,
-		   int row, int count)
+		   int row, int count, GSList **reloc_storage)
 {
-	struct expr_relocate_info reloc_info;
-	GList *deps;
+	ExprRelocateInfo reloc_info;
 	int   i;
 
-	g_return_if_fail (sheet != NULL);
-	g_return_if_fail (IS_SHEET (sheet));
-	g_return_if_fail (count != 0);
+	g_return_val_if_fail (reloc_storage != NULL, TRUE);
+	
+	*reloc_storage = NULL;
+
+	g_return_val_if_fail (sheet != NULL, TRUE);
+	g_return_val_if_fail (IS_SHEET (sheet), TRUE);
+	g_return_val_if_fail (count != 0, TRUE);
 
 	/* Is there any work to do? */
 	if (sheet->rows.max_used < 0)
-		return;
+		return FALSE;
 
 	/* 0. Walk cells in displaced row and ensure arrays aren't divided. */
 	if (row > 0)	/* No need to test leftmost column */
@@ -3462,12 +3478,17 @@ sheet_insert_rows (CommandContext *context, Sheet *sheet,
 					      &avoid_dividing_array_vertical,
 					      NULL) != NULL) {
 			gnumeric_error_splits_array (context);
-			return;
+			return TRUE;
 		}
 
-	/* TODO TODO : Walk the right edge to make sure nothing is split
-	 * due to over run.
-	 */
+	/* Walk the lower edge to make sure nothing is split due to over run.  */
+	if (sheet_cell_foreach_range (sheet, TRUE, 0, SHEET_MAX_ROWS-count,
+				      SHEET_MAX_COLS-1, SHEET_MAX_ROWS-1,
+				      &avoid_dividing_array_vertical,
+				      NULL) != NULL){
+		gnumeric_error_splits_array (context);
+		return TRUE;
+	}
 
 	/* 1. Delete all rows (and their cells) that will fall off the end */
 	for (i = sheet->rows.max_used; i >= SHEET_MAX_ROWS - count ; --i)
@@ -3481,7 +3502,7 @@ sheet_insert_rows (CommandContext *context, Sheet *sheet,
 	reloc_info.origin_sheet = reloc_info.target_sheet = sheet;
 	reloc_info.col_offset = 0;
 	reloc_info.row_offset = count;
-	workbook_expr_relocate (sheet->workbook, &reloc_info);
+	*reloc_storage = workbook_expr_relocate (sheet->workbook, &reloc_info);
 
 	/* 3. Move the rows to their new location (from bottom to top) */
 	for (i = sheet->rows.max_used; i >= row ; --i)
@@ -3492,14 +3513,9 @@ sheet_insert_rows (CommandContext *context, Sheet *sheet,
 	sheet_style_insert_colrow (sheet, row, count, FALSE);
 
 	/* 5. Recompute dependencies */
-	deps = region_get_dependencies (sheet,
-					0, row,
-					SHEET_MAX_COLS-1, SHEET_MAX_ROWS-1);
-	cell_queue_recalc_list (deps, TRUE);
-	workbook_recalc (sheet->workbook);
+	sheet_recalc_dependencies (sheet);
 
-	/* 6. Redraw */
-	sheet_redraw_all (sheet);
+	return FALSE;
 }
 
 /*
@@ -3508,28 +3524,31 @@ sheet_insert_rows (CommandContext *context, Sheet *sheet,
  * @row     At which position we want to start deleting rows
  * @count   The number of rows to be deleted
  */
-void
+gboolean
 sheet_delete_rows (CommandContext *context, Sheet *sheet,
-		   int row, int count)
+		   int row, int count, GSList **reloc_storage)
 {
-	struct expr_relocate_info reloc_info;
-	GList *deps;
+	ExprRelocateInfo reloc_info;
 	int i;
 
-	g_return_if_fail (sheet != NULL);
-	g_return_if_fail (IS_SHEET (sheet));
-	g_return_if_fail (count != 0);
+	g_return_val_if_fail (reloc_storage != NULL, TRUE);
+	
+	*reloc_storage = NULL;
+
+	g_return_val_if_fail (sheet != NULL, TRUE);
+	g_return_val_if_fail (IS_SHEET (sheet), TRUE);
+	g_return_val_if_fail (count != 0, TRUE);
 
 	/* Is there any work to do? */
 	if (sheet->rows.max_used < 0)
-		return;
+		return FALSE;
 
 	/* 0. Walk cells in deleted rows and ensure arrays aren't divided. */
 	if (!sheet_check_for_partial_array (sheet, row, 0, 
 					    row+count-1, SHEET_MAX_COLS-1))
 	{
 		gnumeric_error_splits_array (context);
-		return;
+		return TRUE;
 	}
 
 	/* 1. Delete all cols (and their cells) that will fall off the end */
@@ -3544,14 +3563,16 @@ sheet_delete_rows (CommandContext *context, Sheet *sheet,
 	reloc_info.origin_sheet = reloc_info.target_sheet = sheet;
 	reloc_info.col_offset = SHEET_MAX_COLS; /* send them to infinity */
 	reloc_info.row_offset = SHEET_MAX_ROWS; /*   to force invalidation */
-	workbook_expr_relocate (sheet->workbook, &reloc_info);
+	*reloc_storage = workbook_expr_relocate (sheet->workbook, &reloc_info);
 
 	/* 3. Fix references to and from the cells which are moving */
 	reloc_info.origin.start.row = row + count;
 	reloc_info.origin.end.row = SHEET_MAX_ROWS-1;
 	reloc_info.col_offset = 0;
 	reloc_info.row_offset = -count;
-	workbook_expr_relocate (sheet->workbook, &reloc_info);
+	*reloc_storage = g_slist_concat (*reloc_storage,
+					 workbook_expr_relocate (sheet->workbook,
+								 &reloc_info));
 
 	/* 4. Move the rows to their new location (from top to bottom) */
 	for (i = row + count ; i <= sheet->rows.max_used; ++i)
@@ -3562,14 +3583,9 @@ sheet_delete_rows (CommandContext *context, Sheet *sheet,
 	sheet_style_delete_colrow (sheet, row, count, FALSE);
 
 	/* 6. Recompute dependencies */
-	deps = region_get_dependencies (sheet,
-					0, row,
-					SHEET_MAX_COLS-1, SHEET_MAX_ROWS-1);
-	cell_queue_recalc_list (deps, TRUE);
-	workbook_recalc (sheet->workbook);
+	sheet_recalc_dependencies (sheet);
 
-	/* 7. Redraw */
-	sheet_redraw_all (sheet);
+	return FALSE;
 }
 
 /*
@@ -3578,12 +3594,13 @@ sheet_delete_rows (CommandContext *context, Sheet *sheet,
  */
 void
 sheet_move_range (CommandContext *context,
-		  struct expr_relocate_info const * rinfo)
+		  ExprRelocateInfo const * rinfo)
 {
-	GList *deps, *cells = NULL;
+	GList *cells = NULL;
 	Cell  *cell;
 	gboolean inter_sheet_formula;
 
+	g_return_if_fail (rinfo != NULL);
 	g_return_if_fail (rinfo->origin_sheet != NULL);
 	g_return_if_fail (IS_SHEET (rinfo->origin_sheet));
 	g_return_if_fail (rinfo->target_sheet != NULL);
@@ -3593,7 +3610,12 @@ sheet_move_range (CommandContext *context,
 
 	/* 1. Fix references to and from the cells which are moving */
 	/* All we really want is the workbook so either sheet will do */
-	workbook_expr_relocate (rinfo->origin_sheet->workbook, rinfo);
+
+	/* FIXME : Avoid leaking and free the reloc list for now.  When undo
+	 * for paste_cut is ready we will need this list
+	 */
+	workbook_expr_unrelocate_free (
+		workbook_expr_relocate (rinfo->origin_sheet->workbook, rinfo));
 
 	/* 2. Collect the cells */
 	sheet_cell_foreach_range (rinfo->origin_sheet, TRUE,
@@ -3615,7 +3637,7 @@ sheet_move_range (CommandContext *context,
 			    rinfo->origin.start.row + rinfo->row_offset,
 			    rinfo->origin.end.col + rinfo->col_offset,
 			    rinfo->origin.end.row + rinfo->row_offset,
-			    FALSE); /* Do not to clear styles */
+			    CLEAR_VALUES|CLEAR_COMMENTS); /* Do not to clear styles */
 
 	/* Insert the cells back */
 	for (; cells != NULL ; cells = g_list_remove (cells, cell)) {
@@ -3653,14 +3675,10 @@ sheet_move_range (CommandContext *context,
 	sheet_style_relocate (rinfo);
 
 	/* 6. Recompute dependencies */
-	/* TODO : What region needs this ? the source or the target ? */
-	deps = region_get_dependencies (rinfo->target_sheet,
-					0, 0,
-					SHEET_MAX_COLS-1, SHEET_MAX_ROWS-1);
-	cell_queue_recalc_list (deps, TRUE);
-	workbook_recalc (rinfo->target_sheet->workbook);
+	sheet_recalc_dependencies (rinfo->target_sheet);
 
-	/* 7. Redraw */
+	/* 7. Recalc & Redraw */
+	workbook_recalc (rinfo->target_sheet->workbook);
 	sheet_redraw_all (rinfo->target_sheet);
 }
 
@@ -3682,7 +3700,7 @@ void
 sheet_shift_rows (CommandContext *context, Sheet *sheet,
 		  int col, int start_row, int end_row, int count)
 {
-	struct expr_relocate_info rinfo;
+	ExprRelocateInfo rinfo;
 	rinfo.origin.start.col = col;
 	rinfo.origin.start.row = start_row;
 	rinfo.origin.end.col = SHEET_MAX_COLS-1;
@@ -3711,7 +3729,7 @@ void
 sheet_shift_cols (CommandContext *context, Sheet *sheet,
 		  int start_col, int end_col, int row, int count)
 {
-	struct expr_relocate_info rinfo;
+	ExprRelocateInfo rinfo;
 	rinfo.origin.start.col = start_col;
 	rinfo.origin.start.row = row;
 	rinfo.origin.end.col = end_col;
@@ -3722,3 +3740,78 @@ sheet_shift_cols (CommandContext *context, Sheet *sheet,
 
 	sheet_move_range (context, &rinfo);
 }
+
+double *
+sheet_save_row_col_sizes (Sheet *sheet, gboolean const is_cols,
+			  int index, int count)
+{
+	int i;
+	double *res = NULL;
+
+	g_return_val_if_fail (sheet != NULL, NULL);
+	g_return_val_if_fail (count > 0, NULL);
+
+	res = g_new (double, count);
+
+	for (i = 0 ; i < count ; ++i) {
+		ColRowInfo *info = is_cols
+		    ? sheet_col_get_info (sheet, index + i)
+		    : sheet_row_get_info (sheet, index + i);
+		g_return_val_if_fail (info != NULL, NULL); /* be anal, and leak */
+		res[i] = info->units;
+		if (info->hard_size)
+			res[i] *= -1.;
+	}
+	return res;
+}
+
+void
+sheet_restore_row_col_sizes (Sheet *sheet, gboolean const is_cols,
+			     int index, int count, double *sizes)
+{
+	int i;
+
+	g_return_if_fail (sizes != NULL);
+	g_return_if_fail (sheet != NULL);
+	g_return_if_fail (count > 0);
+
+	for (i = 0 ; i < count ; ++i) {
+		gboolean hard_size = FALSE;
+		if (sizes[i] < 0.) {
+			hard_size = TRUE;
+			sizes[i] *= -1.;
+		}
+		if (is_cols)
+			sheet_col_set_width_units  (sheet, index+i, sizes[i], hard_size);
+		else
+			sheet_row_set_height_units (sheet, index+i, sizes[i], hard_size);
+	}
+
+	g_free (sizes);
+}
+
+/**
+ * sheet_row_col_visible:
+ * @sheet	: the sheet
+ * @is_col	: Are we dealing with rows or columns.
+ * @visible	: Make things visible or invisible.
+ * @index	: The index of the first row/col.
+ * @count	: The number of rows/cols to change the state.
+ *
+ * Change the visibility of the selected range of contiguous rows/cols.
+ */
+void
+sheet_row_col_visible (Sheet *sheet, gboolean const is_col, gboolean const visible,
+		       int index, int count)
+{
+	g_return_if_fail (sheet != NULL);
+
+	while (--count >= 0) {
+		ColRowInfo * const cri = is_col
+		    ? sheet_col_fetch (sheet, index++)
+		    : sheet_row_fetch (sheet, index++);
+
+		if (visible != (cri->pixels >= 0))
+			cri->pixels *= -1;
+	}
+}
