@@ -83,17 +83,17 @@ typedef enum {
 
 	SHEET_MODE_CREATE_BUTTON,
 	SHEET_MODE_CREATE_CHECKBOX,
-	
+
 	/* Object is selected */
 	SHEET_MODE_OBJECT_SELECTED,
 } SheetModeType;
 
 struct _Sheet {
 	int         signature;
-	
+
 	Workbook    *workbook;
 	GList       *sheet_views;
-	
+
 	char        *name;
 
 	SheetStyleData *style_data; /* See sheet-style.c */
@@ -110,13 +110,13 @@ struct _Sheet {
 	SheetSelection *cursor_selection;
 	/* TODO : seperate cursor handling from selection */
 /*	CellPos  selection_corner;*/	/* A corner of the current selection */
-	
+
 	/* The list of cells that have a comment */
 	GList       *comment_list;
-	
+
 	/* User defined names */
 	GList      *names;
-	
+
 	double      last_zoom_factor_used;
 
 	/*
@@ -130,21 +130,28 @@ struct _Sheet {
 	/* Objects */
 	SheetModeType mode;	/* Sheet mode */
 	void        *mode_data; /* Sheet per-mode data */
-	
+
 	GList       *objects;	/* List of objects in the spreadsheet */
 	GList       *coords;	/* During creation time: keeps click coordinates */
 	void        *current_object;
 	void        *active_object_frame;
-	
+
 	gboolean    pristine;
 	gboolean    modified;
-	
+
+	/* Sheet level preferences */
+	gboolean	display_formulas;
+	gboolean	display_zero;
+	gboolean	show_grid;
+	gboolean	show_col_header;
+	gboolean	show_row_header;
+
         /* Solver parameters */
         SolverParameters solver_parameters;
 
-	GHashTable *dependency_hash;
+	DependencyData  *deps;
 
-	void         *corba_server;
+	void            *corba_server;
 
 	PrintInformation *print_info;
 };
@@ -207,6 +214,12 @@ int	    sheet_find_boundary_horizontal (Sheet *sheet, int start_col, int row,
 int	    sheet_find_boundary_vertical   (Sheet *sheet, int col, int start_row,
 					    int count, gboolean jump_to_boundaries);
 
+/* Save and restore the sizes of a set of rows or columns */
+double *    sheet_save_row_col_sizes	   (Sheet *sheet, gboolean const is_cols,
+					    int index, int count);
+void 	    sheet_restore_row_col_sizes	   (Sheet *sheet, gboolean const is_cols,
+					    int index, int count, double *);
+
 /* Duplicates the information of a col/row */
 ColRowInfo *sheet_duplicate_colrow        (ColRowInfo *original);
 
@@ -237,13 +250,14 @@ int         sheet_col_get_distance        (Sheet const *sheet, int from_col, int
 int         sheet_row_get_distance        (Sheet const *sheet, int from_row, int to_row);
 double      sheet_row_get_unit_distance   (Sheet const *sheet, int from_row, int to_row);
 double      sheet_col_get_unit_distance   (Sheet const *sheet, int from_col, int to_col);
- 
+
 /* Sets the width/height of a column row in terms of pixels */
 void        sheet_col_set_width           (Sheet *sheet,
 				           int col, int width);
 void        sheet_col_info_set_width      (Sheet *sheet,
 				           ColRowInfo *ci, int width);
-void        sheet_col_set_width_units     (Sheet *sheet, int col, double width);
+void        sheet_col_set_width_units     (Sheet *sheet, int col, double width,
+					   gboolean set_by_user);
 void        sheet_col_set_internal_width  (Sheet *sheet, ColRowInfo *ci,
 					   double width);
 
@@ -254,7 +268,7 @@ void        sheet_row_set_height_units    (Sheet *sheet, int row, double height,
 					   gboolean height_set_by_user);
 void        sheet_row_info_set_height     (Sheet *sheet,
 				           ColRowInfo *ri, int height,
-				           gboolean height_set_by_user);
+				           gboolean set_by_user);
 void        sheet_row_set_internal_height (Sheet *sheet, ColRowInfo *ri, double height);
 
 int         sheet_col_size_fit            (Sheet *sheet, int col);
@@ -265,9 +279,12 @@ void        sheet_col_set_selection       (Sheet *sheet,
 void        sheet_row_set_selection       (Sheet *sheet,
 					   ColRowInfo *ri, int value);
 void        sheet_set_selection           (Sheet *sheet, SheetSelection const *ss);
-				       
+
+void	    sheet_row_col_visible         (Sheet *sheet, gboolean const is_col,
+					   gboolean const visible,
+					   int index, int count);
+
 /* sheet-style.c */
-struct expr_relocate_info;
 MStyle        *sheet_style_compute              (Sheet const *sheet,
 						 int col, int row);
 void           sheet_style_attach               (Sheet  *sheet, Range   range,
@@ -279,7 +296,7 @@ void           sheet_style_insert_colrow        (Sheet *sheet, int pos, int coun
 						 gboolean is_col);
 void           sheet_style_delete_colrow        (Sheet *sheet, int pos, int count,
 						 gboolean is_col);
-void           sheet_style_relocate             (const struct expr_relocate_info *rinfo);
+void           sheet_style_relocate             (const ExprRelocateInfo *rinfo);
 void           sheet_selection_apply_style      (Sheet *sheet, MStyle *style);
 MStyle        *sheet_selection_get_unique_style (Sheet *sheet,
 						 MStyleBorder **borders);
@@ -307,18 +324,10 @@ void        sheet_redraw_cell_region      (Sheet const *sheet,
 void	    sheet_redraw_cols		  (Sheet const *sheet);
 void	    sheet_redraw_rows		  (Sheet const *sheet);
 void        sheet_redraw_selection        (Sheet const *sheet, SheetSelection const *ss);
+void        sheet_redraw_range            (Sheet const *sheet, Range const *sheet_selection);
 void        sheet_redraw_all              (Sheet const *sheet);
-				       
-void        sheet_update_auto_expr        (Sheet *sheet);
 
-/*
- * A Pair of utilities to temporarily suspend auto_expr's while reading
- * worksheets.
- */
-void        sheet_suspend_auto_expr       (Workbook *wb,
-					   ExprTree **expr, String **desc);
-void        sheet_resume_auto_expr        (Workbook *wb,
-					   ExprTree *expr, String *desc);
+void        sheet_update_auto_expr        (Sheet *sheet);
 
 void        sheet_mark_clean              (Sheet *sheet);
 void        sheet_set_dirty               (Sheet *sheet, gboolean is_dirty);
@@ -326,7 +335,7 @@ gboolean    sheet_is_pristine             (Sheet *sheet);
 
 /* Sheet information manipulation */
 void        sheet_move_range              (CommandContext *context,
-					   struct expr_relocate_info const * rinfo);
+					   ExprRelocateInfo const * rinfo);
 
 Sheet      *sheet_lookup_by_name          (Workbook *wb, const char *name);
 
@@ -348,13 +357,6 @@ int         sheet_row_selection_type      (Sheet const *sheet, int row);
  */
 void        sheet_set_mode_type           (Sheet *sheet, SheetModeType type);
 
-
-/*
- * Callback routines.
- */
-void        sheet_fill_selection_with     (Sheet *sheet, const char *text,
-					   gboolean const is_array);
-
 /*
  * Hiding/showing the cursor
  */
@@ -376,7 +378,7 @@ gboolean     cellref_r1c1_get             (CellRef *out, const char *in,
 void sheet_insert_object (Sheet *sheet, char *repoid);
 
 /*
- * Hooks for CORBA bootstrap: they create the 
+ * Hooks for CORBA bootstrap: they create the
  */
 void sheet_corba_setup       (Sheet *);
 void sheet_corba_shutdown    (Sheet *);
@@ -386,35 +388,37 @@ void sheet_corba_shutdown    (Sheet *);
  * These have undo/redo capabilities
  * and will route error messages to the caller appropriately.
  */
-void  sheet_insert_cols (CommandContext *context, Sheet *sheet,
-			 int col, int count);
-void  sheet_delete_cols (CommandContext *context, Sheet *sheet,
-			 int col, int count);
+gboolean  sheet_insert_cols (CommandContext *context, Sheet *sheet,
+			     int col, int count, GSList **reloc_storage);
+gboolean  sheet_delete_cols (CommandContext *context, Sheet *sheet,
+			     int col, int count, GSList **reloc_storage);
 void  sheet_shift_cols  (CommandContext *context, Sheet *sheet,
 			 int start_col, int end_col,
 			 int row,       int count);
-void  sheet_insert_rows (CommandContext *context, Sheet *sheet,
-			 int row, int count);
-void  sheet_delete_rows (CommandContext *context, Sheet *sheet,
-			 int row, int count);
+gboolean  sheet_insert_rows (CommandContext *context, Sheet *sheet,
+			 int row, int count, GSList **reloc_storage);
+gboolean  sheet_delete_rows (CommandContext *context, Sheet *sheet,
+			 int row, int count, GSList **reloc_storage);
 void  sheet_shift_rows  (CommandContext *context, Sheet *sheet,
 			 int col,
 			 int start_row, int end_row, int count);
 
-void  sheet_clear_region          (CommandContext *context, Sheet *sheet,
-				   int start_col, int start_row,
-				   int end_col,   int end_row,
-				   gboolean const keep_styles);
-void  sheet_clear_region_formats  (Sheet *sheet,
-				   int start_col, int start_row,
-				   int end_col,   int end_row,
-				   void *closure);
-void  sheet_clear_region_content  (CommandContext *context, Sheet *sheet,
-				   int start_col, int start_row,
-				   int end_col,   int end_row);
-void  sheet_clear_region_comments (Sheet *sheet,
-				   int start_col, int start_row,
-				   int end_col,   int end_row,
-				   void *closure);	
+void  sheet_fill_selection_with (CommandContext *context, Sheet *sheet,
+				 const char *text, gboolean const is_array);
+
+void sheet_adjust_preferences (Sheet const *sheet);
+
+typedef enum
+{
+	CLEAR_VALUES   = 0x1,
+	CLEAR_FORMATS  = 0x2,
+	CLEAR_COMMENTS = 0x4,
+} SheetClearFlags;
+
+void  sheet_clear_region (CommandContext *context,
+			  Sheet *sheet,
+			  int const start_col, int const start_row,
+			  int const end_col, int const end_row,
+			  int const clear_flags);
 
 #endif /* GNUMERIC_SHEET_H */
