@@ -24,8 +24,4 @@ int         col_from_name          (const char *cell_str);
 guint       gnumeric_strcase_hash  (gconstpointer v);
 gint        gnumeric_strcase_equal (gconstpointer v, gconstpointer v2);
 
-/* return the gnumeric serial number of a special date */
-guint32     g_date_serial          (GDate  *date);
-GDate      *g_date_new_serial      (guint32 serial);
-
 #endif /* GNUMERIC_UTILS_H */
