@@ -29,6 +29,9 @@ struct _ErrorMessage {
 struct _Value {
 	ValueType type;
 	union {
+		/*
+		 * This element is used as a short hand for cell_range.cell_a
+		 */
 		CellRef cell;
 		struct {
 			CellRef cell_a;
@@ -68,6 +71,7 @@ void         value_release         (Value *value);
 void         value_dump            (Value const *value);
 Value       *value_duplicate       (Value const *value);
 void         value_copy_to         (Value *dest, Value const *source);
+gboolean     value_equal           (const Value *a, const Value *b);
 
 gboolean     value_get_as_bool     (Value const *v, gboolean *err);
 char        *value_get_as_string   (const Value *value);
@@ -107,6 +111,8 @@ void value_array_set       (Value *array, guint col, guint row, Value *v);
 void value_array_resize    (Value *v, guint width, guint height);
 void value_array_copy_to   (Value *dest, const Value *src);
 
+Value * value_is_error (char const * const str, int *offset);
+
 /* Some utility constants to make sure we all spell correctly */
 extern char const *gnumeric_err_NULL;
 extern char const *gnumeric_err_DIV0;
