@@ -14,7 +14,16 @@
 void workbook_view_set_paste_special_state (Workbook *wb, gboolean enable);
 
 void workbook_view_set_undo_redo_state (Workbook const * const wb,
-					gboolean const has_undos,
-					gboolean const has_redos);
+					char const * const undo_suffix,
+					char const * const redo_suffix);
+
+void workbook_view_set_size (Workbook const * const wb,
+			     int width_in_points,
+			     int height_in_points);
+
+void workbook_view_set_title (Workbook const * const wb,
+			      char const * const title);
+
+void workbook_view_pref_visibility (Workbook const * const wb);
 
 #endif /* GNUMERIC_WORKBOOK_VIEW_H */
