@@ -32,6 +32,7 @@
 #include "workbook-format-toolbar.h"
 #include "workbook-view.h"
 #include "command-context-gui.h"
+#include "commands.h"
 
 #ifdef ENABLE_BONOBO
 #include <bonobo/gnome-persist-file.h>
@@ -134,6 +135,18 @@ summary_cmd (GtkWidget *widget, Workbook *wb)
 	dialog_summary_update (wb, wb->summary_info);
 }
 
+static void
+autocorrect_cmd (GtkWidget *widget, Workbook *wb)
+{
+	dialog_autocorrect (wb);
+}
+
+static void
+autosave_cmd (GtkWidget *widget, Workbook *wb)
+{
+	dialog_autosave (wb);
+}
+
 static void
 plugins_cmd (GtkWidget *widget, Workbook *wb)
 {
@@ -231,19 +244,6 @@ cb_sheet_destroy_contents (gpointer key, gpointer value, gpointer user_data)
 	sheet_destroy_contents (sheet);
 }
 
-static void
-dump_dep (gpointer key, gpointer value, gpointer closure)
-{
-	DependencyRange const * const deprange = key;
-	Range const * const range = &(deprange->range);
-
-	/* 2 calls to col_name.  It uses a static buffer */
-	printf ("\t%d : %s%d:", deprange->ref_count,
-		col_name(range->start.col), range->start.row+1);
-	printf ("%s%d\n",
-		col_name(range->end.col), range->end.row+1);
-}
-
 static void
 workbook_do_destroy_private (Workbook *wb)
 {
@@ -257,6 +257,8 @@ workbook_do_destroy (Workbook *wb)
 		GTK_OBJECT (wb->toplevel),
 		GTK_SIGNAL_FUNC (workbook_set_focus), wb);
 
+	gtk_timeout_remove (wb->autosave_timer);
+
 	/*
 	 * Do all deletions that leave the workbook in a working
 	 * order.
@@ -265,9 +267,12 @@ workbook_do_destroy (Workbook *wb)
 	summary_info_free (wb->summary_info);
 	wb->summary_info = NULL;
 
+	command_list_release (wb->undo_commands);
+	command_list_release (wb->redo_commands);
+
 	/* Release the clipboard if it is associated with this workbook */
 	{
-		Sheet * tmp_sheet;
+		Sheet *tmp_sheet;
 		if ((tmp_sheet = application_clipboard_sheet_get ()) != NULL &&
 		    tmp_sheet->workbook == wb)
 			application_clipboard_clear ();
@@ -312,18 +317,8 @@ workbook_do_destroy (Workbook *wb)
 			 * the sheet from the workbook.  Its ugly, but should
 			 * be ok for debug code.
 			 */
-			if (gnumeric_debugging > 0 &&
-			    sheet->dependency_hash != NULL &&
-			    g_hash_table_size (sheet->dependency_hash) != 0) {
-				printf ("Hash %s:%s = %d\n",
-					sheet->workbook->filename
-					?  sheet->workbook->filename
-					: "(no name)",
-					sheet->name,
-					g_hash_table_size (sheet->dependency_hash));
-				g_hash_table_foreach (sheet->dependency_hash, dump_dep, NULL);
-			}
-
+			if (gnumeric_debugging > 0)
+				sheet_dump_dependencies (sheet);
 			/*
 			 * Make sure we alwayws keep the focus on an
 			 * existing widget (otherwise the destruction
@@ -378,7 +373,7 @@ workbook_destroy (GtkObject *wb_object)
 static int
 workbook_delete_event (GtkWidget *widget, GdkEvent *event, Workbook *wb)
 {
-	if (workbook_can_close (wb)){
+	if (workbook_can_close (wb)) {
 #ifdef ENABLE_BONOBO
 		if (wb->bonobo_regions) {
 			gtk_widget_hide (GTK_WIDGET (wb->toplevel));
@@ -525,9 +520,9 @@ workbook_can_close (Workbook *wb)
 		return FALSE;
 	
 	in_can_close = TRUE;
-	while (workbook_is_dirty (wb) &&
-	       !done) {
-		GtkWidget *d, *l;
+	while (workbook_is_dirty (wb) && !done) {
+
+		GtkWidget *d, *l, *cancel_button;
 		int button;
 		char *s;
 
@@ -539,6 +534,8 @@ workbook_can_close (Workbook *wb)
 			GNOME_STOCK_BUTTON_NO,
 			GNOME_STOCK_BUTTON_CANCEL,
 			NULL);
+		cancel_button = g_list_last (GNOME_DIALOG (d)->buttons)->data;
+		gtk_widget_grab_focus (cancel_button);
 		gnome_dialog_set_parent (GNOME_DIALOG (d), GTK_WINDOW (wb->toplevel));
 		
 		if (wb->filename)
@@ -627,19 +624,13 @@ quit_cmd (void)
 static void
 undo_cmd (GtkWidget *widget, Workbook *wb)
 {
-#if 0
-	/* Disable until they are read for prime time */
-	workbook_undo (wb);
-#endif
+	command_undo (workbook_command_context_gui (wb), wb);
 }
 
 static void
 redo_cmd (GtkWidget *widget, Workbook *wb)
 {
-#if 0
-	/* Disable until they are read for prime time */
-	workbook_redo (wb);
-#endif
+	command_redo (workbook_command_context_gui (wb), wb);
 }
 
 static void
@@ -701,10 +692,17 @@ paste_special_cmd (GtkWidget *widget, Workbook *wb)
 static void
 delete_cells_cmd (GtkWidget *widget, Workbook *wb)
 {
-	Sheet *sheet;
+	dialog_delete_cells (wb, workbook_get_current_sheet (wb));
+}
 
-	sheet = workbook_get_current_sheet (wb);
-	dialog_delete_cells (wb, sheet);
+static void
+sheet_action_delete_sheet (GtkWidget *ignored, Sheet *current_sheet);
+
+static void
+delete_sheet_cmd (GtkWidget *widget, Workbook *wb)
+{
+	sheet_action_delete_sheet (widget,
+				   workbook_get_current_sheet (wb));
 }
 
 static void
@@ -771,8 +769,8 @@ insert_cols_cmd (GtkWidget *unused, Workbook *wb)
 	 * at minimum a warning if things are about to be cleared ?
 	 */
 	cols = ss->user.end.col - ss->user.start.col + 1;
-	sheet_insert_cols (workbook_command_context_gui (wb), sheet,
-			   ss->user.start.col, cols);
+	cmd_insert_cols (workbook_command_context_gui (wb), sheet,
+			 ss->user.start.col, cols);
 }
 
 static void
@@ -796,45 +794,40 @@ insert_rows_cmd (GtkWidget *unused, Workbook *wb)
 	 * at minimum a warning if things are about to be cleared ?
 	 */
 	rows = ss->user.end.row - ss->user.start.row + 1;
-	sheet_insert_rows (workbook_command_context_gui (wb), sheet,
-			   ss->user.start.row, rows);
+	cmd_insert_rows (workbook_command_context_gui (wb), sheet,
+			 ss->user.start.row, rows);
 }
 
 static void
 clear_all_cmd (GtkWidget *widget, Workbook *wb)
 {
-	Sheet *sheet;
-
-	sheet = workbook_get_current_sheet (wb);
-
-	sheet_selection_clear (workbook_command_context_gui (wb), sheet);
+	cmd_clear_selection (workbook_command_context_gui (wb), 
+		   workbook_get_current_sheet (wb),
+		   CLEAR_VALUES | CLEAR_FORMATS | CLEAR_COMMENTS);
 }
 
 static void
 clear_formats_cmd (GtkWidget *widget, Workbook *wb)
 {
-	Sheet *sheet;
-
-	sheet = workbook_get_current_sheet (wb);
-	sheet_selection_clear_formats (workbook_command_context_gui (wb), sheet);
+	cmd_clear_selection (workbook_command_context_gui (wb), 
+		   workbook_get_current_sheet (wb),
+		   CLEAR_FORMATS);
 }
 
 static void
 clear_comments_cmd (GtkWidget *widget, Workbook *wb)
 {
-	Sheet *sheet;
-
-	sheet = workbook_get_current_sheet (wb);
-	sheet_selection_clear_comments (workbook_command_context_gui (wb), sheet);
+	cmd_clear_selection (workbook_command_context_gui (wb), 
+		   workbook_get_current_sheet (wb),
+		   CLEAR_COMMENTS);
 }
 
 static void
 clear_content_cmd (GtkWidget *widget, Workbook *wb)
 {
-	Sheet *sheet;
-
-	sheet = workbook_get_current_sheet (wb);
-	sheet_selection_clear_content (workbook_command_context_gui (wb), sheet);
+	cmd_clear_selection (workbook_command_context_gui (wb), 
+		   workbook_get_current_sheet (wb),
+		   CLEAR_VALUES);
 }
 
 static void
@@ -846,21 +839,89 @@ zoom_cmd (GtkWidget *widget, Workbook *wb)
 	dialog_zoom (wb, sheet);
 }
 
+static void
+cb_cell_rerender (gpointer cell, gpointer data)
+{
+        cell_render_value (cell);
+        cell_queue_redraw (cell);
+}
+
+/***********************************************************************/
+/* Sheet preferences */
+
+static void
+cb_sheet_pref_display_formulas (GtkWidget *widget, Workbook *wb)
+{
+	Sheet *sheet = workbook_get_current_sheet (wb);
+	sheet->display_formulas = !sheet->display_formulas;
+	g_list_foreach (wb->formula_cell_list, &cb_cell_rerender, NULL);
+}
+static void
+cb_sheet_pref_hide_zeros (GtkWidget *widget, Workbook *wb)
+{
+	Sheet *sheet = workbook_get_current_sheet (wb);
+	sheet->display_zero = ! sheet->display_zero;
+	sheet_redraw_all (sheet);
+}
+static void
+cb_sheet_pref_hide_grid_lines (GtkWidget *widget, Workbook *wb)
+{
+	Sheet *sheet = workbook_get_current_sheet (wb);
+	sheet->show_grid = !sheet->show_grid;
+	sheet_redraw_all (sheet);
+}
+static void
+cb_sheet_pref_hide_col_header (GtkWidget *widget, Workbook *wb)
+{
+	Sheet *sheet = workbook_get_current_sheet (wb);
+	sheet->show_col_header = ! sheet->show_col_header;
+	sheet_adjust_preferences (sheet);
+}
+static void
+cb_sheet_pref_hide_row_header (GtkWidget *widget, Workbook *wb)
+{
+	Sheet *sheet = workbook_get_current_sheet (wb);
+	sheet->show_row_header = ! sheet->show_row_header;
+	sheet_adjust_preferences (sheet);
+}
+
+/***********************************************************************/
+/* Workbook level preferences */
+
+static void
+cb_wb_pref_hide_hscroll (GtkWidget *widget, Workbook *wb)
+{
+	wb->show_horizontal_scrollbar = ! wb->show_horizontal_scrollbar;
+	workbook_view_pref_visibility (wb);
+}
+
+static void
+cb_wb_pref_hide_vscroll (GtkWidget *widget, Workbook *wb)
+{
+	wb->show_vertical_scrollbar = ! wb->show_vertical_scrollbar;
+	workbook_view_pref_visibility (wb);
+}
+
+static void
+cb_wb_pref_hide_tabs (GtkWidget *widget, Workbook *wb)
+{
+	wb->show_notebook_tabs = ! wb->show_notebook_tabs;
+	workbook_view_pref_visibility (wb);
+}
+
+/***********************************************************************/
+
 static void
 format_cells_cmd (GtkWidget *widget, Workbook *wb)
 {
-	Sheet *sheet;
-
-	sheet = workbook_get_current_sheet (wb);
+	Sheet *sheet = workbook_get_current_sheet (wb);
 	dialog_cell_format (wb, sheet);
 }
 
 static void
 sort_cells_cmd (GtkWidget *widget, Workbook *wb)
 {
-	Sheet *sheet;
-
-	sheet = workbook_get_current_sheet (wb);
+	Sheet *sheet = workbook_get_current_sheet (wb);
 	dialog_cell_sort (wb, sheet);
 }
 
@@ -870,46 +931,20 @@ recalc_cmd (GtkWidget *widget, Workbook *wb)
 	workbook_recalc_all (wb);
 }
 
-static void
-insert_at_cursor (Sheet *sheet, Value *value, const char *format)
-{
-	Cell *cell;
-
-	cell = sheet_cell_fetch (sheet, sheet->cursor_col, sheet->cursor_row);
-	cell_set_value (cell, value);
-	cell_set_format (cell, format);
-
-	workbook_recalc (sheet->workbook);
-}
-
 static void
 insert_current_date_cmd (GtkWidget *widget, Workbook *wb)
 {
 	Sheet *sheet = workbook_get_current_sheet (wb);
-	const char *preferred_date_format = _(">mm/dd/yyyy");
-	int n;
-	GDate *date = g_date_new();
-
-	g_date_set_time (date, time (NULL));
-
-	n = g_date_serial (date);
-
-	g_date_free( date );
-
-	insert_at_cursor (sheet, value_new_int (n), preferred_date_format+1);
+	cmd_set_date_time (workbook_command_context_gui (wb), TRUE,
+			   sheet, sheet->cursor_col, sheet->cursor_row);
 }
 
 static void
 insert_current_time_cmd (GtkWidget *widget, Workbook *wb)
 {
 	Sheet *sheet = workbook_get_current_sheet (wb);
-	time_t t = time (NULL);
-	struct tm *tm = localtime (&t);
-	const char *preferred_time_format = _(">hh:mm");
-	float_t serial;
-
-	serial = (tm->tm_hour * 3600 + tm->tm_min * 60 + tm->tm_sec)/(24*3600.0);
-	insert_at_cursor (sheet, value_new_float (serial), preferred_time_format);
+	cmd_set_date_time (workbook_command_context_gui (wb), TRUE,
+			   sheet, sheet->cursor_col, sheet->cursor_row);
 }
 
 static void
@@ -1063,12 +1098,17 @@ static GnomeUIInfo workbook_menu_edit [] = {
         GNOMEUIINFO_MENU_CUT_ITEM(cut_cmd, NULL),
 	GNOMEUIINFO_MENU_COPY_ITEM(copy_cmd, NULL),
 	GNOMEUIINFO_MENU_PASTE_ITEM(paste_cmd, NULL),
+
+	GNOMEUIINFO_SEPARATOR,
+
 	{ GNOME_APP_UI_ITEM, PASTE_SPECIAL_NAME, NULL,
 	  paste_special_cmd },
-        { GNOME_APP_UI_ITEM, N_("_Delete..."), NULL,
-	  delete_cells_cmd },
         { GNOME_APP_UI_SUBTREE, N_("C_lear"),
 	  N_("Clear the selected cell(s)"), workbook_menu_edit_clear },
+        { GNOME_APP_UI_ITEM, N_("_Delete..."), NULL,
+	  delete_cells_cmd },
+        { GNOME_APP_UI_ITEM, N_("De_lete Sheet"), NULL,
+	  delete_sheet_cmd },
 
 	GNOMEUIINFO_SEPARATOR,
 
@@ -1081,12 +1121,6 @@ static GnomeUIInfo workbook_menu_edit [] = {
 
 	GNOMEUIINFO_SEPARATOR,
 
-#ifdef ENABLE_BONOBO
-	GNOMEUIINFO_ITEM_NONE(N_("Insert object..."),
-			      N_("Inserts a Bonobo object"),
-			      insert_object_cmd),
-#endif
-
 	{ GNOME_APP_UI_ITEM, N_("_Define cell names"), NULL, define_cell_cmd },
 	GNOMEUIINFO_SEPARATOR,
 
@@ -1121,12 +1155,18 @@ static GnomeUIInfo workbook_menu_insert_special [] = {
 static GnomeUIInfo workbook_menu_insert [] = {
 	{ GNOME_APP_UI_ITEM, N_("_Sheet"), N_("Insert a new spreadsheet"),
 	  insert_sheet_cmd },
-	{ GNOME_APP_UI_ITEM, N_("_Cells..."), N_("Insert new cells"),
-	  insert_cells_cmd },
 	{ GNOME_APP_UI_ITEM, N_("_Rows"), N_("Insert new rows"),
 	  insert_rows_cmd  },
-	{ GNOME_APP_UI_ITEM, N_("C_olumns"), N_("Insert new columns"),
+	{ GNOME_APP_UI_ITEM, N_("_Columns"), N_("Insert new columns"),
 	  insert_cols_cmd  },
+	{ GNOME_APP_UI_ITEM, N_("C_ells..."), N_("Insert new cells"),
+	  insert_cells_cmd },
+
+#ifdef ENABLE_BONOBO
+	GNOMEUIINFO_ITEM_NONE(N_("_Object..."),
+			      N_("Inserts a Bonobo object"),
+			      insert_object_cmd),
+#endif
 
 	GNOMEUIINFO_SEPARATOR,
 
@@ -1144,6 +1184,12 @@ static GnomeUIInfo workbook_menu_format_column [] = {
 	  NULL, workbook_cmd_format_column_auto_fit },
 	{ GNOME_APP_UI_ITEM, N_("_Width"),
 	  NULL, workbook_cmd_format_column_width },
+	{ GNOME_APP_UI_ITEM, N_("_Hide"),
+	  NULL, workbook_cmd_format_column_hide },
+	{ GNOME_APP_UI_ITEM, N_("_Unhide"),
+	  NULL, workbook_cmd_format_column_unhide },
+	{ GNOME_APP_UI_ITEM, N_("_Standard Width"),
+	  NULL, workbook_cmd_format_column_std_width },
 	GNOMEUIINFO_END
 };
 
@@ -1152,12 +1198,40 @@ static GnomeUIInfo workbook_menu_format_row [] = {
 	  NULL,  workbook_cmd_format_row_auto_fit },
 	{ GNOME_APP_UI_ITEM, N_("_Height"),
 	  NULL, workbook_cmd_format_row_height },
+	{ GNOME_APP_UI_ITEM, N_("_Hide"),
+	  NULL, workbook_cmd_format_row_hide },
+	{ GNOME_APP_UI_ITEM, N_("_Unhide"),
+	  NULL, workbook_cmd_format_row_unhide },
+	{ GNOME_APP_UI_ITEM, N_("_Standard Height"),
+	  NULL, workbook_cmd_format_row_std_height },
 	GNOMEUIINFO_END
 };
 
 static GnomeUIInfo workbook_menu_format_sheet [] = {
 	{ GNOME_APP_UI_ITEM, N_("_Change name"),   NULL,
 	  workbook_cmd_format_sheet_change_name },
+
+	{ GNOME_APP_UI_TOGGLEITEM, N_("Display _Formulas"),
+	    NULL, &cb_sheet_pref_display_formulas },
+	{ GNOME_APP_UI_TOGGLEITEM, N_("Hide _Zeros"),
+	    NULL, &cb_sheet_pref_hide_zeros },
+
+	{ GNOME_APP_UI_TOGGLEITEM, N_("Hide _Gridlines"),
+	    NULL, &cb_sheet_pref_hide_grid_lines },
+	{ GNOME_APP_UI_TOGGLEITEM, N_("Hide _Column Header"),
+	    NULL, &cb_sheet_pref_hide_col_header },
+	{ GNOME_APP_UI_TOGGLEITEM, N_("Hide _Row Header"),
+	    NULL, &cb_sheet_pref_hide_row_header },
+	GNOMEUIINFO_END
+};
+
+static GnomeUIInfo workbook_menu_format_workbook [] = {
+	{ GNOME_APP_UI_TOGGLEITEM, N_("Hide _Horizontal Scrollbar"),
+	    NULL, &cb_wb_pref_hide_hscroll },
+	{ GNOME_APP_UI_TOGGLEITEM, N_("Hide _Vertical Scrollbar"),
+	    NULL, &cb_wb_pref_hide_vscroll },
+	{ GNOME_APP_UI_TOGGLEITEM, N_("Hide Sheet _Tabs"),
+	    NULL, &cb_wb_pref_hide_tabs },
 	GNOMEUIINFO_END
 };
 
@@ -1168,20 +1242,31 @@ static GnomeUIInfo workbook_menu_format [] = {
 	{ GNOME_APP_UI_SUBTREE, N_("C_olumn"), NULL, workbook_menu_format_column },
 	{ GNOME_APP_UI_SUBTREE, N_("_Row"),    NULL, workbook_menu_format_row },
 	{ GNOME_APP_UI_SUBTREE, N_("_Sheet"),  NULL, workbook_menu_format_sheet },
-
+	{ GNOME_APP_UI_SUBTREE, N_("_Workbook"),  NULL, workbook_menu_format_workbook },
 	GNOMEUIINFO_END
 };
 
 /* Tools menu */
 static GnomeUIInfo workbook_menu_tools [] = {
-	{ GNOME_APP_UI_ITEM, N_("_Sort..."),
-	  N_("Sort the selected cells"), sort_cells_cmd },
+	{ GNOME_APP_UI_ITEM, N_("Auto _Correct..."), N_("Auto Correct"),
+	  autocorrect_cmd },
+	{ GNOME_APP_UI_ITEM, N_("_Auto Save..."), N_("Auto Save"),
+	  autosave_cmd },
+	GNOMEUIINFO_SEPARATOR,
 	{ GNOME_APP_UI_ITEM, N_("_Goal Seek..."), NULL, goal_seek_cmd },
 	{ GNOME_APP_UI_ITEM, N_("_Solver..."),    NULL, solver_cmd },
+	GNOMEUIINFO_SEPARATOR,
 	{ GNOME_APP_UI_ITEM, N_("_Data Analysis..."), NULL, data_analysis_cmd },
 	GNOMEUIINFO_END
 };
 
+/* Data menu */
+static GnomeUIInfo workbook_menu_data [] = {
+	{ GNOME_APP_UI_ITEM, N_("_Sort..."),
+	  N_("Sort the selected cells"), sort_cells_cmd },
+	GNOMEUIINFO_END
+};
+
 #ifndef ENABLE_BONOBO
 static GnomeUIInfo workbook_menu_help [] = {
 	GNOMEUIINFO_HELP ("gnumeric"),
@@ -1197,6 +1282,7 @@ static GnomeUIInfo workbook_menu [] = {
 	{ GNOME_APP_UI_SUBTREE, N_("_Insert"), NULL, workbook_menu_insert },
 	{ GNOME_APP_UI_SUBTREE, N_("F_ormat"), NULL, workbook_menu_format },
 	{ GNOME_APP_UI_SUBTREE, N_("_Tools"), NULL, workbook_menu_tools },
+	{ GNOME_APP_UI_SUBTREE, N_("_Data"), NULL, workbook_menu_data },
 #ifdef ENABLE_BONOBO
 #warning Should enable this when Bonobo gets menu help support
 #else
@@ -1360,17 +1446,6 @@ workbook_focus_sheet (Sheet *sheet)
 	}
 }
 
-static void
-wb_input_finished (GtkEntry *entry, Workbook *wb)
-{
-	Sheet *sheet;
-
-	sheet = workbook_get_current_sheet (wb);
-
-	sheet_set_current_value (sheet);
-	workbook_focus_current_sheet (wb);
-}
-
 static int
 wb_edit_key_pressed (GtkEntry *entry, GdkEventKey *event, Workbook *wb)
 {
@@ -1396,7 +1471,8 @@ wb_edit_key_pressed (GtkEntry *entry, GdkEventKey *event, Workbook *wb)
 		event->keyval = GDK_VoidSymbol;
 
 		/* Only apply do this for formulas */
-		if (entry->text [0] != '=')
+		if (!gnumeric_char_start_expr_p (entry->text[0]) ||
+		    entry->text_length < 1)
 			return TRUE;
 
 		/*
@@ -1538,7 +1614,6 @@ static void
 accept_input (GtkWidget *widget, Workbook *wb)
 {
 	Sheet *sheet = workbook_get_current_sheet (wb);
-
 	sheet_set_current_value (sheet);
 	workbook_focus_current_sheet (wb);
 }
@@ -1575,7 +1650,7 @@ wizard_input (GtkWidget *widget, Workbook *wb)
 	g_free (txt);
 	txt = gtk_entry_get_text (entry);
 
-	if (txt [0] != '=')
+	if (!gnumeric_char_start_expr_p (txt[0]))
 	        edittxt = g_strconcat ("=", txt, NULL);
 	else
 		edittxt = g_strdup (txt);
@@ -1588,7 +1663,6 @@ misc_output (GtkWidget *widget, Workbook *wb)
 {
 	Sheet *sheet = workbook_get_current_sheet (wb);
 	Range const * selection;
-	GList *list;
 
 	if (gnumeric_debugging > 3) {
 		summary_info_dump (wb->summary_info);
@@ -1599,34 +1673,17 @@ misc_output (GtkWidget *widget, Workbook *wb)
 				_("Selection must be a single range"));
 			return;
 		}
-		
-		printf ("The cells that depend on ");
-		range_dump (selection);
-		
-		list = region_get_dependencies (sheet,
-						selection->start.col, selection->start.row,
-						selection->end.col, selection->end.row);
-		
-		if (!list)
-			printf ("No dependencies\n");
-		
-		while (list) {
-			Cell *cell = list->data;
-			
-			list = g_list_next (list);
-			if (!cell)
-				continue;
-			
-			if (sheet != cell->sheet && cell->sheet)
-				printf ("%s", cell->sheet->name);
-			
-			printf ("%s\n", cell_name (cell->col->pos, cell->row->pos));
-		}
 	}
+
 	if (style_debugging > 0) {
 		printf ("Style list\n");
 		sheet_styles_dump (sheet);
 	}
+
+	if (dependency_debugging > 0) {
+		printf ("Dependencies\n");
+		sheet_dump_dependencies (sheet);
+	}
 }
 
 static void
@@ -1675,7 +1732,8 @@ workbook_setup_edit_area (Workbook *wb)
 
 	/* Dependency + Style debugger */
 	if (gnumeric_debugging > 9 ||
-	    style_debugging > 0) {
+	    style_debugging > 0 ||
+	    dependency_debugging > 0) {
 		deps_button = gtk_button_new ();
 		pix = gnome_stock_pixmap_widget_new (wb->toplevel, GNOME_STOCK_PIXMAP_BOOK_RED);
 		gtk_container_add (GTK_CONTAINER (deps_button), pix);
@@ -1694,7 +1752,7 @@ workbook_setup_edit_area (Workbook *wb)
 
 	/* Do signal setup for the editing input line */
 	gtk_signal_connect (GTK_OBJECT (wb->ea_input), "activate",
-			    GTK_SIGNAL_FUNC (wb_input_finished),
+			    GTK_SIGNAL_FUNC (accept_input),
 			    wb);
 	gtk_signal_connect (GTK_OBJECT (wb->ea_input), "key_press_event",
 			    GTK_SIGNAL_FUNC (wb_edit_key_pressed),
@@ -1740,44 +1798,6 @@ workbook_set_auto_expr (Workbook *wb,
 	wb->auto_expr_desc = string_get (description);
 }
 
-/*
- * A utility to temporarily remove a workbook auto expr.  The current
- * expression and description are returned to the caller in @expr and @desc so
- * that they can be reset via sheet_resume_auto_expr later.  The caller is
- * responsible for managing the string and expression until they are returned
- * to the workbook via sheet_suspend_auto_expr.
- */
-void
-sheet_suspend_auto_expr (Workbook *wb, ExprTree **expr, String **desc)
-{
-	g_return_if_fail (wb != NULL);
-
-	*expr = wb->auto_expr;
-	wb->auto_expr = NULL;
-	*desc = wb->auto_expr_desc;
-	wb->auto_expr_desc = NULL;
-}
-
-/*
- * The counter point to sheet_suspend_auto_expr it reset previously suspended
- * auto expressions and retakes responsibility for managing the strings and
- * expressions.
- */
-void
-sheet_resume_auto_expr (Workbook *wb, ExprTree *expr, String *desc)
-{
-	Sheet *sheet;
-
-	g_return_if_fail (wb != NULL);
-	g_return_if_fail (expr != NULL);
-	g_return_if_fail (desc != NULL);
-
-       	sheet = workbook_get_current_sheet (wb);
-	wb->auto_expr = expr;
-	wb->auto_expr_desc = desc;
-	sheet_update_auto_expr (sheet);
-}
-
 static void
 change_auto_expr (GtkWidget *item, Workbook *wb)
 {
@@ -2037,6 +2057,9 @@ workbook_init (GtkObject *object)
 	wb->summary_info   = summary_info_new ();
 	summary_info_default (wb->summary_info);
 
+	/* Nothing to undo or redo */
+	wb->undo_commands = wb->redo_commands = NULL;
+
 	/* Set the default operation to be performed over selections */
 	wb->auto_expr      = NULL;
 	wb->auto_expr_desc = NULL;
@@ -2138,13 +2161,24 @@ workbook_new (void)
 	wb = gtk_type_new (workbook_get_type ());
 	wb->toplevel  = gnome_app_new ("Gnumeric", "Gnumeric");
 	wb->table     = gtk_table_new (0, 0, 0);
+	wb->autosave = FALSE;
+	wb->autosave_prompt = TRUE;
+	wb->autosave_minutes = 15;
+
+	wb->autosave_timer = 
+	        gtk_timeout_add (wb->autosave_minutes*60000, 
+				 (GtkFunction) dialog_autosave_callback, wb);
+
+	wb->show_horizontal_scrollbar = TRUE;
+	wb->show_vertical_scrollbar = TRUE;
+	wb->show_notebook_tabs = TRUE;
 
 	gtk_window_set_policy (GTK_WINDOW (wb->toplevel), 1, 1, 0);
 	sx = MAX (gdk_screen_width  () - 64, 400);
 	sy = MAX (gdk_screen_height () - 64, 200);
 	sx = (sx * 3) / 4;
 	sy = (sy * 3) / 4;
-	gtk_window_set_default_size (GTK_WINDOW (wb->toplevel), sx, sy);
+	workbook_view_set_size (wb, sx, sy);
 
 	/* Assign a default name */
 	do {
@@ -2186,14 +2220,15 @@ workbook_new (void)
 	}
 #endif
 
-	/* Disable undo/redo for now */
-	workbook_view_set_undo_redo_state (wb, FALSE, FALSE);
+	/* There is nothing to undo/redo yet */
+	workbook_view_set_undo_redo_state (wb, NULL, NULL);
 
 	/*
-	 * Disable paste & paste special, they will be enabled when
-	 * there is something to paste
+	 * Enable paste special, this assumes that the default is to
+	 * paste from the X clipboard.  It will be disabled when
+	 * something is cut.
 	 */
-	workbook_view_set_paste_special_state (wb, FALSE);
+	workbook_view_set_paste_special_state (wb, TRUE);
 
  	workbook_create_toolbars (wb);
 
@@ -2355,12 +2390,16 @@ sheet_label_text_changed_signal (EditableLabel *el, const char *new_name, Workbo
 {
 	gboolean ans;
 
+	/* FIXME : Why do we care ?
+	 * Why are the tests here ?
+	 */
 	if (strchr (new_name, '"'))
 		return FALSE;
 	if (strchr (new_name, '\''))
 		return FALSE;
 
-	ans = workbook_rename_sheet (wb, el->text, new_name);
+	ans = cmd_rename_sheet (workbook_command_context_gui (wb),
+				wb, el->text, new_name);
 	workbook_focus_current_sheet (wb);
 
 	return ans;
@@ -2381,7 +2420,7 @@ sheet_action_add_sheet (GtkWidget *widget, Sheet *current_sheet)
  * Invoked when the user selects the option to remove a sheet
  */
 static void
-sheet_action_delete_sheet (GtkWidget *widget, Sheet *current_sheet)
+sheet_action_delete_sheet (GtkWidget *ignored, Sheet *current_sheet)
 {
 	GtkWidget *d;
 	Workbook *wb = current_sheet->workbook;
@@ -2406,7 +2445,7 @@ sheet_action_delete_sheet (GtkWidget *widget, Sheet *current_sheet)
 	g_free (message);
 	gnome_dialog_set_parent (GNOME_DIALOG (d), GTK_WINDOW (wb->toplevel));
 
-	r = gnome_dialog_run (GNOME_DIALOG (d));
+	r = gnumeric_dialog_run (wb, GNOME_DIALOG (d));
 
 	if (r != 0)
 		return;
@@ -2417,7 +2456,7 @@ sheet_action_delete_sheet (GtkWidget *widget, Sheet *current_sheet)
 	 * to infinity (and beyond)
 	 */
 	{
-		struct expr_relocate_info rinfo;
+		ExprRelocateInfo rinfo;
 		rinfo.origin.start.col = 0;
 		rinfo.origin.start.row = 0;
 		rinfo.origin.end.col = SHEET_MAX_COLS-1;
@@ -2426,9 +2465,13 @@ sheet_action_delete_sheet (GtkWidget *widget, Sheet *current_sheet)
 		rinfo.col_offset = SHEET_MAX_COLS-1;
 		rinfo.row_offset = SHEET_MAX_ROWS-1;
 
-		workbook_expr_relocate (wb, &rinfo);
+		workbook_expr_unrelocate_free (workbook_expr_relocate (wb, &rinfo));
 	}
 
+	/* Clear the cliboard to avoid dangling references to the deleted sheet */
+	if (current_sheet == application_clipboard_sheet_get ())
+		application_clipboard_clear ();
+
 	/*
 	 * All is fine, remove the sheet
 	 */
@@ -2437,6 +2480,26 @@ sheet_action_delete_sheet (GtkWidget *widget, Sheet *current_sheet)
 	workbook_recalc_all (wb);
 }
 
+/*
+ * sheet_action_rename_sheet:
+ * Invoked when the user selects the option to rename a sheet
+ */
+static void
+sheet_action_rename_sheet (GtkWidget *widget, Sheet *current_sheet)
+{
+	char *new_name;
+	Workbook *wb = current_sheet->workbook;
+	
+	new_name = dialog_get_sheet_name (wb, current_sheet->name);
+	if (!new_name)
+		return;
+
+	/* We do not care if it fails */
+	(void) cmd_rename_sheet (workbook_command_context_gui (wb),
+				 wb, current_sheet->name, new_name);
+	g_free (new_name);
+}
+
 #define SHEET_CONTEXT_TEST_SIZE 1
 
 struct {
@@ -2446,6 +2509,7 @@ struct {
 } sheet_label_context_actions [] = {
 	{ N_("Add another sheet"), sheet_action_add_sheet, 0 },
 	{ N_("Remove this sheet"), sheet_action_delete_sheet, SHEET_CONTEXT_TEST_SIZE },
+	{ N_("Rename this sheet"), sheet_action_rename_sheet, 0 },
 	{ NULL, NULL }
 };
 
@@ -2585,35 +2649,6 @@ workbook_attach_sheet (Workbook *wb, Sheet *sheet)
 		gtk_notebook_set_scrollable (GTK_NOTEBOOK (wb->notebook), TRUE);
 }
 
-/**
- * workbook_can_detach_sheet:
- * @wb: workbook.
- * @sheet: the sheet that we want to detach from the workbook
- *
- * Returns true whether the sheet can be safely detached from the
- * workbook.
- */
-gboolean
-workbook_can_detach_sheet (Workbook *wb, Sheet *sheet)
-{
-	GList *dependency_list;
-
-	g_return_val_if_fail (wb != NULL, FALSE);
-	g_return_val_if_fail (sheet != NULL, FALSE);
-	g_return_val_if_fail (IS_SHEET (sheet), FALSE);
-	g_return_val_if_fail (sheet->workbook != NULL, FALSE);
-	g_return_val_if_fail (sheet->workbook == wb, FALSE);
-	g_return_val_if_fail (workbook_sheet_lookup (wb, sheet->name) == sheet, FALSE);
-
-	dependency_list = region_get_dependencies (sheet, 0, 0, SHEET_MAX_COLS, SHEET_MAX_ROWS);
-	if (dependency_list == NULL)
-		return TRUE;
-
-	g_list_free (dependency_list);
-
-	return FALSE;
-}
-
 /**
  * workbook_detach_sheet:
  * @wb: workbook.
@@ -2637,14 +2672,6 @@ workbook_detach_sheet (Workbook *wb, Sheet *sheet, gboolean force)
 	notebook = GTK_NOTEBOOK (wb->notebook);
 	sheets = workbook_sheet_count (sheet->workbook);
 
-	if (!force) {
-		if (sheets == 1)
-			return FALSE;
-
-		if (!workbook_can_detach_sheet (wb, sheet))
-			return FALSE;
-	}
-
 	/*
 	 * Remove our reference to this sheet
 	 */
@@ -2767,27 +2794,6 @@ workbook_realized (Workbook *workbook, GdkWindow *window)
 {
 }
 
-/**
- * workbook_set_title:
- * @wb: the workbook to modify
- * @title: the title for the toplevel window
- *
- * Sets the toplelve window title of @wb to be @title
- */
-void
-workbook_set_title (Workbook *wb, const char *title)
-{
-	char *full_title;
-
-	g_return_if_fail (wb != NULL);
-	g_return_if_fail (title != NULL);
-
-	full_title = g_strconcat (_("Gnumeric: "), title, NULL);
-
- 	gtk_window_set_title (GTK_WINDOW (wb->toplevel), full_title);
-	g_free (full_title);
-}
-
 /**
  * workbook_set_filename:
  * @wb: the workbook to modify
@@ -2812,7 +2818,7 @@ workbook_set_filename (Workbook *wb, const char *name)
 
 	wb->filename = g_strdup (name);
 
-	workbook_set_title (wb, g_basename (name));
+	workbook_view_set_title (wb, g_basename (name));
 
 	wb->needs_name = FALSE;
 	return TRUE;
@@ -2831,42 +2837,105 @@ workbook_foreach (WorkbookCallback cback, gpointer data)
 	}
 }
 
+struct expr_relocate_storage
+{
+	EvalPosition pos;
+	ExprTree *oldtree;
+};
+
 /**
- * workbook_expr_relocate:
- * @wb: the workbook to modify
- * @sheet: the sheet containing the column/row that was moved
- * @col: starting column that was moved.
- * @row: starting row that was moved.
- * @coldelta: signed column distance that cells were moved.
- * @rowcount: signed row distance that cells were moved.
- *
- * Fixes references after a column or row move.
+ * workbook_expr_unrelocate_free : Release the storage associated with
+ *    the list.
  */
+void
+workbook_expr_unrelocate_free (GSList *info)
+{
+	while (info != NULL) {
+		GSList *cur = info;
+		struct expr_relocate_storage *tmp =
+		    (struct expr_relocate_storage *)(info->data);
+
+		info = info->next;
+		expr_tree_unref (tmp->oldtree);
+		g_free (tmp);
+		g_slist_free_1 (cur);
+	}
+}
 
 void
-workbook_expr_relocate (Workbook *wb, struct expr_relocate_info const *info)
+workbook_expr_unrelocate (Workbook *wb, GSList *info)
+{
+	while (info != NULL) {
+		GSList *cur = info;
+		struct expr_relocate_storage *tmp =
+		    (struct expr_relocate_storage *)info->data;
+		Cell *cell = sheet_cell_get (tmp->pos.sheet,
+					     tmp->pos.eval.col,
+					     tmp->pos.eval.row);
+
+		g_return_if_fail (cell != NULL);
+
+		cell_set_formula_tree (cell, tmp->oldtree);
+		expr_tree_unref (tmp->oldtree);
+
+		info = info->next;
+		g_free (tmp);
+		g_slist_free_1 (cur);
+	}
+}
+
+/**
+ * workbook_expr_relocate:
+ * Fixes references to or from a region that is going to be moved.
+ *
+ * @wb: the workbook to modify
+ * @info : the descriptor record for what is being moved where.
+ *
+ * Returns a list of the locations and expressions that were changed
+ * outside of the region.
+ */
+GSList *
+workbook_expr_relocate (Workbook *wb, ExprRelocateInfo const *info)
 {
 	GList *cells, *l;
-	EvalPosition pos;
+	GSList *undo_info = NULL;
 
-	g_return_if_fail (wb != NULL);
+	if (info->col_offset == 0 && info->row_offset == 0 &&
+	    info->origin_sheet == info->target_sheet)
+		return NULL;
 
-	if (info->col_offset == 0 && info->row_offset == 0)
-		return;
+	g_return_val_if_fail (wb != NULL, NULL);
 
 	/* Copy the list since it will change underneath us.  */
 	cells = g_list_copy (wb->formula_cell_list);
 
 	for (l = cells; l; l = l->next)	{
 		Cell *cell = l->data;
+		EvalPosition pos;
 		ExprTree *newtree = expr_relocate (cell->parsed_node,
 						   eval_pos_cell (&pos, cell),
 						   info);
-		if (newtree)
+
+		if (newtree) {
+			/* Don't store relocations if they were inside the region
+			 * being moved.  That is handled elsewhere */
+			if (info->origin_sheet != pos.sheet ||
+			    !range_contains (&info->origin, pos.eval.col, pos.eval.row)) {
+				struct expr_relocate_storage *tmp =
+				    g_new (struct expr_relocate_storage, 1);
+				tmp->pos = pos;
+				tmp->oldtree = cell->parsed_node;
+				expr_tree_ref (tmp->oldtree);
+				undo_info = g_slist_prepend (undo_info, tmp);
+			}
+
 			cell_set_formula_tree (cell, newtree);
+		}
 	}
 
 	g_list_free (cells);
+
+	return undo_info;
 }
 
 GList *
