@@ -76,6 +76,10 @@ struct _Workbook {
 	char       *filename;
 	gboolean    needs_name;
 
+	/* Undo support */
+	GSList	   *undo_commands;
+	GSList	   *redo_commands;
+
 	/* The auto-expression */
 	ExprTree   *auto_expr;
 	String     *auto_expr_desc;
@@ -91,7 +95,7 @@ struct _Workbook {
 	GList      *formula_cell_list;
 
 	/* A queue with the cells to be evaluated */
-	GList      *eval_queue;
+	GList     *eval_queue;
 	int        max_iterations;
 
 	guint8     generation;
@@ -107,11 +111,19 @@ struct _Workbook {
 	 * to the asyncronous paste callback
 	 */
 	void       *clipboard_paste_callback_data;
-	gboolean    have_x_selection;
 
 	void       *corba_server;
 
 	WorkbookPrivate *priv;
+
+	/* Workbook level preferences */
+        gboolean   autosave;
+        gboolean   autosave_prompt;
+        gint       autosave_minutes;
+        gint       autosave_timer;
+	gboolean   show_horizontal_scrollbar;
+	gboolean   show_vertical_scrollbar;
+	gboolean   show_notebook_tabs;
 };
 
 typedef struct {
@@ -128,7 +140,6 @@ Workbook   *workbook_core_new            (void);
 Workbook   *workbook_new_with_sheets     (int sheet_count);
 
 gboolean    workbook_set_filename        (Workbook *, const char *);
-void        workbook_set_title           (Workbook *, const char *);
 Workbook   *workbook_try_read            (const char *filename, char **error_msg);
 Workbook   *workbook_read                (const char *filename);
 
@@ -152,13 +163,14 @@ gboolean    workbook_rename_sheet        (Workbook *wb,
 					  const char *old_name,
 					  const char *new_name);
 int         workbook_sheet_count         (Workbook *wb);
-gboolean    workbook_can_detach_sheet    (Workbook *wb, Sheet *sheet);
 GList      *workbook_sheets              (Workbook *wb);
 char       *workbook_selection_to_string (Workbook *wb, Sheet *base_sheet);
 
-struct expr_relocate_info;
-void        workbook_expr_relocate       (Workbook *wb,
-					  struct expr_relocate_info const *info);
+GSList     *workbook_expr_relocate       (Workbook *wb,
+					  ExprRelocateInfo const *info);
+void        workbook_expr_unrelocate     (Workbook *wb, GSList *info);
+void        workbook_expr_unrelocate_free(GSList *info);
+
 /*
  * Does any pending recalculations
  */
