@@ -5,7 +5,7 @@
  *   Daniel Veillard <Daniel.Veillard@w3.org>
  *   Miguel de Icaza <miguel@gnu.org>
  *
- * $Id: xml-io.c,v 1.94 2000/01/02 02:00:40 unammx Exp $
+ * $Id: xml-io.c,v 1.100 2000/01/18 21:01:16 jody Exp $
  */
 
 #include <config.h>
@@ -16,6 +16,7 @@
 #include "gnumeric.h"
 #include "gnome-xml/tree.h"
 #include "gnome-xml/parser.h"
+#include "gnome-xml/parserInternals.h"
 #include "gnome-xml/xmlmemory.h"
 #include "color.h"
 #include "border.h"
@@ -631,8 +632,7 @@ xml_write_style_border (parse_xml_context_t *ctxt,
 	for (i = MSTYLE_BORDER_TOP; i <= MSTYLE_BORDER_REV_DIAGONAL; i++) {
 		MStyleBorder const *border;
 		if (mstyle_is_element_set (style, i) &&
-		    NULL != (border = mstyle_get_border (style, i)) &&
-		    border->line_type != STYLE_BORDER_NONE) {
+		    NULL != (border = mstyle_get_border (style, i))) {
 			break;
 		}
 	}
@@ -644,8 +644,7 @@ xml_write_style_border (parse_xml_context_t *ctxt,
 	for (i = MSTYLE_BORDER_TOP; i <= MSTYLE_BORDER_REV_DIAGONAL; i++) {
 		MStyleBorder const *border;
 		if (mstyle_is_element_set (style, i) &&
-		    NULL != (border = mstyle_get_border (style, i)) &&
-		    border->line_type != STYLE_BORDER_NONE) {
+		    NULL != (border = mstyle_get_border (style, i))) {
 			StyleBorderType t = border->line_type;
 			StyleColor *col   = border->color;
  			side = xmlNewChild (cur, ctxt->ns,
@@ -828,7 +827,7 @@ xml_read_names (parse_xml_context_t *ctxt, xmlNodePtr tree, Workbook *wb,
 					if (!expr_name_create (wb, sheet, name, txt, &error))
 						g_warning (error);
 
-					g_free (txt);
+					xmlFree (txt);
 				}
 				bits = bits->next;
 			}
@@ -920,7 +919,7 @@ xml_read_summary (parse_xml_context_t *ctxt, xmlNodePtr tree, SummaryInfo *summa
 
 						if (sit)
 							summary_info_add (summary_info, sit);
-						g_free (txt);
+						xmlFree (txt);
 					}
 				}
 				bits = bits->next;
@@ -1237,7 +1236,8 @@ xml_read_style (parse_xml_context_t *ctxt, xmlNodePtr tree)
 			double units = 14;
 			int t;
 
-			xml_get_value_double (child, "Unit", &units);
+			if (xml_get_value_double (child, "Unit", &units))
+				mstyle_set_font_size (mstyle, units);
 
 			if (xml_get_value_int (child, "Bold", &t))
 				mstyle_set_font_bold (mstyle, t);
@@ -1346,6 +1346,7 @@ xml_write_colrow_info (Sheet *sheet, ColRowInfo *info, void *user_data)
 		xml_set_value_double (cur, "MarginA", info->margin_a_pt);
 		xml_set_value_double (cur, "MarginB", info->margin_b);
 		xml_set_value_int (cur, "HardSize", info->hard_size);
+		xml_set_value_int (cur, "Hidden", info->pixels < 0);
 
 		xmlAddChild (closure->container, cur);
 	}
@@ -1387,6 +1388,8 @@ xml_read_colrow_info (parse_xml_context_t *ctxt, xmlNodePtr tree, ColRowInfo *re
 	xml_get_value_double (tree, "MarginB", &ret->margin_b_pt);
 	if (xml_get_value_int (tree, "HardSize", &val))
 		ret->hard_size = val;
+	if (xml_get_value_int (tree, "Hidden", &val) && val)
+		ret->pixels = -1;	/* Any negative will result in things being hidden */
 
 	return ret;
 }
@@ -1699,6 +1702,7 @@ static xmlNodePtr
 xml_sheet_write (parse_xml_context_t *ctxt, Sheet *sheet)
 {
 	xmlNodePtr cur;
+	xmlNsPtr gmr;
 	xmlNodePtr child;
 	xmlNodePtr rows;
 	xmlNodePtr cols;
@@ -1717,6 +1721,11 @@ xml_sheet_write (parse_xml_context_t *ctxt, Sheet *sheet)
 	cur = xmlNewDocNode (ctxt->doc, ctxt->ns, "Sheet", NULL);
 	if (cur == NULL)
 		return NULL;
+	if (ctxt->ns == NULL) {
+	    gmr = xmlNewNs (cur, "http://www.gnome.org/gnumeric/", "gmr");
+	    xmlSetNs(cur, gmr);
+	    ctxt->ns = gmr;
+	}
 	tstr = xmlEncodeEntitiesReentrant (ctxt->doc, sheet->name);
 	xmlNewChild (cur, ctxt->ns, "Name",  tstr);
 	if (tstr) xmlFree (tstr);
@@ -1838,7 +1847,7 @@ xml_read_cols_info (parse_xml_context_t *ctxt, Sheet *sheet, xmlNodePtr tree)
 		if (!info)
 			continue;
 		sheet_col_add (sheet, info);
-		sheet_col_set_width_units (ctxt->sheet, info->pos, units);
+		sheet_col_set_width_units (ctxt->sheet, info->pos, units, info->hard_size);
 	}
 }
 
@@ -1982,6 +1991,7 @@ static xmlNodePtr
 xml_workbook_write (parse_xml_context_t *ctxt, Workbook *wb)
 {
 	xmlNodePtr cur;
+	xmlNsPtr gmr;
 	xmlNodePtr child;
 	GList *sheets, *sheets0;
 	char *oldlocale;
@@ -1992,6 +2002,11 @@ xml_workbook_write (parse_xml_context_t *ctxt, Workbook *wb)
 	cur = xmlNewDocNode (ctxt->doc, ctxt->ns, "Workbook", NULL);	/* the Workbook name !!! */
 	if (cur == NULL)
 		return NULL;
+	if (ctxt->ns == NULL) {
+	    gmr = xmlNewNs (cur, "http://www.gnome.org/gnumeric/v2", "gmr");
+	    xmlSetNs(cur, gmr);
+	    ctxt->ns = gmr;
+	}
 
 	oldlocale = g_strdup (setlocale (LC_NUMERIC, NULL));
 	setlocale (LC_NUMERIC, "C");
@@ -2172,17 +2187,36 @@ xml_workbook_read (Workbook *wb, parse_xml_context_t *ctxt, xmlNodePtr tree)
 static gboolean
 xml_probe (const char *filename)
 {
-	xmlDocPtr res;
+        int ret;
+	xmlDocPtr res = NULL;
 	xmlNsPtr gmr;
+	xmlParserCtxtPtr ctxt;
+	xmlSAXHandler silent, *old;
 
-	res = xmlParseFile (filename);
-	if (res == NULL) {
-		/* FIXME: make probing silent.  */
-		fprintf (stderr,
-			 "File `%s' does not look like an xml-document\n"
-			 "Please ignore complaints about `Extra content at the end of the document'\n",
-			 filename);
-		return FALSE;
+	/*
+	 * Do a silent call to the XML parser.
+	 */
+	ctxt = xmlCreateFileParserCtxt(filename);
+	memcpy(&silent, ctxt->sax, sizeof(silent));
+	old = ctxt->sax;
+	silent.error = NULL;
+	silent.warning = NULL;
+	silent.fatalError = NULL;
+	ctxt->sax = &silent;
+
+	xmlParseDocument(ctxt);
+
+	ret = ctxt->wellFormed;
+	res = ctxt->myDoc;
+	ctxt->sax = old;
+	xmlFreeParserCtxt(ctxt);
+
+	/*
+	 * This is not well formed.
+	 */
+	if (!ret) {
+		xmlFreeDoc(res);
+	        return FALSE;
 	}
 
 	if (res->root == NULL) {
@@ -2257,7 +2291,6 @@ gnumeric_xml_write_sheet (Sheet *sheet, const char *filename)
 {
 	parse_xml_context_t ctxt;
 	xmlDocPtr xml;
-	xmlNsPtr gmr;
 	int ret;
 
 	g_return_val_if_fail (sheet != NULL, -1);
@@ -2271,9 +2304,8 @@ gnumeric_xml_write_sheet (Sheet *sheet, const char *filename)
 	if (xml == NULL){
 		return -1;
 	}
-	gmr = xmlNewGlobalNs (xml, "http://www.gnome.org/gnumeric/", "gmr");
 	ctxt.doc = xml;
-	ctxt.ns = gmr;
+	ctxt.ns = NULL;
 
 	xml->root = xml_sheet_write (&ctxt, sheet);
 
@@ -2343,7 +2375,6 @@ int
 gnumeric_xml_write_workbook (Workbook *wb, const char *filename)
 {
 	xmlDocPtr xml;
-	xmlNsPtr gmr;
 	parse_xml_context_t ctxt;
 	int ret;
 
@@ -2357,9 +2388,8 @@ gnumeric_xml_write_workbook (Workbook *wb, const char *filename)
 	if (xml == NULL){
 		return -1;
 	}
-	gmr = xmlNewGlobalNs (xml, "http://www.gnome.org/gnumeric/v2", "gmr");
 	ctxt.doc = xml;
-	ctxt.ns = gmr;
+	ctxt.ns = NULL;
 
 	xml->root = xml_workbook_write (&ctxt, wb);
 
